/* 
Morley and Singh (2015)
Bayesian estimation

Note that priors for proposal are different than in text, but priors for posterior simulation are same as in June 2015 version of the paper.

Set lambda_k_eps=0

kappa_t = mu_kappa + kappa_t-1 + omega_t (aka v_t) + lambda_k_eta*eta_t + lambda_k_eps*eps_t

lambda_y_eta = 1 + lambda_i_eta + lambda_s_eta + lambda_k_eta
lambda_y_eps = 1 + lambda_i_eps + lambda_k_eps
lambda_y_omega = 1 + lambda_i_omega

Note: bounds for proposal estimation assume lambda_k's are zero, but they are allowed for in MH estimation


UC model of Sales and Inventory with implicit UC process for output
Allows for correlation between permanent and transitory movements in sales
*/


new;
format /m1 /rd 9,5;
library optmum, pgraph;

@screen off;@

overwrite_proposal=0; @1=estimate new proposal, 0=use previously estimated proposal@

@SELECT DATA HERE@
data_code=3; @3=aggregate; 4=durables@


if data_code==3;
load data[217,4]=s_di_y_i_14.txt; @ sales, change in inventories, real GDP, and stock of inventories 1960:Q1 to 2014:Q1@
elseif data_code==4;
load data[270,3]=s_di_y_dg_14.txt; @ sales, change in inventories, and real GDP 1947:Q1 to 2014:Q2@ 
endif;
 

 
t=rows(data);



@CONSTRUCT LN(SALES) AND APPOX. OF LN(INV) HERE@

lsales=ln(data[.,1]);
loutput=ln(data[.,3]);
@linv_stock=ln(data[.,4]);@  @uncomment this line to consider log of actual stock@

@Constructing the level of inventories@
@Based on y = s + Di approximation, where lower case y and s denotes natural logs@
ldinv=loutput-lsales;

@Based on log-linear approximation, where lower case y and s denotes natural logs@
rho_=1/(1+meanc(data[.,2]./data[.,1]));
kappa_=-ln(rho_)-(1-rho_)*ln((1-rho_)/rho_);
ldinv_ts=(1/(1-rho_))*(loutput-rho_*lsales-kappa_);






linv=lsales[1]; @this is arbitrary@
j=2;
do until j>t;
	linv=linv|linv[j-1]+ldinv[j];
j=j+1;
endo;

  
y=lsales~linv~loutput;             @change the second series between linv1 and linv2 to select different inventory approaches@
@y=lsales~linv_stock~loutput;@             @this line changed from previous drafts to consider log of actual stock@

y=100*y;

@OTHER METHOD@
dinv=data[.,3]-data[.,1];
dinv_n=dinv[2:rows(dinv)]./data[1:rows(dinv)-1,3];
ldinv_n=ldinv[2:rows(ldinv)];





@PRINT DATA TO FILE FOR USE IN EVIEWS@
output file=output_mh/data.out reset;  
output on; y; output off;
cls;



@SET UP SAMPLE PERIOD for UPDATED DATA@
start_date=1;  @1=1960:Q1@
break_date=97; @97=1984Q1, meaning new parameters in 1984Q2@


if data_code==4;
@SET UP SAMPLE PERIOD FOR UPDATED DURABLES@
start_date=53;  @1=1947:Q1, 21=1952:Q1, 53=1960:Q1@
break_date=149; @149=1984Q1, meaning new parameters in 1984Q2@
endif;




beta_prior=0; @beta_prior=0 means truncated normal for lambdas, beta_prior=1 means beta priors for rescaled lambdas@
var_prior=0; @0=Inverse Gamma, 1=Gamma prior for variances@


@=================  PRIOR PARAMETERS ====================@
@ Gamma parameters for 1/sig^2; Note that this is  a naturally conjugate prior@
@v0 is 2*alpha and d0 is 2*beta@
if data_code==3;
v0_ = @5@@4@0.02;    @5 is good for total@
d0_ = @1@@2@0.02;    @1 is good for total@
elseif data_code==4;
v0_ = @10@0.02;    @10 is good for total@
d0_ = @2@0.02;    @2 is good for total@
endif;

@Normal Prior for phi_s1, phi_s2, phi_i1, phi_i2, gamma_itau@ 
b0_m = zeros(5,1);
b0_m[1:4]=0*ones(4,1);
b0_v=(0.25^2)*eye(5);
b0_v[5,5]=10^2;

@Normal Prior for lambda_s_eta,lambda_y_eta,lambda_y_eps,lambda_i_v@ 
b0_ml = zeros(5,1);
b0_vl=(0.5^2)*eye(5);
b0_ml[1]=-0.5;
b0_ml[2]=0;
b0_ml[3]=0.5;
b0_ml[4]=-0.5;
b0_ml[5]=0; 


 
@Beta prior for rescaled lambdas@
U1_01_=2;  U1_00_= 2;            
@NOTE: -1<lambda_s_eta<0, -2<lambda_y_eta<1, 0<lamdbda_y_eps<1, -1<lambda_i_v<0@ 
 







@POSTERIOR MODE ESTIMATION@
@PROVIDES MEAN AND VARIANCE FOR PROPOSAL DISTRIBUTION@

START=2;	@evaluate likelihood in second quarter of data, as in MNZ@
prior=10000;	@diffuse prior on initial value of trends@


@Method 1 of initializing parameters@
@Try a number of starting values to find global max@
xout_mat={};
prm_mat={};
j=1;
do until j>1;

PRMTR_IN=0.15*rndn(1,28);       @change the number of parameters when model is restricted@
PRMTR_IN=PRMTR_IN';

PRMTR_IN=PRMTR_IN+0.05*rndn(rows(prmtr_in),1);

 _opmiter =500;  @ This controls the maximum number of iterations @
 {xout,fout,gout,cout}=@qnewton@optmum(&post_fcn,PRMTR_IN);

xout_mat=xout_mat|xout'~-fout;
prm_mat=prm_mat|prmtr_in'~-fout;

j=j+1;
endo;

xout_mat=rev(sortc(xout_mat,cols(xout_mat)));      @sort to find highest likelihood@
prm_mat=rev(sortc(prm_mat,cols(prm_mat)));       @sort to find highest likelihood@


prmtr_in=xout_mat[1,1:cols(prm_mat)-1]';      @use final estimates at highest likelihood as starting values for one more optimization@



 
                     @Initial values of the parameters @

 _opmiter = 500;  @ This controls the maximum number of iterations @
 {xout,fout,gout,cout}=@qnewton@optmum(&post_fcn,PRMTR_IN);
                     @ prmtr estimates, -log lik value, Grandient, code@


xout_pre=xout[1:14];
xout_post=xout_pre;
xout_post[1:14]=xout[15:28];  
    


prm_fnl=trans(xout_pre)|trans(xout_post);

prm_fnl_=prm_fnl; @store for later. construction of counterfactuals, etc..., based on posterior mode@

prm_fnl_all=prm_fnl;    @pre and post parametrs@
prm_fnl_all[5:8 19:22]=sqrt(prm_fnl_all[5:8 19:22]);

trap 1;
 "Calculating Hessian..... Please be patient!!!!";
                  hessn0=hessp(&post_fcn,xout);
                  cov0=inv(hessn0);
grdn_fnl=@gradfd@gradp(&TRANS_fnl,xout);
cov_h=grdn_fnl*cov0*grdn_fnl';

if det(cov_h)>0;

sd_fnl_h1 =sqrt(diag(cov_h)); @Standard errors of the estimated coefficients@
cov_h=diag(cov_h).*eye(rows(cov_h));

else;

sd_fnl_h1=0*ones(rows(prm_fnl),1);

endif;

trap 0;

if overwrite_proposal==1;
if data_code==3 or data_code==4;
output file=output_mh/good_proposal_mean_total.out reset;
output on;
prm_fnl;
output off;
output file=output_mh/good_proposal_var_total.out reset;
output on;
vec(cov_h);
output off;
endif;
elseif overwrite_proposal==0;
if data_code==3 or data_code==4;
load prm_fnl[28,1]=output_mh/good_proposal_mean_total.out;
load cov_h[28^2,1]=output_mh/good_proposal_var_total.out;
cov_h=reshape(cov_h,28,28);
endif;
prm_fnl_=prm_fnl; @store for later. construction of counterfactuals, etc..., based on posterior mode@
prm_fnl_all=prm_fnl;    @pre and post parametrs@
prm_fnl_all[5:8 19:22]=sqrt(prm_fnl_all[5:8 19:22]);
sd_fnl_h1 =sqrt(diag(cov_h)); @Standard errors of the estimated coefficients@
endif;

screen on;

output file=output_mh/uc_si.out reset;  
output on;
"overwrite_proposal=";;overwrite_proposal;
"(if 0, ignore starting values and likelihood value)";
"Starting values:"; prmtr_in';
output off;

output on;
"==FINAL OUTPUT================================================";
"likelihood value is ";; -fout;
"code";;cout;
" ";
"Estimated parameters are:";
"";
"Pre_Break";
prm_fnl_all[1:14]~sd_fnl_h1[1:14];
"Post_Break";
prm_fnl_all[15:rows(prm_fnl)]~sd_fnl_h1[15:rows(prm_fnl)];
"==============================================================";
"Pre-transformed estimates are:";
"(";;XOUT';;")";

output off;




@Normal Prior for phi_s1, phi_s2, phi_i1, phi_i2, gamma_itau@ 
b0_m = zeros(5,1);
b0_m[1:4]=0*ones(4,1);
b0_v=(1^2)*eye(5);
b0_v[5,5]=10^2;



@Normal Prior for lambda_s_eta,lambda_y_eta,lambda_y_eps,lambda_i_v@ 
b0_ml = zeros(5,1);
b0_vl=(1^2)*eye(5);
b0_ml[1]=-0.5;
b0_ml[2]=0;
b0_ml[3]=0.5;
b0_ml[4]=-0.5;
b0_ml[5]=0;


@POSTERIOR SIMULATION@

@for b_mat place first parameter in block in column 1 and last parameter in block in column 2@
b_mat=
1~2|
3~4|
5~6|
7~8|
9~9|
10~11|
12~14|
15~16|
17~18|
19~20|
21~22|
23~24|
25~26|
27~28;

blocks=rows(b_mat);


k=0.1*ones(blocks,1);
k[1]=0.2;
k[3]=0.5;
k[4]=0.5;
k[6]=0.2;
k[9]=0.5;




t_nu=rows(y)-3; @degrees of freedom for t-distribution used in proposal density@
a_=1;   @0=IA, 1=RW@


prm_p=prm_fnl; @set proposal mean to posterior mode@
if det(cov_h)>0 and minc(diag(cov_h))>0; 
cov_p=cov_h; @set proposal variance to inverse hessian at mode@
else;
cov_p=0.1*eye(rows(prm_p));
endif;





@===== Number of Simulations ===========@
N0 = 20000;       @NUMBER OF DRAWS TO LEAVE OUT@  @20,000 for Oct10 tables@
MM = 500000;       @NUMBER OF DRAWS AFTER BURN-IN@ @500,000 for Oct10 tables@
CAPN = N0 + MM;  @TOTAL NUMBER OF DRAWS@






@========INITIAL VALUES for THE MH ALGORITHM ===========@

thetatt=prm_p;



@===== STORAGE SPACES==========================@
thetamm={};
cttmm={};
accept=zeros(blocks,1);
counter_p=0;  @for rejection sampling from proposal distribution@

sig_dy_premm={};
sig_dy_postmm={};
dsig_dy_postmm={};
sig_ds_premm={};
sig_ds_postmm={};
dsig_ds_postmm={};
sig_ddi_premm={};
sig_ddi_postmm={};
sig_di_premm={};
sig_di_postmm={};
sig_di_u_premm={};
sig_di_u_postmm={};
sig_di_p_premm={};
sig_di_p_postmm={};
sig_dy_cf5mm={};
sig_ds_cf5mm={};
sig_dy_cf6mm={};
sig_ds_cf6mm={};
sig_dy_cf7mm={};
sig_ds_cf7mm={};
sig_dy_cf8mm={};
sig_ds_cf8mm={};
sig_dy_cf9mm={};
sig_ds_cf9mm={};

dsig_dy_cf5mm={};
dsig_ds_cf5mm={};
dsig_dy_cf6mm={};
dsig_ds_cf6mm={};
dsig_dy_cf7mm={};
dsig_ds_cf7mm={};
dsig_dy_cf8mm={};
dsig_ds_cf8mm={};
dsig_dy_cf9mm={};
dsig_ds_cf9mm={};

ratio_eta_Di_u_premm={};
ratio_v_Di_u_premm={};
ratio_eps_Di_u_premm={};
ratio_u_Di_u_premm={};
alpha_y_eta_premm={};
alpha_y_v_premm={};
alpha_y_eps_premm={};
alpha_y_u_premm={};
alpha_y_premm={};
alpha_s_eta_premm={};
alpha_s_v_premm={};
alpha_s_eps_premm={};
alpha_s_u_premm={};
alpha_s_premm={};

ratio_eta_Di_u_postmm={};
ratio_v_Di_u_postmm={};
ratio_eps_Di_u_postmm={};
ratio_u_Di_u_postmm={};
alpha_y_eta_postmm={};
alpha_y_v_postmm={};
alpha_y_eps_postmm={};
alpha_y_u_postmm={};
alpha_y_postmm={};
alpha_s_eta_postmm={};
alpha_s_v_postmm={};
alpha_s_eps_postmm={};
alpha_s_u_postmm={};
alpha_s_postmm={};


dsig_etamm={};
dsig_epsmm={};
dphi_smm={};
dlambda_s_etamm={};
dsig_vmm={};
dsig_umm={};
dphi_imm={};
dmu_kmm={};
dlambda_y_etamm={};
dlambda_y_epsmm={};
dlambda_i_vmm={};
dalpha_y_umm={};
dalpha_s_umm={};


corr_ddids_premm={};
corr_ddids_postmm={};
corr_diuds_premm={};
corr_diuds_postmm={};

dcorr_ddidsmm={};
dcorr_diudsmm={};



ITR=1;
do while ITR le CAPN;

@=================START SAMPLING================@

 
bbb=1;
do until bbb>blocks;
           
bs=b_mat[bbb,1];
be=b_mat[bbb,2];

 
{thetatt_sub,fthetatt,qthetatt} = GEN_Theta_mb; 
 	
thetatt[bs:be]=thetatt_sub;

bbb=bbb+1;
endo;





if ITR gt N0;


    
    
@CALCULATE VARIANCES OF UNOBSERVED COMPONENTS, ETC...@
prm_fnl_all=thetatt;    @pre and post parametrs@
prm_fnl_all[5:8 19:22]=sqrt(prm_fnl_all[5:8 19:22]);

    thetamm=thetamm~prm_fnl_all;


prm_fnl_pre=prm_fnl_all[1:14];    @parameters before the break@
prm_fnl_post=prm_fnl_pre;  @start with the same parameters. Then alter for parameters that undergo a break@
prm_fnl_post[1:14]=prm_fnl_all[15:28];  
                          

prm_fnl=prm_fnl_pre;

phi_s1_fnl=prm_fnl[1]+prm_fnl[2];
phi_s2_fnl=-prm_fnl[2];
phi_i1_fnl=prm_fnl[3]+prm_fnl[4];
phi_i2_fnl=-prm_fnl[4];
sig_eta_fnl=prm_fnl[5];
sig_eps_fnl=prm_fnl[6];
sig_v_fnl=prm_fnl[7];
sig_u_fnl=prm_fnl[8];
gamma_itao_fnl=1;
lambda_s_eta_fnl=prm_fnl[10];          
lambda_y_eta_fnl=prm_fnl[11];
lambda_y_eps_fnl=prm_fnl[12];         
lambda_i_v_fnl=  prm_fnl[13];  
lambda_k_eta_fnl=  prm_fnl[14];  
lambda_k_eps_fnl=  0;
lambda_i_eta_fnl=lambda_y_eta_fnl-lambda_s_eta_fnl - gamma_itao_fnl-lambda_k_eta_fnl;
lambda_i_eps_fnl=lambda_y_eps_fnl-1-lambda_k_eps_fnl;
  


sig_di_u=sqrt(((lambda_y_eta_fnl-lambda_s_eta_fnl)^2)*sig_eta_fnl^2+((lambda_y_eps_fnl-1)^2)*sig_eps_fnl^2
+((1+lambda_i_v_fnl)^2)*sig_v_fnl^2+sig_u_fnl^2);      @standard deviation of unplanned/unexpected inventories Feb 2009@




@Calculate standard deviation of change in output and of change in inventories@
@phi(L)*z_t = x_t - x_t-1@
@x_t = lambda_ieta*eta + lambda_iv*v + lambda_ieps*eps + u@
@z_t = phi(L)^-1*x_t - phi(L)^-1*x_t-1@
@see notes from Nov. 26, 2008@
@Beta_t = F * Beta_t-1 + w_t@
@where Beta_t = z_st, z_st-1, z_it, z_it-1, x_st, x_it@
@w_t = x_st, 0, x_it, 0, x_st, x_it@
@updated for Aug 2014 document@



f_= phi_s1_fnl~phi_s2_fnl~0~0~-1~0|
    1~0~0~0~0~0|
    0~0~phi_i1_fnl~phi_i2_fnl~0~-1|
    0~0~1~0~0~0|
    0~0~0~0~0~0|
    0~0~0~0~0~0;
q_= zeros(rows(f_),rows(f_));
q_[1,1]=lambda_s_eta_fnl^2*sig_eta_fnl^2 + sig_eps_fnl^2;
q_[5,5]=q_[1,1];
q_[1,5]=q_[1,1];
q_[5,1]=q_[1,1];
q_[3,3]=lambda_i_eta_fnl^2*sig_eta_fnl^2+lambda_i_v_fnl^2*sig_v_fnl^2+lambda_i_eps_fnl^2*sig_eps_fnl^2+sig_u_fnl^2;
q_[6,6]=q_[3,3];
q_[3,6]=q_[3,3];
q_[6,3]=q_[3,3];
q_[1,3]=lambda_s_eta_fnl*lambda_i_eta_fnl*sig_eta_fnl^2 + lambda_i_eps_fnl*sig_eps_fnl^2;
q_[3,1]=q_[1,3];
q_[1,6]=q_[1,3];
q_[6,1]=q_[1,3];
q_[3,5]=q_[1,3];
q_[5,3]=q_[1,3];
q_[5,6]=q_[1,3];
q_[6,5]=q_[1,3];

var_beta=reshape(inv(eye(6^2)-f_.*.f_)*vec(q_),6,6);


sig_di=sqrt((gamma_itao_fnl+lambda_k_eta_fnl)^2*sig_eta_fnl^2+lambda_k_eps_fnl^2*sig_eps_fnl^2+sig_v_fnl^2+var_beta[3,3]+
            2*(gamma_itao_fnl+lambda_k_eta_fnl)*lambda_i_eta_fnl*sig_eta_fnl^2+2*lambda_k_eps_fnl*lambda_i_eps_fnl*sig_eps_fnl^2+2*lambda_i_v_fnl*sig_v_fnl^2);

sig_ds = sqrt( sig_eta_fnl^2 + var_beta[1,1] + 2*lambda_s_eta_fnl*sig_eta_fnl^2);

cov_il=(phi_i1_fnl-1)*((gamma_itao_fnl+lambda_k_eta_fnl)*lambda_i_eta_fnl*sig_eta_fnl^2 + lambda_i_v_fnl*sig_v_fnl^2+lambda_i_eps_fnl*lambda_k_eps_fnl*sig_eps_fnl^2)
        +var_beta[3,4];

cov_is=sig_eta_fnl^2*(gamma_itao_fnl + lambda_k_eta_fnl +lambda_i_eta_fnl + (gamma_itao_fnl+lambda_k_eta_fnl)*lambda_s_eta_fnl) + lambda_k_eps_fnl*sig_eps_fnl^2
        +var_beta[1,3];

cov_ils=(phi_s1_fnl-1)*((gamma_itao_fnl+lambda_k_eta_fnl)*lambda_s_eta_fnl*sig_eta_fnl^2+lambda_k_eps_fnl*sig_eps_fnl^2) 
        +var_beta[1,4];

@see updated version of Nov. 26, 2008 document for following@
sig_dy = sqrt( 2*sig_di^2 +sig_ds^2 
                -2*cov_il + 2*cov_is - 2*cov_ils ); 

sig_ddi = sqrt( 2*sig_di^2 - 2*cov_il);

@see notes from Nov. 3, 2014 for the following@

cov_ddids=(1+lambda_k_eta_fnl+lambda_i_eta_fnl+lambda_s_eta_fnl*(1+lambda_k_eta_fnl)-(phi_s1_fnl -1)*lambda_s_eta_fnl*(1+lambda_k_eta_fnl))*sig_eta_fnl^2+
                                var_beta[1,3]-var_beta[1,4];		
cov_diuds=(lambda_y_eta_fnl-lambda_s_eta_fnl)*sig_eta_fnl^2+(lambda_y_eps_fnl-1)*sig_eps_fnl^2+lambda_s_eta_fnl*(lambda_y_eta_fnl-lambda_s_eta_fnl)*sig_eta_fnl^2;
corr_ddids=cov_ddids/(sig_ddi*sig_ds);
corr_diuds=cov_diuds/(sig_di_u*sig_ds);

corr_ddids_pre=corr_ddids;
corr_diuds_pre=corr_diuds;


      
sig_ddi_pre=sig_ddi;
cov_il_pre=cov_il;
cov_is_pre=cov_is;
cov_ils_pre=cov_ils;
sig_dy_pre=sig_dy;
sig_ds_pre=sig_ds;
sig_eta_pre=sig_eta_fnl;
sig_v_pre=sig_v_fnl;
sig_eps_pre=sig_eps_fnl;
sig_di_pre=sig_di;
sig_di_u_pre=sig_di_u;
sig_di_p_pre=sqrt(sig_di^2-sig_di_u^2);
ratio_pre=sig_di_u/sig_di; if ratio_pre ge 1; "not possible"; endif;

sig_di_u_eta_pre=sqrt(((lambda_y_eta_fnl-lambda_s_eta_fnl)^2)*sig_eta_fnl^2);
sig_di_u_eps_pre=sqrt(((lambda_y_eps_fnl-1)^2)*sig_eps_fnl^2);
sig_di_u_v_pre=sqrt(((1+lambda_i_v_fnl)^2)*sig_v_fnl^2);

ratio_eta_di_u_pre=sig_di_u_eta_pre^2/(sig_di_u^2);
ratio_eps_di_u_pre=sig_di_u_eps_pre^2/(sig_di_u^2);
ratio_v_di_u_pre=sig_di_u_v_pre^2/(sig_di_u^2);
ratio_u_di_u_pre=sig_u_fnl^2/(sig_di_u^2);

alpha_y_eta_pre=((phi_s1_fnl-1)*lambda_s_eta_fnl+(phi_i1_fnl-2)*lambda_i_eta_fnl-gamma_itao_fnl-lambda_k_eta_fnl)/(gamma_itao_fnl+lambda_i_eta_fnl+lambda_k_eta_fnl);
alpha_y_eps_pre=((phi_s1_fnl-1)+(phi_i1_fnl-2)*lambda_i_eps_fnl)/(lambda_i_eps_fnl+lambda_k_eps_fnl);
alpha_y_v_pre=((phi_i1_fnl-2)*lambda_i_v_fnl-1)/(1+lambda_i_v_fnl);
alpha_y_u_pre=(phi_i1_fnl-2);

alpha_s_eta_pre=((phi_s1_fnl-1)*lambda_s_eta_fnl)/(gamma_itao_fnl+lambda_i_eta_fnl+lambda_k_eta_fnl);
alpha_s_eps_pre=((phi_s1_fnl-1))/(lambda_i_eps_fnl+lambda_k_eps_fnl);
alpha_s_v_pre=0;
alpha_s_u_pre=0;


prm_fnl=prm_fnl_post;

phi_s1_fnl=prm_fnl[1]+prm_fnl[2];
phi_s2_fnl=-prm_fnl[2];
phi_i1_fnl=prm_fnl[3]+prm_fnl[4];
phi_i2_fnl=-prm_fnl[4];
sig_eta_fnl=prm_fnl[5];
sig_eps_fnl=prm_fnl[6];
sig_v_fnl=prm_fnl[7];
sig_u_fnl=prm_fnl[8];
gamma_itao_fnl=1;
lambda_s_eta_fnl=prm_fnl[10];          
lambda_y_eta_fnl=prm_fnl[11];
lambda_y_eps_fnl=prm_fnl[12];         
lambda_i_v_fnl=  prm_fnl[13];      
lambda_k_eta_fnl=  prm_fnl[14];  
lambda_k_eps_fnl=  0;
lambda_i_eta_fnl=lambda_y_eta_fnl-lambda_s_eta_fnl - gamma_itao_fnl-lambda_k_eta_fnl;
lambda_i_eps_fnl=lambda_y_eps_fnl-1-lambda_k_eps_fnl; 


sig_di_u=sqrt(((lambda_y_eta_fnl-lambda_s_eta_fnl)^2)*sig_eta_fnl^2+((lambda_y_eps_fnl-1)^2)*sig_eps_fnl^2
+((1+lambda_i_v_fnl)^2)*sig_v_fnl^2+sig_u_fnl^2);      @standard deviation of unplanned/unexpected inventories Feb 2009@




@Calculate standard deviation of change in output and of change in inventories@
@phi(L)*z_t = x_t - x_t-1@
@x_t = lambda_ieta*eta + lambda_iv*v + lambda_ieps*eps + u@
@z_t = phi(L)^-1*x_t - phi(L)^-1*x_t-1@
@see notes from Nov. 26, 2008@
@Beta_t = F * Beta_t-1 + w_t@
@where Beta_t = z_st, z_st-1, z_it, z_it-1, x_st, x_it@
@w_t = x_st, 0, x_it, 0, x_st, x_it@
@updated for Aug 2014 document@



f_= phi_s1_fnl~phi_s2_fnl~0~0~-1~0|
    1~0~0~0~0~0|
    0~0~phi_i1_fnl~phi_i2_fnl~0~-1|
    0~0~1~0~0~0|
    0~0~0~0~0~0|
    0~0~0~0~0~0;
q_= zeros(rows(f_),rows(f_));
q_[1,1]=lambda_s_eta_fnl^2*sig_eta_fnl^2 + sig_eps_fnl^2;
q_[5,5]=q_[1,1];
q_[1,5]=q_[1,1];
q_[5,1]=q_[1,1];
q_[3,3]=lambda_i_eta_fnl^2*sig_eta_fnl^2+lambda_i_v_fnl^2*sig_v_fnl^2+lambda_i_eps_fnl^2*sig_eps_fnl^2+sig_u_fnl^2;
q_[6,6]=q_[3,3];
q_[3,6]=q_[3,3];
q_[6,3]=q_[3,3];
q_[1,3]=lambda_s_eta_fnl*lambda_i_eta_fnl*sig_eta_fnl^2 + lambda_i_eps_fnl*sig_eps_fnl^2;
q_[3,1]=q_[1,3];
q_[1,6]=q_[1,3];
q_[6,1]=q_[1,3];
q_[3,5]=q_[1,3];
q_[5,3]=q_[1,3];
q_[5,6]=q_[1,3];
q_[6,5]=q_[1,3];

var_beta=reshape(inv(eye(6^2)-f_.*.f_)*vec(q_),6,6);


sig_di=sqrt((gamma_itao_fnl+lambda_k_eta_fnl)^2*sig_eta_fnl^2+lambda_k_eps_fnl^2*sig_eps_fnl^2+sig_v_fnl^2+var_beta[3,3]+
            2*(gamma_itao_fnl+lambda_k_eta_fnl)*lambda_i_eta_fnl*sig_eta_fnl^2+2*lambda_k_eps_fnl*lambda_i_eps_fnl*sig_eps_fnl^2+2*lambda_i_v_fnl*sig_v_fnl^2);

sig_ds = sqrt( sig_eta_fnl^2 + var_beta[1,1] + 2*lambda_s_eta_fnl*sig_eta_fnl^2);

cov_il=(phi_i1_fnl-1)*((gamma_itao_fnl+lambda_k_eta_fnl)*lambda_i_eta_fnl*sig_eta_fnl^2 + lambda_i_v_fnl*sig_v_fnl^2+lambda_i_eps_fnl*lambda_k_eps_fnl*sig_eps_fnl^2)
        +var_beta[3,4];

cov_is=sig_eta_fnl^2*(gamma_itao_fnl + lambda_k_eta_fnl +lambda_i_eta_fnl + (gamma_itao_fnl+lambda_k_eta_fnl)*lambda_s_eta_fnl) + lambda_k_eps_fnl*sig_eps_fnl^2
        +var_beta[1,3];

cov_ils=(phi_s1_fnl-1)*((gamma_itao_fnl+lambda_k_eta_fnl)*lambda_s_eta_fnl*sig_eta_fnl^2+lambda_k_eps_fnl*sig_eps_fnl^2) 
        +var_beta[1,4];


@see updated version of Nov. 26, 2008 document for following@
sig_dy = sqrt( 2*sig_di^2 +sig_ds^2 
                -2*cov_il + 2*cov_is - 2*cov_ils );

sig_ddi = sqrt( 2*sig_di^2 - 2*cov_il);


@see notes from Nov. 3, 2014 for the following@
cov_ddids=(1+lambda_k_eta_fnl+lambda_i_eta_fnl+lambda_s_eta_fnl*(1+lambda_k_eta_fnl)-(phi_s1_fnl -1)*lambda_s_eta_fnl*(1+lambda_k_eta_fnl))*sig_eta_fnl^2+
                                var_beta[1,3]-var_beta[1,4];		
cov_diuds=(lambda_y_eta_fnl-lambda_s_eta_fnl)*sig_eta_fnl^2+(lambda_y_eps_fnl-1)*sig_eps_fnl^2+lambda_s_eta_fnl*(lambda_y_eta_fnl-lambda_s_eta_fnl)*sig_eta_fnl^2;
corr_ddids=cov_ddids/(sig_ddi*sig_ds);
corr_diuds=cov_diuds/(sig_di_u*sig_ds);

corr_ddids_post=corr_ddids;
corr_diuds_post=corr_diuds;
      
sig_ddi_post=sig_ddi;
cov_il_post=cov_il;
cov_is_post=cov_is;
cov_ils_post=cov_ils;
sig_dy_post=sig_dy;
sig_ds_post=sig_ds;
sig_eta_post=sig_eta_fnl;
sig_v_post=sig_v_fnl;
sig_eps_post=sig_eps_fnl;
sig_di_post=sig_di;
sig_di_u_post=sig_di_u;
sig_di_p_post=sqrt(sig_di^2-sig_di_u^2);
ratio_post=sig_di_u/sig_di;

sig_di_u_eta_post=sqrt(((lambda_y_eta_fnl-lambda_s_eta_fnl)^2)*sig_eta_fnl^2);
sig_di_u_eps_post=sqrt(((lambda_y_eps_fnl-1)^2)*sig_eps_fnl^2);
sig_di_u_v_post=sqrt(((1+lambda_i_v_fnl)^2)*sig_v_fnl^2);

ratio_eta_di_u_post=sig_di_u_eta_post^2/(sig_di_u^2);
ratio_eps_di_u_post=sig_di_u_eps_post^2/(sig_di_u^2);
ratio_v_di_u_post=sig_di_u_v_post^2/(sig_di_u^2);
ratio_u_di_u_post=sig_u_fnl^2/(sig_di_u^2);

alpha_y_eta_post=((phi_s1_fnl-1)*lambda_s_eta_fnl+(phi_i1_fnl-2)*lambda_i_eta_fnl-gamma_itao_fnl-lambda_k_eta_fnl)/(gamma_itao_fnl+lambda_i_eta_fnl+lambda_k_eta_fnl);
alpha_y_eps_post=((phi_s1_fnl-1)+(phi_i1_fnl-2)*lambda_i_eps_fnl)/(lambda_i_eps_fnl+lambda_k_eps_fnl);
alpha_y_v_post=((phi_i1_fnl-2)*lambda_i_v_fnl-1)/(1+lambda_i_v_fnl);
alpha_y_u_post=(phi_i1_fnl-2);

alpha_s_eta_post=((phi_s1_fnl-1)*lambda_s_eta_fnl)/(gamma_itao_fnl+lambda_i_eta_fnl+lambda_k_eta_fnl);
alpha_s_eps_post=((phi_s1_fnl-1))/(lambda_i_eps_fnl+lambda_k_eps_fnl);
alpha_s_v_post=0;
alpha_s_u_post=0;






@*******************************************************************@
@Counterfactual Variances for change in shocks only@
@Start with Pre sample as benchmark@
@Write over some pre-parameters with post-parameters@
@E.g., only shock variances are overwritten below@

prm_fnl=prm_fnl_pre;

phi_s1_fnl=prm_fnl[1]+prm_fnl[2];
phi_s2_fnl=-prm_fnl[2];
phi_i1_fnl=prm_fnl[3]+prm_fnl[4];
phi_i2_fnl=-prm_fnl[4];
sig_eta_fnl=prm_fnl[5];
sig_eps_fnl=prm_fnl[6];
sig_v_fnl=prm_fnl[7];
sig_u_fnl=prm_fnl[8];
gamma_itao_fnl=1;
lambda_s_eta_fnl=prm_fnl[10];          
lambda_y_eta_fnl=prm_fnl[11];
lambda_y_eps_fnl=prm_fnl[12];         
lambda_i_v_fnl=  prm_fnl[13];      
lambda_k_eta_fnl=  prm_fnl[14];  
lambda_k_eps_fnl=  0;
lambda_i_eta_fnl=lambda_y_eta_fnl-lambda_s_eta_fnl - gamma_itao_fnl-lambda_k_eta_fnl;
lambda_i_eps_fnl=lambda_y_eps_fnl-1-lambda_k_eps_fnl; 

prm_fnl=prm_fnl_post;


sig_eta_fnl=prm_fnl[5];
sig_eps_fnl=prm_fnl[6];
sig_v_fnl=prm_fnl[7];
sig_u_fnl=prm_fnl[8];


sig_di_u=sqrt(((lambda_y_eta_fnl-lambda_s_eta_fnl)^2)*sig_eta_fnl^2+((lambda_y_eps_fnl-1)^2)*sig_eps_fnl^2
+((1+lambda_i_v_fnl)^2)*sig_v_fnl^2+sig_u_fnl^2);      @standard deviation of unplanned/unexpected inventories Feb 2009@

@Calculate standard deviation of change in output and of change in inventories@
@phi(L)*z_t = x_t - x_t-1@
@x_t = lambda_ieta*eta + lambda_iv*v + lambda_ieps*eps + u@
@z_t = phi(L)^-1*x_t - phi(L)^-1*x_t-1@
@see notes from Nov. 26, 2008@
@Beta_t = F * Beta_t-1 + w_t@
@where Beta_t = z_st, z_st-1, z_it, z_it-1, x_st, x_it@
@w_t = x_st, 0, x_it, 0, x_st, x_it@



f_= phi_s1_fnl~phi_s2_fnl~0~0~-1~0|
    1~0~0~0~0~0|
    0~0~phi_i1_fnl~phi_i2_fnl~0~-1|
    0~0~1~0~0~0|
    0~0~0~0~0~0|
    0~0~0~0~0~0;
q_= zeros(rows(f_),rows(f_));
q_[1,1]=lambda_s_eta_fnl^2*sig_eta_fnl^2 + sig_eps_fnl^2;
q_[5,5]=q_[1,1];
q_[1,5]=q_[1,1];
q_[5,1]=q_[1,1];
q_[3,3]=lambda_i_eta_fnl^2*sig_eta_fnl^2+lambda_i_v_fnl^2*sig_v_fnl^2+lambda_i_eps_fnl^2*sig_eps_fnl^2+sig_u_fnl^2;
q_[6,6]=q_[3,3];
q_[3,6]=q_[3,3];
q_[6,3]=q_[3,3];
q_[1,3]=lambda_s_eta_fnl*lambda_i_eta_fnl*sig_eta_fnl^2 + lambda_i_eps_fnl*sig_eps_fnl^2;
q_[3,1]=q_[1,3];
q_[1,6]=q_[1,3];
q_[6,1]=q_[1,3];
q_[3,5]=q_[1,3];
q_[5,3]=q_[1,3];
q_[5,6]=q_[1,3];
q_[6,5]=q_[1,3];

var_beta=reshape(inv(eye(6^2)-f_.*.f_)*vec(q_),6,6);


sig_di=sqrt((gamma_itao_fnl+lambda_k_eta_fnl)^2*sig_eta_fnl^2+lambda_k_eps_fnl^2*sig_eps_fnl^2+sig_v_fnl^2+var_beta[3,3]+
            2*(gamma_itao_fnl+lambda_k_eta_fnl)*lambda_i_eta_fnl*sig_eta_fnl^2+2*lambda_k_eps_fnl*lambda_i_eps_fnl*sig_eps_fnl^2+2*lambda_i_v_fnl*sig_v_fnl^2);

sig_ds = sqrt( sig_eta_fnl^2 + var_beta[1,1] + 2*lambda_s_eta_fnl*sig_eta_fnl^2);

cov_il=(phi_i1_fnl-1)*((gamma_itao_fnl+lambda_k_eta_fnl)*lambda_i_eta_fnl*sig_eta_fnl^2 + lambda_i_v_fnl*sig_v_fnl^2+lambda_i_eps_fnl*lambda_k_eps_fnl*sig_eps_fnl^2)
        +var_beta[3,4];

cov_is=sig_eta_fnl^2*(gamma_itao_fnl + lambda_k_eta_fnl +lambda_i_eta_fnl + (gamma_itao_fnl+lambda_k_eta_fnl)*lambda_s_eta_fnl) + lambda_k_eps_fnl*sig_eps_fnl^2
        +var_beta[1,3];

cov_ils=(phi_s1_fnl-1)*((gamma_itao_fnl+lambda_k_eta_fnl)*lambda_s_eta_fnl*sig_eta_fnl^2+lambda_k_eps_fnl*sig_eps_fnl^2) 
        +var_beta[1,4];


@see updated version of Nov. 26, 2008 document for following@
sig_dy = sqrt( 2*sig_di^2 +sig_ds^2 
                -2*cov_il + 2*cov_is - 2*cov_ils );

sig_ddi = sqrt( 2*sig_di^2 - 2*cov_il);

@see notes from Nov. 3, 2014 for the following@
cov_ddids=(1+lambda_k_eta_fnl+lambda_i_eta_fnl+lambda_s_eta_fnl*(1+lambda_k_eta_fnl)-(phi_s1_fnl -1)*lambda_s_eta_fnl*(1+lambda_k_eta_fnl))*sig_eta_fnl^2+
                                var_beta[1,3]-var_beta[1,4];		
cov_diuds=(lambda_y_eta_fnl-lambda_s_eta_fnl)*sig_eta_fnl^2+(lambda_y_eps_fnl-1)*sig_eps_fnl^2+lambda_s_eta_fnl*(lambda_y_eta_fnl-lambda_s_eta_fnl)*sig_eta_fnl^2;
corr_ddids=cov_ddids/(sig_ddi*sig_ds);
corr_diuds=cov_diuds/(sig_di_u*sig_ds);

corr_ddids_cf1=corr_ddids;
corr_diuds_cf1=corr_diuds;      
	  
sig_ddi_cf1=sig_ddi;
cov_il_cf1=cov_il;
cov_is_cf1=cov_is;
cov_ils_cf1=cov_ils;
sig_dy_cf1=sig_dy;
sig_ds_cf1=sig_ds;
dsig_dy_cf1=sig_dy-sig_dy_pre;
dsig_ds_cf1=sig_ds-sig_ds_pre;
sig_eta_cf1=sig_eta_fnl;
sig_v_cf1=sig_v_fnl;
sig_eps_cf1=sig_eps_fnl;
sig_di_cf1=sig_di;
sig_di_u_cf1=sig_di_u;
sig_di_p_cf1=sqrt(sig_di^2-sig_di_u^2);
ratio_cf1=sig_di_u/sig_di;


@********@

@Counterfactual Variances for change in AR and impact coefficients@
@Start with Pre sample as benchmark@
@Write over some pre-parameters with post-parameters@
@E.g., everything but shock variances are overwritten below@

prm_fnl=prm_fnl_pre;

phi_s1_fnl=prm_fnl[1]+prm_fnl[2];
phi_s2_fnl=-prm_fnl[2];
phi_i1_fnl=prm_fnl[3]+prm_fnl[4];
phi_i2_fnl=-prm_fnl[4];
sig_eta_fnl=prm_fnl[5];
sig_eps_fnl=prm_fnl[6];
sig_v_fnl=prm_fnl[7];
sig_u_fnl=prm_fnl[8];
gamma_itao_fnl=1;
lambda_s_eta_fnl=prm_fnl[10];          
lambda_y_eta_fnl=prm_fnl[11];
lambda_y_eps_fnl=prm_fnl[12];         
lambda_i_v_fnl=  prm_fnl[13];      
lambda_k_eta_fnl=  prm_fnl[14];  
lambda_k_eps_fnl=  0;
lambda_i_eta_fnl=lambda_y_eta_fnl-lambda_s_eta_fnl - gamma_itao_fnl-lambda_k_eta_fnl;
lambda_i_eps_fnl=lambda_y_eps_fnl-1-lambda_k_eps_fnl;

prm_fnl=prm_fnl_post;

phi_s1_fnl=prm_fnl[1]+prm_fnl[2];
phi_s2_fnl=-prm_fnl[2];
phi_i1_fnl=prm_fnl[3]+prm_fnl[4];
phi_i2_fnl=-prm_fnl[4];

gamma_itao_fnl=1;
lambda_s_eta_fnl=prm_fnl[10];          
lambda_y_eta_fnl=prm_fnl[11];
lambda_y_eps_fnl=prm_fnl[12];         
lambda_i_v_fnl=  prm_fnl[13];      
lambda_k_eta_fnl=  prm_fnl[14];  
lambda_k_eps_fnl=  0;
lambda_i_eta_fnl=lambda_y_eta_fnl-lambda_s_eta_fnl - gamma_itao_fnl-lambda_k_eta_fnl;
lambda_i_eps_fnl=lambda_y_eps_fnl-1-lambda_k_eps_fnl; 



sig_di_u=sqrt(((lambda_y_eta_fnl-lambda_s_eta_fnl)^2)*sig_eta_fnl^2+((lambda_y_eps_fnl-1)^2)*sig_eps_fnl^2
+((1+lambda_i_v_fnl)^2)*sig_v_fnl^2+sig_u_fnl^2);      @standard deviation of unplanned/unexpected inventories Feb 2009@


@Calculate standard deviation of change in output and of change in inventories@
@phi(L)*z_t = x_t - x_t-1@
@x_t = lambda_ieta*eta + lambda_iv*v + lambda_ieps*eps + u@
@z_t = phi(L)^-1*x_t - phi(L)^-1*x_t-1@
@see notes from Nov. 26, 2008@
@Beta_t = F * Beta_t-1 + w_t@
@where Beta_t = z_st, z_st-1, z_it, z_it-1, x_st, x_it@
@w_t = x_st, 0, x_it, 0, x_st, x_it@



f_= phi_s1_fnl~phi_s2_fnl~0~0~-1~0|
    1~0~0~0~0~0|
    0~0~phi_i1_fnl~phi_i2_fnl~0~-1|
    0~0~1~0~0~0|
    0~0~0~0~0~0|
    0~0~0~0~0~0;
q_= zeros(rows(f_),rows(f_));
q_[1,1]=lambda_s_eta_fnl^2*sig_eta_fnl^2 + sig_eps_fnl^2;
q_[5,5]=q_[1,1];
q_[1,5]=q_[1,1];
q_[5,1]=q_[1,1];
q_[3,3]=lambda_i_eta_fnl^2*sig_eta_fnl^2+lambda_i_v_fnl^2*sig_v_fnl^2+lambda_i_eps_fnl^2*sig_eps_fnl^2+sig_u_fnl^2;
q_[6,6]=q_[3,3];
q_[3,6]=q_[3,3];
q_[6,3]=q_[3,3];
q_[1,3]=lambda_s_eta_fnl*lambda_i_eta_fnl*sig_eta_fnl^2 + lambda_i_eps_fnl*sig_eps_fnl^2;
q_[3,1]=q_[1,3];
q_[1,6]=q_[1,3];
q_[6,1]=q_[1,3];
q_[3,5]=q_[1,3];
q_[5,3]=q_[1,3];
q_[5,6]=q_[1,3];
q_[6,5]=q_[1,3];

var_beta=reshape(inv(eye(6^2)-f_.*.f_)*vec(q_),6,6);


sig_di=sqrt((gamma_itao_fnl+lambda_k_eta_fnl)^2*sig_eta_fnl^2+lambda_k_eps_fnl^2*sig_eps_fnl^2+sig_v_fnl^2+var_beta[3,3]+
            2*(gamma_itao_fnl+lambda_k_eta_fnl)*lambda_i_eta_fnl*sig_eta_fnl^2+2*lambda_k_eps_fnl*lambda_i_eps_fnl*sig_eps_fnl^2+2*lambda_i_v_fnl*sig_v_fnl^2);

sig_ds = sqrt( sig_eta_fnl^2 + var_beta[1,1] + 2*lambda_s_eta_fnl*sig_eta_fnl^2);

cov_il=(phi_i1_fnl-1)*((gamma_itao_fnl+lambda_k_eta_fnl)*lambda_i_eta_fnl*sig_eta_fnl^2 + lambda_i_v_fnl*sig_v_fnl^2+lambda_i_eps_fnl*lambda_k_eps_fnl*sig_eps_fnl^2)
        +var_beta[3,4];

cov_is=sig_eta_fnl^2*(gamma_itao_fnl + lambda_k_eta_fnl +lambda_i_eta_fnl + (gamma_itao_fnl+lambda_k_eta_fnl)*lambda_s_eta_fnl) + lambda_k_eps_fnl*sig_eps_fnl^2
        +var_beta[1,3];

cov_ils=(phi_s1_fnl-1)*((gamma_itao_fnl+lambda_k_eta_fnl)*lambda_s_eta_fnl*sig_eta_fnl^2+lambda_k_eps_fnl*sig_eps_fnl^2) 
        +var_beta[1,4];


@see updated version of Nov. 26, 2008 document for following@
sig_dy = sqrt( 2*sig_di^2 +sig_ds^2 
                -2*cov_il + 2*cov_is - 2*cov_ils );

sig_ddi = sqrt( 2*sig_di^2 - 2*cov_il);

@see notes from Nov. 3, 2014 for the following@
cov_ddids=(1+lambda_k_eta_fnl+lambda_i_eta_fnl+lambda_s_eta_fnl*(1+lambda_k_eta_fnl)-(phi_s1_fnl -1)*lambda_s_eta_fnl*(1+lambda_k_eta_fnl))*sig_eta_fnl^2+
                                var_beta[1,3]-var_beta[1,4];		
cov_diuds=(lambda_y_eta_fnl-lambda_s_eta_fnl)*sig_eta_fnl^2+(lambda_y_eps_fnl-1)*sig_eps_fnl^2+lambda_s_eta_fnl*(lambda_y_eta_fnl-lambda_s_eta_fnl)*sig_eta_fnl^2;
corr_ddids=cov_ddids/(sig_ddi*sig_ds);
corr_diuds=cov_diuds/(sig_di_u*sig_ds);

corr_ddids_cf2=corr_ddids;
corr_diuds_cf2=corr_diuds;


sig_ddi_cf2=sig_ddi;
cov_il_cf2=cov_il;
cov_is_cf2=cov_is;
cov_ils_cf2=cov_ils;
sig_dy_cf2=sig_dy;
sig_ds_cf2=sig_ds;
dsig_dy_cf2=sig_dy-sig_dy_pre;
dsig_ds_cf2=sig_ds-sig_ds_pre;
sig_eta_cf2=sig_eta_fnl;
sig_v_cf2=sig_v_fnl;
sig_eps_cf2=sig_eps_fnl;
sig_di_cf2=sig_di;
sig_di_u_cf2=sig_di_u;
sig_di_p_cf2=sqrt(sig_di^2-sig_di_u^2);
ratio_cf2=sig_di_u/sig_di;



@***********@
@Counterfactual Variances for change in AR only@
@Start with Pre sample as benchmark@
@Write over some pre-parameters with post-parameters@
@E.g., only AR coeffs are overwritten below@

prm_fnl=prm_fnl_pre;

phi_s1_fnl=prm_fnl[1]+prm_fnl[2];
phi_s2_fnl=-prm_fnl[2];
phi_i1_fnl=prm_fnl[3]+prm_fnl[4];
phi_i2_fnl=-prm_fnl[4];
sig_eta_fnl=prm_fnl[5];
sig_eps_fnl=prm_fnl[6];
sig_v_fnl=prm_fnl[7];
sig_u_fnl=prm_fnl[8];
gamma_itao_fnl=1;
lambda_s_eta_fnl=prm_fnl[10];          
lambda_y_eta_fnl=prm_fnl[11];
lambda_y_eps_fnl=prm_fnl[12];         
lambda_i_v_fnl=  prm_fnl[13];      
lambda_k_eta_fnl=  prm_fnl[14];  
lambda_k_eps_fnl=  0;
lambda_i_eta_fnl=lambda_y_eta_fnl-lambda_s_eta_fnl - gamma_itao_fnl-lambda_k_eta_fnl;
lambda_i_eps_fnl=lambda_y_eps_fnl-1-lambda_k_eps_fnl; 

prm_fnl=prm_fnl_post;

phi_s1_fnl=prm_fnl[1]+prm_fnl[2];
phi_s2_fnl=-prm_fnl[2];
phi_i1_fnl=prm_fnl[3]+prm_fnl[4];
phi_i2_fnl=-prm_fnl[4];    
 



sig_di_u=sqrt(((lambda_y_eta_fnl-lambda_s_eta_fnl)^2)*sig_eta_fnl^2+((lambda_y_eps_fnl-1)^2)*sig_eps_fnl^2
+((1+lambda_i_v_fnl)^2)*sig_v_fnl^2+sig_u_fnl^2);      @standard deviation of unplanned/unexpected inventories Feb 2009@


@Calculate standard deviation of change in output and of change in inventories@
@phi(L)*z_t = x_t - x_t-1@
@x_t = lambda_ieta*eta + lambda_iv*v + lambda_ieps*eps + u@
@z_t = phi(L)^-1*x_t - phi(L)^-1*x_t-1@
@see notes from Nov. 26, 2008@
@Beta_t = F * Beta_t-1 + w_t@
@where Beta_t = z_st, z_st-1, z_it, z_it-1, x_st, x_it@
@w_t = x_st, 0, x_it, 0, x_st, x_it@



f_= phi_s1_fnl~phi_s2_fnl~0~0~-1~0|
    1~0~0~0~0~0|
    0~0~phi_i1_fnl~phi_i2_fnl~0~-1|
    0~0~1~0~0~0|
    0~0~0~0~0~0|
    0~0~0~0~0~0;
q_= zeros(rows(f_),rows(f_));
q_[1,1]=lambda_s_eta_fnl^2*sig_eta_fnl^2 + sig_eps_fnl^2;
q_[5,5]=q_[1,1];
q_[1,5]=q_[1,1];
q_[5,1]=q_[1,1];
q_[3,3]=lambda_i_eta_fnl^2*sig_eta_fnl^2+lambda_i_v_fnl^2*sig_v_fnl^2+lambda_i_eps_fnl^2*sig_eps_fnl^2+sig_u_fnl^2;
q_[6,6]=q_[3,3];
q_[3,6]=q_[3,3];
q_[6,3]=q_[3,3];
q_[1,3]=lambda_s_eta_fnl*lambda_i_eta_fnl*sig_eta_fnl^2 + lambda_i_eps_fnl*sig_eps_fnl^2;
q_[3,1]=q_[1,3];
q_[1,6]=q_[1,3];
q_[6,1]=q_[1,3];
q_[3,5]=q_[1,3];
q_[5,3]=q_[1,3];
q_[5,6]=q_[1,3];
q_[6,5]=q_[1,3];

var_beta=reshape(inv(eye(6^2)-f_.*.f_)*vec(q_),6,6);


sig_di=sqrt((gamma_itao_fnl+lambda_k_eta_fnl)^2*sig_eta_fnl^2+lambda_k_eps_fnl^2*sig_eps_fnl^2+sig_v_fnl^2+var_beta[3,3]+
            2*(gamma_itao_fnl+lambda_k_eta_fnl)*lambda_i_eta_fnl*sig_eta_fnl^2+2*lambda_k_eps_fnl*lambda_i_eps_fnl*sig_eps_fnl^2+2*lambda_i_v_fnl*sig_v_fnl^2);

sig_ds = sqrt( sig_eta_fnl^2 + var_beta[1,1] + 2*lambda_s_eta_fnl*sig_eta_fnl^2);

cov_il=(phi_i1_fnl-1)*((gamma_itao_fnl+lambda_k_eta_fnl)*lambda_i_eta_fnl*sig_eta_fnl^2 + lambda_i_v_fnl*sig_v_fnl^2+lambda_i_eps_fnl*lambda_k_eps_fnl*sig_eps_fnl^2)
        +var_beta[3,4];

cov_is=sig_eta_fnl^2*(gamma_itao_fnl + lambda_k_eta_fnl +lambda_i_eta_fnl + (gamma_itao_fnl+lambda_k_eta_fnl)*lambda_s_eta_fnl) + lambda_k_eps_fnl*sig_eps_fnl^2
        +var_beta[1,3];

cov_ils=(phi_s1_fnl-1)*((gamma_itao_fnl+lambda_k_eta_fnl)*lambda_s_eta_fnl*sig_eta_fnl^2+lambda_k_eps_fnl*sig_eps_fnl^2) 
        +var_beta[1,4];


@see updated version of Nov. 26, 2008 document for following@
sig_dy = sqrt( 2*sig_di^2 +sig_ds^2 
                -2*cov_il + 2*cov_is - 2*cov_ils );

sig_ddi = sqrt( 2*sig_di^2 - 2*cov_il);

@see notes from Nov. 3, 2014 for the following@
cov_ddids=(1+lambda_k_eta_fnl+lambda_i_eta_fnl+lambda_s_eta_fnl*(1+lambda_k_eta_fnl)-(phi_s1_fnl -1)*lambda_s_eta_fnl*(1+lambda_k_eta_fnl))*sig_eta_fnl^2+
                                var_beta[1,3]-var_beta[1,4];		
cov_diuds=(lambda_y_eta_fnl-lambda_s_eta_fnl)*sig_eta_fnl^2+(lambda_y_eps_fnl-1)*sig_eps_fnl^2+lambda_s_eta_fnl*(lambda_y_eta_fnl-lambda_s_eta_fnl)*sig_eta_fnl^2;
corr_ddids=cov_ddids/(sig_ddi*sig_ds);
corr_diuds=cov_diuds/(sig_di_u*sig_ds);

corr_ddids_cf3=corr_ddids;
corr_diuds_cf3=corr_diuds;
      
sig_ddi_cf3=sig_ddi;
cov_il_cf3=cov_il;
cov_is_cf3=cov_is;
cov_ils_cf3=cov_ils;
sig_dy_cf3=sig_dy;
sig_ds_cf3=sig_ds;
dsig_dy_cf3=sig_dy-sig_dy_pre;
dsig_ds_cf3=sig_ds-sig_ds_pre;
sig_eta_cf3=sig_eta_fnl;
sig_v_cf3=sig_v_fnl;
sig_eps_cf3=sig_eps_fnl;
sig_di_cf3=sig_di;
sig_di_u_cf3=sig_di_u;
sig_di_p_cf3=sqrt(sig_di^2-sig_di_u^2);
ratio_cf3=sig_di_u/sig_di;


@***********@
@Counterfactual Variances for change in impact coeffs only@
@Start with Pre sample as benchmark@
@Write over some pre-parameters with post-parameters@
@E.g., only impact coeffs are overwritten below@

prm_fnl=prm_fnl_pre;

phi_s1_fnl=prm_fnl[1]+prm_fnl[2];
phi_s2_fnl=-prm_fnl[2];
phi_i1_fnl=prm_fnl[3]+prm_fnl[4];
phi_i2_fnl=-prm_fnl[4];
sig_eta_fnl=prm_fnl[5];
sig_eps_fnl=prm_fnl[6];
sig_v_fnl=prm_fnl[7];
sig_u_fnl=prm_fnl[8];
gamma_itao_fnl=1;
lambda_s_eta_fnl=prm_fnl[10];          
lambda_y_eta_fnl=prm_fnl[11];
lambda_y_eps_fnl=prm_fnl[12];         
lambda_i_v_fnl=  prm_fnl[13];      
lambda_k_eta_fnl=  prm_fnl[14];  
lambda_k_eps_fnl=  0;
lambda_i_eta_fnl=lambda_y_eta_fnl-lambda_s_eta_fnl - gamma_itao_fnl-lambda_k_eta_fnl;
lambda_i_eps_fnl=lambda_y_eps_fnl-1-lambda_k_eps_fnl;

prm_fnl=prm_fnl_post;


gamma_itao_fnl=1;
lambda_s_eta_fnl=prm_fnl[10];          
lambda_y_eta_fnl=prm_fnl[11];
lambda_y_eps_fnl=prm_fnl[12];         
lambda_i_v_fnl=  prm_fnl[13];      
lambda_k_eta_fnl=  prm_fnl[14];  
lambda_k_eps_fnl=  0;
lambda_i_eta_fnl=lambda_y_eta_fnl-lambda_s_eta_fnl - gamma_itao_fnl-lambda_k_eta_fnl;
lambda_i_eps_fnl=lambda_y_eps_fnl-1-lambda_k_eps_fnl; 


sig_di_u=sqrt(((lambda_y_eta_fnl-lambda_s_eta_fnl)^2)*sig_eta_fnl^2+((lambda_y_eps_fnl-1)^2)*sig_eps_fnl^2
+((1+lambda_i_v_fnl)^2)*sig_v_fnl^2+sig_u_fnl^2);      @standard deviation of unplanned/unexpected inventories Feb 2009@


@Calculate standard deviation of change in output and of change in inventories@
@phi(L)*z_t = x_t - x_t-1@
@x_t = lambda_ieta*eta + lambda_iv*v + lambda_ieps*eps + u@
@z_t = phi(L)^-1*x_t - phi(L)^-1*x_t-1@
@see notes from Nov. 26, 2008@
@Beta_t = F * Beta_t-1 + w_t@
@where Beta_t = z_st, z_st-1, z_it, z_it-1, x_st, x_it@
@w_t = x_st, 0, x_it, 0, x_st, x_it@



f_= phi_s1_fnl~phi_s2_fnl~0~0~-1~0|
    1~0~0~0~0~0|
    0~0~phi_i1_fnl~phi_i2_fnl~0~-1|
    0~0~1~0~0~0|
    0~0~0~0~0~0|
    0~0~0~0~0~0;
q_= zeros(rows(f_),rows(f_));
q_[1,1]=lambda_s_eta_fnl^2*sig_eta_fnl^2 + sig_eps_fnl^2;
q_[5,5]=q_[1,1];
q_[1,5]=q_[1,1];
q_[5,1]=q_[1,1];
q_[3,3]=lambda_i_eta_fnl^2*sig_eta_fnl^2+lambda_i_v_fnl^2*sig_v_fnl^2+lambda_i_eps_fnl^2*sig_eps_fnl^2+sig_u_fnl^2;
q_[6,6]=q_[3,3];
q_[3,6]=q_[3,3];
q_[6,3]=q_[3,3];
q_[1,3]=lambda_s_eta_fnl*lambda_i_eta_fnl*sig_eta_fnl^2 + lambda_i_eps_fnl*sig_eps_fnl^2;
q_[3,1]=q_[1,3];
q_[1,6]=q_[1,3];
q_[6,1]=q_[1,3];
q_[3,5]=q_[1,3];
q_[5,3]=q_[1,3];
q_[5,6]=q_[1,3];
q_[6,5]=q_[1,3];

var_beta=reshape(inv(eye(6^2)-f_.*.f_)*vec(q_),6,6);


sig_di=sqrt((gamma_itao_fnl+lambda_k_eta_fnl)^2*sig_eta_fnl^2+lambda_k_eps_fnl^2*sig_eps_fnl^2+sig_v_fnl^2+var_beta[3,3]+
            2*(gamma_itao_fnl+lambda_k_eta_fnl)*lambda_i_eta_fnl*sig_eta_fnl^2+2*lambda_k_eps_fnl*lambda_i_eps_fnl*sig_eps_fnl^2+2*lambda_i_v_fnl*sig_v_fnl^2);

sig_ds = sqrt( sig_eta_fnl^2 + var_beta[1,1] + 2*lambda_s_eta_fnl*sig_eta_fnl^2);

cov_il=(phi_i1_fnl-1)*((gamma_itao_fnl+lambda_k_eta_fnl)*lambda_i_eta_fnl*sig_eta_fnl^2 + lambda_i_v_fnl*sig_v_fnl^2+lambda_i_eps_fnl*lambda_k_eps_fnl*sig_eps_fnl^2)
        +var_beta[3,4];

cov_is=sig_eta_fnl^2*(gamma_itao_fnl + lambda_k_eta_fnl +lambda_i_eta_fnl + (gamma_itao_fnl+lambda_k_eta_fnl)*lambda_s_eta_fnl) + lambda_k_eps_fnl*sig_eps_fnl^2
        +var_beta[1,3];

cov_ils=(phi_s1_fnl-1)*((gamma_itao_fnl+lambda_k_eta_fnl)*lambda_s_eta_fnl*sig_eta_fnl^2+lambda_k_eps_fnl*sig_eps_fnl^2) 
        +var_beta[1,4];


@see updated version of Nov. 26, 2008 document for following@
sig_dy = sqrt( 2*sig_di^2 +sig_ds^2 
                -2*cov_il + 2*cov_is - 2*cov_ils );

sig_ddi = sqrt( 2*sig_di^2 - 2*cov_il);

@see notes from Nov. 3, 2014 for the following@
cov_ddids=(1+lambda_k_eta_fnl+lambda_i_eta_fnl+lambda_s_eta_fnl*(1+lambda_k_eta_fnl)-(phi_s1_fnl -1)*lambda_s_eta_fnl*(1+lambda_k_eta_fnl))*sig_eta_fnl^2+
                                var_beta[1,3]-var_beta[1,4];		
cov_diuds=(lambda_y_eta_fnl-lambda_s_eta_fnl)*sig_eta_fnl^2+(lambda_y_eps_fnl-1)*sig_eps_fnl^2+lambda_s_eta_fnl*(lambda_y_eta_fnl-lambda_s_eta_fnl)*sig_eta_fnl^2;
corr_ddids=cov_ddids/(sig_ddi*sig_ds);
corr_diuds=cov_diuds/(sig_di_u*sig_ds);

corr_ddids_cf4=corr_ddids;
corr_diuds_cf4=corr_diuds;
      
sig_ddi_cf4=sig_ddi;
cov_il_cf4=cov_il;
cov_is_cf4=cov_is;
cov_ils_cf4=cov_ils;
sig_dy_cf4=sig_dy;
sig_ds_cf4=sig_ds;
dsig_dy_cf4=sig_dy-sig_dy_pre;
dsig_ds_cf4=sig_ds-sig_ds_pre;
sig_eta_cf4=sig_eta_fnl;
sig_v_cf4=sig_v_fnl;
sig_eps_cf4=sig_eps_fnl;
sig_di_cf4=sig_di;
sig_di_u_cf4=sig_di_u;
sig_di_p_cf4=sqrt(sig_di^2-sig_di_u^2);
ratio_cf4=sig_di_u/sig_di;

@*******************************************************************@
@5th Counterfactual Variances for change in inventory process only@
@Start with Pre sample as benchmark@
@Write over some pre-parameters with post-parameters@

prm_fnl=prm_fnl_pre;

phi_s1_fnl=prm_fnl[1]+prm_fnl[2];
phi_s2_fnl=-prm_fnl[2];
phi_i1_fnl=prm_fnl[3]+prm_fnl[4];
phi_i2_fnl=-prm_fnl[4];
sig_eta_fnl=prm_fnl[5];
sig_eps_fnl=prm_fnl[6];
sig_v_fnl=prm_fnl[7];
sig_u_fnl=prm_fnl[8];
gamma_itao_fnl=1;
lambda_s_eta_fnl=prm_fnl[10];          
lambda_y_eta_fnl=prm_fnl[11];
lambda_y_eps_fnl=prm_fnl[12];         
lambda_i_v_fnl=  prm_fnl[13];      
lambda_k_eta_fnl=  prm_fnl[14];  
lambda_k_eps_fnl=  0;
lambda_i_eta_fnl=lambda_y_eta_fnl-lambda_s_eta_fnl - gamma_itao_fnl-lambda_k_eta_fnl;
lambda_i_eps_fnl=lambda_y_eps_fnl-1-lambda_k_eps_fnl;


prm_fnl=prm_fnl_post;



phi_i1_fnl=prm_fnl[3]+prm_fnl[4];
phi_i2_fnl=-prm_fnl[4];
sig_v_fnl=prm_fnl[7];
sig_u_fnl=prm_fnl[8];
gamma_itao_fnl=1;         
lambda_y_eta_fnl=prm_fnl[11];
lambda_y_eps_fnl=prm_fnl[12];         
lambda_i_v_fnl=  prm_fnl[13];      
lambda_k_eta_fnl=  prm_fnl[14];  
lambda_k_eps_fnl=  0;
lambda_i_eta_fnl=lambda_y_eta_fnl-lambda_s_eta_fnl - gamma_itao_fnl-lambda_k_eta_fnl;
lambda_i_eps_fnl=lambda_y_eps_fnl-1-lambda_k_eps_fnl;


sig_di_u=sqrt(((lambda_y_eta_fnl-lambda_s_eta_fnl)^2)*sig_eta_fnl^2+((lambda_y_eps_fnl-1)^2)*sig_eps_fnl^2
+((1+lambda_i_v_fnl)^2)*sig_v_fnl^2+sig_u_fnl^2);      @standard deviation of unplanned/unexpected inventories Feb 2009@


@Calculate standard deviation of change in output and of change in inventories@
@phi(L)*z_t = x_t - x_t-1@
@x_t = lambda_ieta*eta + lambda_iv*v + lambda_ieps*eps + u@
@z_t = phi(L)^-1*x_t - phi(L)^-1*x_t-1@
@see notes from Nov. 26, 2008@
@Beta_t = F * Beta_t-1 + w_t@
@where Beta_t = z_st, z_st-1, z_it, z_it-1, x_st, x_it@
@w_t = x_st, 0, x_it, 0, x_st, x_it@



f_= phi_s1_fnl~phi_s2_fnl~0~0~-1~0|
    1~0~0~0~0~0|
    0~0~phi_i1_fnl~phi_i2_fnl~0~-1|
    0~0~1~0~0~0|
    0~0~0~0~0~0|
    0~0~0~0~0~0;
q_= zeros(rows(f_),rows(f_));
q_[1,1]=lambda_s_eta_fnl^2*sig_eta_fnl^2 + sig_eps_fnl^2;
q_[5,5]=q_[1,1];
q_[1,5]=q_[1,1];
q_[5,1]=q_[1,1];
q_[3,3]=lambda_i_eta_fnl^2*sig_eta_fnl^2+lambda_i_v_fnl^2*sig_v_fnl^2+lambda_i_eps_fnl^2*sig_eps_fnl^2+sig_u_fnl^2;
q_[6,6]=q_[3,3];
q_[3,6]=q_[3,3];
q_[6,3]=q_[3,3];
q_[1,3]=lambda_s_eta_fnl*lambda_i_eta_fnl*sig_eta_fnl^2 + lambda_i_eps_fnl*sig_eps_fnl^2;
q_[3,1]=q_[1,3];
q_[1,6]=q_[1,3];
q_[6,1]=q_[1,3];
q_[3,5]=q_[1,3];
q_[5,3]=q_[1,3];
q_[5,6]=q_[1,3];
q_[6,5]=q_[1,3];

var_beta=reshape(inv(eye(6^2)-f_.*.f_)*vec(q_),6,6);


sig_di=sqrt((gamma_itao_fnl+lambda_k_eta_fnl)^2*sig_eta_fnl^2+lambda_k_eps_fnl^2*sig_eps_fnl^2+sig_v_fnl^2+var_beta[3,3]+
            2*(gamma_itao_fnl+lambda_k_eta_fnl)*lambda_i_eta_fnl*sig_eta_fnl^2+2*lambda_k_eps_fnl*lambda_i_eps_fnl*sig_eps_fnl^2+2*lambda_i_v_fnl*sig_v_fnl^2);

sig_ds = sqrt( sig_eta_fnl^2 + var_beta[1,1] + 2*lambda_s_eta_fnl*sig_eta_fnl^2);

cov_il=(phi_i1_fnl-1)*((gamma_itao_fnl+lambda_k_eta_fnl)*lambda_i_eta_fnl*sig_eta_fnl^2 + lambda_i_v_fnl*sig_v_fnl^2+lambda_i_eps_fnl*lambda_k_eps_fnl*sig_eps_fnl^2)
        +var_beta[3,4];

cov_is=sig_eta_fnl^2*(gamma_itao_fnl + lambda_k_eta_fnl +lambda_i_eta_fnl + (gamma_itao_fnl+lambda_k_eta_fnl)*lambda_s_eta_fnl) + lambda_k_eps_fnl*sig_eps_fnl^2
        +var_beta[1,3];

cov_ils=(phi_s1_fnl-1)*((gamma_itao_fnl+lambda_k_eta_fnl)*lambda_s_eta_fnl*sig_eta_fnl^2+lambda_k_eps_fnl*sig_eps_fnl^2) 
        +var_beta[1,4];


@see updated version of Nov. 26, 2008 document for following@
sig_dy = sqrt( 2*sig_di^2 +sig_ds^2 
                -2*cov_il + 2*cov_is - 2*cov_ils );

sig_ddi = sqrt( 2*sig_di^2 - 2*cov_il);

@see notes from Nov. 3, 2014 for the following@
cov_ddids=(1+lambda_k_eta_fnl+lambda_i_eta_fnl+lambda_s_eta_fnl*(1+lambda_k_eta_fnl)-(phi_s1_fnl -1)*lambda_s_eta_fnl*(1+lambda_k_eta_fnl))*sig_eta_fnl^2+
                                var_beta[1,3]-var_beta[1,4];		
cov_diuds=(lambda_y_eta_fnl-lambda_s_eta_fnl)*sig_eta_fnl^2+(lambda_y_eps_fnl-1)*sig_eps_fnl^2+lambda_s_eta_fnl*(lambda_y_eta_fnl-lambda_s_eta_fnl)*sig_eta_fnl^2;
corr_ddids=cov_ddids/(sig_ddi*sig_ds);
corr_diuds=cov_diuds/(sig_di_u*sig_ds);

corr_ddids_cf5=corr_ddids;
corr_diuds_cf5=corr_diuds;
      
sig_ddi_cf5=sig_ddi;
cov_il_cf5=cov_il;
cov_is_cf5=cov_is;
cov_ils_cf5=cov_ils;
sig_dy_cf5=sig_dy;
sig_ds_cf5=sig_ds;
dsig_dy_cf5=sig_dy-sig_dy_pre;
dsig_ds_cf5=sig_ds-sig_ds_pre;
sig_eta_cf5=sig_eta_fnl;
sig_v_cf5=sig_v_fnl;
sig_eps_cf5=sig_eps_fnl;
sig_di_cf5=sig_di;
sig_di_u_cf5=sig_di_u;
sig_di_p_cf5=sqrt(sig_di^2-sig_di_u^2);
ratio_cf5=sig_di_u/sig_di;


@********@

@*******************************************************************@
@6th Counterfactual Variances for change in INVENTORY shocks only@
@Start with Pre sample as benchmark@
@Write over some pre-parameters with post-parameters@
@E.g., only shock variances are overwritten below@

prm_fnl=prm_fnl_pre;

phi_s1_fnl=prm_fnl[1]+prm_fnl[2];
phi_s2_fnl=-prm_fnl[2];
phi_i1_fnl=prm_fnl[3]+prm_fnl[4];
phi_i2_fnl=-prm_fnl[4];
sig_eta_fnl=prm_fnl[5];
sig_eps_fnl=prm_fnl[6];
sig_v_fnl=prm_fnl[7];
sig_u_fnl=prm_fnl[8];
gamma_itao_fnl=1;
lambda_s_eta_fnl=prm_fnl[10];          
lambda_y_eta_fnl=prm_fnl[11];
lambda_y_eps_fnl=prm_fnl[12];         
lambda_i_v_fnl=  prm_fnl[13];      
lambda_k_eta_fnl=  prm_fnl[14];  
lambda_k_eps_fnl=  0;
lambda_i_eta_fnl=lambda_y_eta_fnl-lambda_s_eta_fnl - gamma_itao_fnl-lambda_k_eta_fnl;
lambda_i_eps_fnl=lambda_y_eps_fnl-1-lambda_k_eps_fnl;

prm_fnl=prm_fnl_post;

sig_v_fnl=prm_fnl[7];
sig_u_fnl=prm_fnl[8];    



sig_di_u=sqrt(((lambda_y_eta_fnl-lambda_s_eta_fnl)^2)*sig_eta_fnl^2+((lambda_y_eps_fnl-1)^2)*sig_eps_fnl^2
+((1+lambda_i_v_fnl)^2)*sig_v_fnl^2+sig_u_fnl^2);      @standard deviation of unplanned/unexpected inventories Feb 2009@


@Calculate standard deviation of change in output and of change in inventories@
@phi(L)*z_t = x_t - x_t-1@
@x_t = lambda_ieta*eta + lambda_iv*v + lambda_ieps*eps + u@
@z_t = phi(L)^-1*x_t - phi(L)^-1*x_t-1@
@see notes from Nov. 26, 2008@
@Beta_t = F * Beta_t-1 + w_t@
@where Beta_t = z_st, z_st-1, z_it, z_it-1, x_st, x_it@
@w_t = x_st, 0, x_it, 0, x_st, x_it@



f_= phi_s1_fnl~phi_s2_fnl~0~0~-1~0|
    1~0~0~0~0~0|
    0~0~phi_i1_fnl~phi_i2_fnl~0~-1|
    0~0~1~0~0~0|
    0~0~0~0~0~0|
    0~0~0~0~0~0;
q_= zeros(rows(f_),rows(f_));
q_[1,1]=lambda_s_eta_fnl^2*sig_eta_fnl^2 + sig_eps_fnl^2;
q_[5,5]=q_[1,1];
q_[1,5]=q_[1,1];
q_[5,1]=q_[1,1];
q_[3,3]=lambda_i_eta_fnl^2*sig_eta_fnl^2+lambda_i_v_fnl^2*sig_v_fnl^2+lambda_i_eps_fnl^2*sig_eps_fnl^2+sig_u_fnl^2;
q_[6,6]=q_[3,3];
q_[3,6]=q_[3,3];
q_[6,3]=q_[3,3];
q_[1,3]=lambda_s_eta_fnl*lambda_i_eta_fnl*sig_eta_fnl^2 + lambda_i_eps_fnl*sig_eps_fnl^2;
q_[3,1]=q_[1,3];
q_[1,6]=q_[1,3];
q_[6,1]=q_[1,3];
q_[3,5]=q_[1,3];
q_[5,3]=q_[1,3];
q_[5,6]=q_[1,3];
q_[6,5]=q_[1,3];

var_beta=reshape(inv(eye(6^2)-f_.*.f_)*vec(q_),6,6);


sig_di=sqrt((gamma_itao_fnl+lambda_k_eta_fnl)^2*sig_eta_fnl^2+lambda_k_eps_fnl^2*sig_eps_fnl^2+sig_v_fnl^2+var_beta[3,3]+
            2*(gamma_itao_fnl+lambda_k_eta_fnl)*lambda_i_eta_fnl*sig_eta_fnl^2+2*lambda_k_eps_fnl*lambda_i_eps_fnl*sig_eps_fnl^2+2*lambda_i_v_fnl*sig_v_fnl^2);

sig_ds = sqrt( sig_eta_fnl^2 + var_beta[1,1] + 2*lambda_s_eta_fnl*sig_eta_fnl^2);

cov_il=(phi_i1_fnl-1)*((gamma_itao_fnl+lambda_k_eta_fnl)*lambda_i_eta_fnl*sig_eta_fnl^2 + lambda_i_v_fnl*sig_v_fnl^2+lambda_i_eps_fnl*lambda_k_eps_fnl*sig_eps_fnl^2)
        +var_beta[3,4];

cov_is=sig_eta_fnl^2*(gamma_itao_fnl + lambda_k_eta_fnl +lambda_i_eta_fnl + (gamma_itao_fnl+lambda_k_eta_fnl)*lambda_s_eta_fnl) + lambda_k_eps_fnl*sig_eps_fnl^2
        +var_beta[1,3];

cov_ils=(phi_s1_fnl-1)*((gamma_itao_fnl+lambda_k_eta_fnl)*lambda_s_eta_fnl*sig_eta_fnl^2+lambda_k_eps_fnl*sig_eps_fnl^2) 
        +var_beta[1,4];


@see updated version of Nov. 26, 2008 document for following@
sig_dy = sqrt( 2*sig_di^2 +sig_ds^2 
                -2*cov_il + 2*cov_is - 2*cov_ils );

sig_ddi = sqrt( 2*sig_di^2 - 2*cov_il);


@see notes from Nov. 3, 2014 for the following@
cov_ddids=(1+lambda_k_eta_fnl+lambda_i_eta_fnl+lambda_s_eta_fnl*(1+lambda_k_eta_fnl)-(phi_s1_fnl -1)*lambda_s_eta_fnl*(1+lambda_k_eta_fnl))*sig_eta_fnl^2+
                                var_beta[1,3]-var_beta[1,4];		
cov_diuds=(lambda_y_eta_fnl-lambda_s_eta_fnl)*sig_eta_fnl^2+(lambda_y_eps_fnl-1)*sig_eps_fnl^2+lambda_s_eta_fnl*(lambda_y_eta_fnl-lambda_s_eta_fnl)*sig_eta_fnl^2;
corr_ddids=cov_ddids/(sig_ddi*sig_ds);
corr_diuds=cov_diuds/(sig_di_u*sig_ds);

corr_ddids_cf6=corr_ddids;
corr_diuds_cf6=corr_diuds;
      
sig_ddi_cf6=sig_ddi;
cov_il_cf6=cov_il;
cov_is_cf6=cov_is;
cov_ils_cf6=cov_ils;
sig_dy_cf6=sig_dy;
sig_ds_cf6=sig_ds;
dsig_dy_cf6=sig_dy-sig_dy_pre;
dsig_ds_cf6=sig_ds-sig_ds_pre;
sig_eta_cf6=sig_eta_fnl;
sig_v_cf6=sig_v_fnl;
sig_eps_cf6=sig_eps_fnl;
sig_di_cf6=sig_di;
sig_di_u_cf6=sig_di_u;
sig_di_p_cf6=sqrt(sig_di^2-sig_di_u^2);
ratio_cf6=sig_di_u/sig_di;


@********@

@*******************************************************************@
@7th Counterfactual Variances for change in INVENTORY propagation only@
@Start with Pre sample as benchmark@
@Write over some pre-parameters with post-parameters@
@E.g., only shock variances are overwritten below@

prm_fnl=prm_fnl_pre;

phi_s1_fnl=prm_fnl[1]+prm_fnl[2];
phi_s2_fnl=-prm_fnl[2];
phi_i1_fnl=prm_fnl[3]+prm_fnl[4];
phi_i2_fnl=-prm_fnl[4];
sig_eta_fnl=prm_fnl[5];
sig_eps_fnl=prm_fnl[6];
sig_v_fnl=prm_fnl[7];
sig_u_fnl=prm_fnl[8];
gamma_itao_fnl=1;
lambda_s_eta_fnl=prm_fnl[10];          
lambda_y_eta_fnl=prm_fnl[11];
lambda_y_eps_fnl=prm_fnl[12];         
lambda_i_v_fnl=  prm_fnl[13];      
lambda_k_eta_fnl=  prm_fnl[14];  
lambda_k_eps_fnl=  0;
lambda_i_eta_fnl=lambda_y_eta_fnl-lambda_s_eta_fnl - gamma_itao_fnl-lambda_k_eta_fnl;
lambda_i_eps_fnl=lambda_y_eps_fnl-1-lambda_k_eps_fnl;

prm_fnl=prm_fnl_post;


phi_i1_fnl=prm_fnl[3]+prm_fnl[4];
phi_i2_fnl=-prm_fnl[4];
gamma_itao_fnl=1;        
lambda_y_eta_fnl=prm_fnl[11];
lambda_y_eps_fnl=prm_fnl[12];         
lambda_i_v_fnl=  prm_fnl[13];      
lambda_k_eta_fnl=  prm_fnl[14];  
lambda_k_eps_fnl=  0;
lambda_i_eta_fnl=lambda_y_eta_fnl-lambda_s_eta_fnl - gamma_itao_fnl-lambda_k_eta_fnl;
lambda_i_eps_fnl=lambda_y_eps_fnl-1-lambda_k_eps_fnl; 


sig_di_u=sqrt(((lambda_y_eta_fnl-lambda_s_eta_fnl)^2)*sig_eta_fnl^2+((lambda_y_eps_fnl-1)^2)*sig_eps_fnl^2
+((1+lambda_i_v_fnl)^2)*sig_v_fnl^2+sig_u_fnl^2);      @standard deviation of unplanned/unexpected inventories Feb 2009@

@Calculate standard deviation of change in output and of change in inventories@
@phi(L)*z_t = x_t - x_t-1@
@x_t = lambda_ieta*eta + lambda_iv*v + lambda_ieps*eps + u@
@z_t = phi(L)^-1*x_t - phi(L)^-1*x_t-1@
@see notes from Nov. 26, 2008@
@Beta_t = F * Beta_t-1 + w_t@
@where Beta_t = z_st, z_st-1, z_it, z_it-1, x_st, x_it@
@w_t = x_st, 0, x_it, 0, x_st, x_it@



f_= phi_s1_fnl~phi_s2_fnl~0~0~-1~0|
    1~0~0~0~0~0|
    0~0~phi_i1_fnl~phi_i2_fnl~0~-1|
    0~0~1~0~0~0|
    0~0~0~0~0~0|
    0~0~0~0~0~0;
q_= zeros(rows(f_),rows(f_));
q_[1,1]=lambda_s_eta_fnl^2*sig_eta_fnl^2 + sig_eps_fnl^2;
q_[5,5]=q_[1,1];
q_[1,5]=q_[1,1];
q_[5,1]=q_[1,1];
q_[3,3]=lambda_i_eta_fnl^2*sig_eta_fnl^2+lambda_i_v_fnl^2*sig_v_fnl^2+lambda_i_eps_fnl^2*sig_eps_fnl^2+sig_u_fnl^2;
q_[6,6]=q_[3,3];
q_[3,6]=q_[3,3];
q_[6,3]=q_[3,3];
q_[1,3]=lambda_s_eta_fnl*lambda_i_eta_fnl*sig_eta_fnl^2 + lambda_i_eps_fnl*sig_eps_fnl^2;
q_[3,1]=q_[1,3];
q_[1,6]=q_[1,3];
q_[6,1]=q_[1,3];
q_[3,5]=q_[1,3];
q_[5,3]=q_[1,3];
q_[5,6]=q_[1,3];
q_[6,5]=q_[1,3];

var_beta=reshape(inv(eye(6^2)-f_.*.f_)*vec(q_),6,6);


sig_di=sqrt((gamma_itao_fnl+lambda_k_eta_fnl)^2*sig_eta_fnl^2+lambda_k_eps_fnl^2*sig_eps_fnl^2+sig_v_fnl^2+var_beta[3,3]+
            2*(gamma_itao_fnl+lambda_k_eta_fnl)*lambda_i_eta_fnl*sig_eta_fnl^2+2*lambda_k_eps_fnl*lambda_i_eps_fnl*sig_eps_fnl^2+2*lambda_i_v_fnl*sig_v_fnl^2);

sig_ds = sqrt( sig_eta_fnl^2 + var_beta[1,1] + 2*lambda_s_eta_fnl*sig_eta_fnl^2);

cov_il=(phi_i1_fnl-1)*((gamma_itao_fnl+lambda_k_eta_fnl)*lambda_i_eta_fnl*sig_eta_fnl^2 + lambda_i_v_fnl*sig_v_fnl^2+lambda_i_eps_fnl*lambda_k_eps_fnl*sig_eps_fnl^2)
        +var_beta[3,4];

cov_is=sig_eta_fnl^2*(gamma_itao_fnl + lambda_k_eta_fnl +lambda_i_eta_fnl + (gamma_itao_fnl+lambda_k_eta_fnl)*lambda_s_eta_fnl) + lambda_k_eps_fnl*sig_eps_fnl^2
        +var_beta[1,3];

cov_ils=(phi_s1_fnl-1)*((gamma_itao_fnl+lambda_k_eta_fnl)*lambda_s_eta_fnl*sig_eta_fnl^2+lambda_k_eps_fnl*sig_eps_fnl^2) 
        +var_beta[1,4];


@see updated version of Nov. 26, 2008 document for following@
sig_dy = sqrt( 2*sig_di^2 +sig_ds^2 
                -2*cov_il + 2*cov_is - 2*cov_ils );

sig_ddi = sqrt( 2*sig_di^2 - 2*cov_il);

@see notes from Nov. 3, 2014 for the following@
cov_ddids=(1+lambda_k_eta_fnl+lambda_i_eta_fnl+lambda_s_eta_fnl*(1+lambda_k_eta_fnl)-(phi_s1_fnl -1)*lambda_s_eta_fnl*(1+lambda_k_eta_fnl))*sig_eta_fnl^2+
                                var_beta[1,3]-var_beta[1,4];		
cov_diuds=(lambda_y_eta_fnl-lambda_s_eta_fnl)*sig_eta_fnl^2+(lambda_y_eps_fnl-1)*sig_eps_fnl^2+lambda_s_eta_fnl*(lambda_y_eta_fnl-lambda_s_eta_fnl)*sig_eta_fnl^2;
corr_ddids=cov_ddids/(sig_ddi*sig_ds);
corr_diuds=cov_diuds/(sig_di_u*sig_ds);

corr_ddids_cf7=corr_ddids;
corr_diuds_cf7=corr_diuds;
      
sig_ddi_cf7=sig_ddi;
cov_il_cf7=cov_il;
cov_is_cf7=cov_is;
cov_ils_cf7=cov_ils;
sig_dy_cf7=sig_dy;
sig_ds_cf7=sig_ds;
dsig_dy_cf7=sig_dy-sig_dy_pre;
dsig_ds_cf7=sig_ds-sig_ds_pre;
sig_eta_cf7=sig_eta_fnl;
sig_v_cf7=sig_v_fnl;
sig_eps_cf7=sig_eps_fnl;
sig_di_cf7=sig_di;
sig_di_u_cf7=sig_di_u;
sig_di_p_cf7=sqrt(sig_di^2-sig_di_u^2);
ratio_cf7=sig_di_u/sig_di;


@********@

@*******************************************************************@
@8th Counterfactual Variances for change in SALES process only@
@Start with Pre sample as benchmark@
@Write over some pre-parameters with post-parameters@


prm_fnl=prm_fnl_pre;

phi_s1_fnl=prm_fnl[1]+prm_fnl[2];
phi_s2_fnl=-prm_fnl[2];
phi_i1_fnl=prm_fnl[3]+prm_fnl[4];
phi_i2_fnl=-prm_fnl[4];
sig_eta_fnl=prm_fnl[5];
sig_eps_fnl=prm_fnl[6];
sig_v_fnl=prm_fnl[7];
sig_u_fnl=prm_fnl[8];
gamma_itao_fnl=1;
lambda_s_eta_fnl=prm_fnl[10];          
lambda_y_eta_fnl=prm_fnl[11];
lambda_y_eps_fnl=prm_fnl[12];         
lambda_i_v_fnl=  prm_fnl[13];      
lambda_k_eta_fnl=  prm_fnl[14];  
lambda_k_eps_fnl=  0;
lambda_i_eta_fnl=lambda_y_eta_fnl-lambda_s_eta_fnl - gamma_itao_fnl-lambda_k_eta_fnl;
lambda_i_eps_fnl=lambda_y_eps_fnl-1-lambda_k_eps_fnl;

prm_fnl=prm_fnl_post;

phi_s1_fnl=prm_fnl[1]+prm_fnl[2];
phi_s2_fnl=-prm_fnl[2];
sig_eta_fnl=prm_fnl[5];
sig_eps_fnl=prm_fnl[6];
lambda_s_eta_fnl=prm_fnl[10];            
lambda_i_eta_fnl=lambda_y_eta_fnl-lambda_s_eta_fnl - gamma_itao_fnl-lambda_k_eta_fnl;
lambda_i_eps_fnl=lambda_y_eps_fnl-1-lambda_k_eps_fnl;



sig_di_u=sqrt(((lambda_y_eta_fnl-lambda_s_eta_fnl)^2)*sig_eta_fnl^2+((lambda_y_eps_fnl-1)^2)*sig_eps_fnl^2
+((1+lambda_i_v_fnl)^2)*sig_v_fnl^2+sig_u_fnl^2);      @standard deviation of unplanned/unexpected inventories Feb 2009@


@Calculate standard deviation of change in output and of change in inventories@
@phi(L)*z_t = x_t - x_t-1@
@x_t = lambda_ieta*eta + lambda_iv*v + lambda_ieps*eps + u@
@z_t = phi(L)^-1*x_t - phi(L)^-1*x_t-1@
@see notes from Nov. 26, 2008@
@Beta_t = F * Beta_t-1 + w_t@
@where Beta_t = z_st, z_st-1, z_it, z_it-1, x_st, x_it@
@w_t = x_st, 0, x_it, 0, x_st, x_it@



f_= phi_s1_fnl~phi_s2_fnl~0~0~-1~0|
    1~0~0~0~0~0|
    0~0~phi_i1_fnl~phi_i2_fnl~0~-1|
    0~0~1~0~0~0|
    0~0~0~0~0~0|
    0~0~0~0~0~0;
q_= zeros(rows(f_),rows(f_));
q_[1,1]=lambda_s_eta_fnl^2*sig_eta_fnl^2 + sig_eps_fnl^2;
q_[5,5]=q_[1,1];
q_[1,5]=q_[1,1];
q_[5,1]=q_[1,1];
q_[3,3]=lambda_i_eta_fnl^2*sig_eta_fnl^2+lambda_i_v_fnl^2*sig_v_fnl^2+lambda_i_eps_fnl^2*sig_eps_fnl^2+sig_u_fnl^2;
q_[6,6]=q_[3,3];
q_[3,6]=q_[3,3];
q_[6,3]=q_[3,3];
q_[1,3]=lambda_s_eta_fnl*lambda_i_eta_fnl*sig_eta_fnl^2 + lambda_i_eps_fnl*sig_eps_fnl^2;
q_[3,1]=q_[1,3];
q_[1,6]=q_[1,3];
q_[6,1]=q_[1,3];
q_[3,5]=q_[1,3];
q_[5,3]=q_[1,3];
q_[5,6]=q_[1,3];
q_[6,5]=q_[1,3];

var_beta=reshape(inv(eye(6^2)-f_.*.f_)*vec(q_),6,6);


sig_di=sqrt((gamma_itao_fnl+lambda_k_eta_fnl)^2*sig_eta_fnl^2+lambda_k_eps_fnl^2*sig_eps_fnl^2+sig_v_fnl^2+var_beta[3,3]+
            2*(gamma_itao_fnl+lambda_k_eta_fnl)*lambda_i_eta_fnl*sig_eta_fnl^2+2*lambda_k_eps_fnl*lambda_i_eps_fnl*sig_eps_fnl^2+2*lambda_i_v_fnl*sig_v_fnl^2);

sig_ds = sqrt( sig_eta_fnl^2 + var_beta[1,1] + 2*lambda_s_eta_fnl*sig_eta_fnl^2);

cov_il=(phi_i1_fnl-1)*((gamma_itao_fnl+lambda_k_eta_fnl)*lambda_i_eta_fnl*sig_eta_fnl^2 + lambda_i_v_fnl*sig_v_fnl^2+lambda_i_eps_fnl*lambda_k_eps_fnl*sig_eps_fnl^2)
        +var_beta[3,4];

cov_is=sig_eta_fnl^2*(gamma_itao_fnl + lambda_k_eta_fnl +lambda_i_eta_fnl + (gamma_itao_fnl+lambda_k_eta_fnl)*lambda_s_eta_fnl) + lambda_k_eps_fnl*sig_eps_fnl^2
        +var_beta[1,3];

cov_ils=(phi_s1_fnl-1)*((gamma_itao_fnl+lambda_k_eta_fnl)*lambda_s_eta_fnl*sig_eta_fnl^2+lambda_k_eps_fnl*sig_eps_fnl^2) 
        +var_beta[1,4];


@see updated version of Nov. 26, 2008 document for following@
sig_dy = sqrt( 2*sig_di^2 +sig_ds^2 
                -2*cov_il + 2*cov_is - 2*cov_ils );

sig_ddi = sqrt( 2*sig_di^2 - 2*cov_il);


@see notes from Nov. 3, 2014 for the following@
cov_ddids=(1+lambda_k_eta_fnl+lambda_i_eta_fnl+lambda_s_eta_fnl*(1+lambda_k_eta_fnl)-(phi_s1_fnl -1)*lambda_s_eta_fnl*(1+lambda_k_eta_fnl))*sig_eta_fnl^2+
                                var_beta[1,3]-var_beta[1,4];		
cov_diuds=(lambda_y_eta_fnl-lambda_s_eta_fnl)*sig_eta_fnl^2+(lambda_y_eps_fnl-1)*sig_eps_fnl^2+lambda_s_eta_fnl*(lambda_y_eta_fnl-lambda_s_eta_fnl)*sig_eta_fnl^2;
corr_ddids=cov_ddids/(sig_ddi*sig_ds);
corr_diuds=cov_diuds/(sig_di_u*sig_ds);

corr_ddids_cf8=corr_ddids;
corr_diuds_cf8=corr_diuds;
      
sig_ddi_cf8=sig_ddi;
cov_il_cf8=cov_il;
cov_is_cf8=cov_is;
cov_ils_cf8=cov_ils;
sig_dy_cf8=sig_dy;
sig_ds_cf8=sig_ds;
dsig_dy_cf8=sig_dy-sig_dy_pre;
dsig_ds_cf8=sig_ds-sig_ds_pre;
sig_eta_cf8=sig_eta_fnl;
sig_v_cf8=sig_v_fnl;
sig_eps_cf8=sig_eps_fnl;
sig_di_cf8=sig_di;
sig_di_u_cf8=sig_di_u;
sig_di_p_cf8=sqrt(sig_di^2-sig_di_u^2);
ratio_cf8=sig_di_u/sig_di;


@********@
@*******************************************************************@
@9th Counterfactual Variances for change in "u" only@
@Start with Pre sample as benchmark@
@Write over some pre-parameters with post-parameters@


prm_fnl=prm_fnl_pre;

phi_s1_fnl=prm_fnl[1]+prm_fnl[2];
phi_s2_fnl=-prm_fnl[2];
phi_i1_fnl=prm_fnl[3]+prm_fnl[4];
phi_i2_fnl=-prm_fnl[4];
sig_eta_fnl=prm_fnl[5];
sig_eps_fnl=prm_fnl[6];
sig_v_fnl=prm_fnl[7];
sig_u_fnl=prm_fnl[8];
gamma_itao_fnl=1;
lambda_s_eta_fnl=prm_fnl[10];          
lambda_y_eta_fnl=prm_fnl[11];
lambda_y_eps_fnl=prm_fnl[12];         
lambda_i_v_fnl=  prm_fnl[13];      
lambda_k_eta_fnl=  prm_fnl[14];  
lambda_k_eps_fnl=  0;
lambda_i_eta_fnl=lambda_y_eta_fnl-lambda_s_eta_fnl - gamma_itao_fnl-lambda_k_eta_fnl;
lambda_i_eps_fnl=lambda_y_eps_fnl-1-lambda_k_eps_fnl; 


prm_fnl=prm_fnl_post;

sig_u_fnl=prm_fnl[8];


sig_di_u=sqrt(((lambda_y_eta_fnl-lambda_s_eta_fnl)^2)*sig_eta_fnl^2+((lambda_y_eps_fnl-1)^2)*sig_eps_fnl^2
+((1+lambda_i_v_fnl)^2)*sig_v_fnl^2+sig_u_fnl^2);      @standard deviation of unplanned/unexpected inventories Feb 2009@

@Calculate standard deviation of change in output and of change in inventories@
@phi(L)*z_t = x_t - x_t-1@
@x_t = lambda_ieta*eta + lambda_iv*v + lambda_ieps*eps + u@
@z_t = phi(L)^-1*x_t - phi(L)^-1*x_t-1@
@see notes from Nov. 26, 2008@
@Beta_t = F * Beta_t-1 + w_t@
@where Beta_t = z_st, z_st-1, z_it, z_it-1, x_st, x_it@
@w_t = x_st, 0, x_it, 0, x_st, x_it@



f_= phi_s1_fnl~phi_s2_fnl~0~0~-1~0|
    1~0~0~0~0~0|
    0~0~phi_i1_fnl~phi_i2_fnl~0~-1|
    0~0~1~0~0~0|
    0~0~0~0~0~0|
    0~0~0~0~0~0;
q_= zeros(rows(f_),rows(f_));
q_[1,1]=lambda_s_eta_fnl^2*sig_eta_fnl^2 + sig_eps_fnl^2;
q_[5,5]=q_[1,1];
q_[1,5]=q_[1,1];
q_[5,1]=q_[1,1];
q_[3,3]=lambda_i_eta_fnl^2*sig_eta_fnl^2+lambda_i_v_fnl^2*sig_v_fnl^2+lambda_i_eps_fnl^2*sig_eps_fnl^2+sig_u_fnl^2;
q_[6,6]=q_[3,3];
q_[3,6]=q_[3,3];
q_[6,3]=q_[3,3];
q_[1,3]=lambda_s_eta_fnl*lambda_i_eta_fnl*sig_eta_fnl^2 + lambda_i_eps_fnl*sig_eps_fnl^2;
q_[3,1]=q_[1,3];
q_[1,6]=q_[1,3];
q_[6,1]=q_[1,3];
q_[3,5]=q_[1,3];
q_[5,3]=q_[1,3];
q_[5,6]=q_[1,3];
q_[6,5]=q_[1,3];

var_beta=reshape(inv(eye(6^2)-f_.*.f_)*vec(q_),6,6);


sig_di=sqrt((gamma_itao_fnl+lambda_k_eta_fnl)^2*sig_eta_fnl^2+lambda_k_eps_fnl^2*sig_eps_fnl^2+sig_v_fnl^2+var_beta[3,3]+
            2*(gamma_itao_fnl+lambda_k_eta_fnl)*lambda_i_eta_fnl*sig_eta_fnl^2+2*lambda_k_eps_fnl*lambda_i_eps_fnl*sig_eps_fnl^2+2*lambda_i_v_fnl*sig_v_fnl^2);

sig_ds = sqrt( sig_eta_fnl^2 + var_beta[1,1] + 2*lambda_s_eta_fnl*sig_eta_fnl^2);

cov_il=(phi_i1_fnl-1)*((gamma_itao_fnl+lambda_k_eta_fnl)*lambda_i_eta_fnl*sig_eta_fnl^2 + lambda_i_v_fnl*sig_v_fnl^2+lambda_i_eps_fnl*lambda_k_eps_fnl*sig_eps_fnl^2)
        +var_beta[3,4];

cov_is=sig_eta_fnl^2*(gamma_itao_fnl + lambda_k_eta_fnl +lambda_i_eta_fnl + (gamma_itao_fnl+lambda_k_eta_fnl)*lambda_s_eta_fnl) + lambda_k_eps_fnl*sig_eps_fnl^2
        +var_beta[1,3];

cov_ils=(phi_s1_fnl-1)*((gamma_itao_fnl+lambda_k_eta_fnl)*lambda_s_eta_fnl*sig_eta_fnl^2+lambda_k_eps_fnl*sig_eps_fnl^2) 
        +var_beta[1,4];


@see updated version of Nov. 26, 2008 document for following@
sig_dy = sqrt( 2*sig_di^2 +sig_ds^2 
                -2*cov_il + 2*cov_is - 2*cov_ils );

sig_ddi = sqrt( 2*sig_di^2 - 2*cov_il);

@see notes from Nov. 3, 2014 for the following@
cov_ddids=(1+lambda_k_eta_fnl+lambda_i_eta_fnl+lambda_s_eta_fnl*(1+lambda_k_eta_fnl)-(phi_s1_fnl -1)*lambda_s_eta_fnl*(1+lambda_k_eta_fnl))*sig_eta_fnl^2+
                                var_beta[1,3]-var_beta[1,4];		
cov_diuds=(lambda_y_eta_fnl-lambda_s_eta_fnl)*sig_eta_fnl^2+(lambda_y_eps_fnl-1)*sig_eps_fnl^2+lambda_s_eta_fnl*(lambda_y_eta_fnl-lambda_s_eta_fnl)*sig_eta_fnl^2;
corr_ddids=cov_ddids/(sig_ddi*sig_ds);
corr_diuds=cov_diuds/(sig_di_u*sig_ds);

corr_ddids_cf9=corr_ddids;
corr_diuds_cf9=corr_diuds;
      
sig_ddi_cf9=sig_ddi;
cov_il_cf9=cov_il;
cov_is_cf9=cov_is;
cov_ils_cf9=cov_ils;
sig_dy_cf9=sig_dy;
sig_ds_cf9=sig_ds;
dsig_dy_cf9=sig_dy-sig_dy_pre;
dsig_ds_cf9=sig_ds-sig_ds_pre;
sig_eta_cf9=sig_eta_fnl;
sig_v_cf9=sig_v_fnl;
sig_eps_cf9=sig_eps_fnl;
sig_di_cf9=sig_di;
sig_di_u_cf9=sig_di_u;
sig_di_p_cf9=sqrt(sig_di^2-sig_di_u^2);
ratio_cf9=sig_di_u/sig_di;


@********@

@***************************************************@

sig_dy_premm=sig_dy_premm~sig_dy_pre;
sig_dy_postmm=sig_dy_postmm~sig_dy_post;
dsig_dy_postmm=dsig_dy_postmm~sig_dy_post-sig_dy_pre;
sig_ds_premm=sig_ds_premm~sig_ds_pre;
sig_ds_postmm=sig_ds_postmm~sig_ds_post;
dsig_ds_postmm=dsig_ds_postmm~sig_ds_post-sig_ds_pre;
sig_ddi_premm=sig_ddi_premm~sig_ddi_pre;
sig_ddi_postmm=sig_ddi_postmm~sig_ddi_post;
sig_di_u_premm=sig_di_u_premm~sig_di_u_pre;
sig_di_u_postmm=sig_di_u_postmm~sig_di_u_post;
sig_di_p_premm=sig_di_p_premm~sig_di_p_pre;
sig_di_p_postmm=sig_di_p_postmm~sig_di_p_post;
sig_dy_cf5mm=sig_dy_cf5mm~sig_dy_cf5;
sig_ds_cf5mm=sig_ds_cf5mm~sig_ds_cf5;
sig_dy_cf6mm=sig_dy_cf6mm~sig_dy_cf6;
sig_ds_cf6mm=sig_ds_cf6mm~sig_ds_cf6;
sig_dy_cf7mm=sig_dy_cf7mm~sig_dy_cf7;
sig_ds_cf7mm=sig_ds_cf7mm~sig_ds_cf7;
sig_dy_cf8mm=sig_dy_cf8mm~sig_dy_cf8;
sig_ds_cf8mm=sig_ds_cf8mm~sig_ds_cf8;
sig_dy_cf9mm=sig_dy_cf9mm~sig_dy_cf9;
sig_ds_cf9mm=sig_ds_cf9mm~sig_ds_cf9;
dsig_dy_cf5mm=dsig_dy_cf5mm~dsig_dy_cf5;
dsig_ds_cf5mm=dsig_ds_cf5mm~dsig_ds_cf5;
dsig_dy_cf6mm=dsig_dy_cf6mm~dsig_dy_cf6;
dsig_ds_cf6mm=dsig_ds_cf6mm~dsig_ds_cf6;
dsig_dy_cf7mm=dsig_dy_cf7mm~dsig_dy_cf7;
dsig_ds_cf7mm=dsig_ds_cf7mm~dsig_ds_cf7;
dsig_dy_cf8mm=dsig_dy_cf8mm~dsig_dy_cf8;
dsig_ds_cf8mm=dsig_ds_cf8mm~dsig_ds_cf8;
dsig_dy_cf9mm=dsig_dy_cf9mm~dsig_dy_cf9;
dsig_ds_cf9mm=dsig_ds_cf9mm~dsig_ds_cf9;

ratio_eta_Di_u_premm=ratio_eta_Di_u_premm~ratio_eta_Di_u_pre;
ratio_v_Di_u_premm=ratio_v_Di_u_premm~ratio_v_Di_u_pre;
ratio_eps_Di_u_premm=ratio_eps_Di_u_premm~ratio_eps_Di_u_pre;
ratio_u_Di_u_premm=ratio_u_Di_u_premm~ratio_u_Di_u_pre;
alpha_y_eta_premm=alpha_y_eta_premm~alpha_y_eta_pre;
alpha_y_v_premm=alpha_y_v_premm~alpha_y_v_pre;
alpha_y_eps_premm=alpha_y_eps_premm~alpha_y_eps_pre;
alpha_y_u_premm=alpha_y_u_premm~alpha_y_u_pre;
alpha_y_premm=alpha_y_premm~(ratio_eta_Di_u_pre*alpha_y_eta_pre + ratio_v_Di_u_pre*alpha_y_v_pre + ratio_eps_Di_u_pre*alpha_y_eps_pre + ratio_u_Di_u_pre*alpha_y_u_pre);
alpha_s_eta_premm=alpha_s_eta_premm~alpha_s_eta_pre;
alpha_s_v_premm=alpha_s_v_premm~alpha_s_v_pre;
alpha_s_eps_premm=alpha_s_eps_premm~alpha_s_eps_pre;
alpha_s_u_premm=alpha_s_u_premm~alpha_s_u_pre;
alpha_s_premm=alpha_s_premm~(ratio_eta_Di_u_pre*alpha_s_eta_pre + ratio_v_Di_u_pre*alpha_s_v_pre + ratio_eps_Di_u_pre*alpha_s_eps_pre + ratio_u_Di_u_pre*alpha_s_u_pre);

ratio_eta_Di_u_postmm=ratio_eta_Di_u_postmm~ratio_eta_Di_u_post;
ratio_v_Di_u_postmm=ratio_v_Di_u_postmm~ratio_v_Di_u_post;
ratio_eps_Di_u_postmm=ratio_eps_Di_u_postmm~ratio_eps_Di_u_post;
ratio_u_Di_u_postmm=ratio_u_Di_u_postmm~ratio_u_Di_u_post;
alpha_y_eta_postmm=alpha_y_eta_postmm~alpha_y_eta_post;
alpha_y_v_postmm=alpha_y_v_postmm~alpha_y_v_post;
alpha_y_eps_postmm=alpha_y_eps_postmm~alpha_y_eps_post;
alpha_y_u_postmm=alpha_y_u_postmm~alpha_y_u_post;
alpha_y_postmm=alpha_y_postmm~(ratio_eta_Di_u_post*alpha_y_eta_post + ratio_v_Di_u_post*alpha_y_v_post + ratio_eps_Di_u_post*alpha_y_eps_post + ratio_u_Di_u_post*alpha_y_u_post);
alpha_s_eta_postmm=alpha_s_eta_postmm~alpha_s_eta_post;
alpha_s_v_postmm=alpha_s_v_postmm~alpha_s_v_post;
alpha_s_eps_postmm=alpha_s_eps_postmm~alpha_s_eps_post;
alpha_s_u_postmm=alpha_s_u_postmm~alpha_s_u_post;
alpha_s_postmm=alpha_s_postmm~(ratio_eta_Di_u_post*alpha_s_eta_post + ratio_v_Di_u_post*alpha_s_v_post + ratio_eps_Di_u_post*alpha_s_eps_post + ratio_u_Di_u_post*alpha_s_u_post);

sig_di_premm=sig_di_premm~sig_di_pre;
sig_di_postmm=sig_di_postmm~sig_di_post;
    

corr_ddids_premm=corr_ddids_premm~corr_ddids_pre;
corr_ddids_postmm=corr_ddids_postmm~corr_ddids_post;
corr_diuds_premm=corr_diuds_premm~corr_diuds_pre;
corr_diuds_postmm=corr_diuds_postmm~corr_diuds_post;


endif;

ITR=ITR+1;
endo;

dsig_etamm=thetamm[19,.]-thetamm[5,.];
dsig_epsmm=thetamm[20,.]-thetamm[6,.];
dphi_smm=thetamm[15,.]-thetamm[1,.];
dlambda_s_etamm=thetamm[24,.]-thetamm[10,.];
dsig_vmm=thetamm[21,.]-thetamm[7,.];
dsig_umm=thetamm[22,.]-thetamm[8,.];
dphi_imm=thetamm[17,.]-thetamm[3,.];
dmu_kmm=thetamm[23,.]-thetamm[9,.];
dlambda_y_etamm=thetamm[25,.]-thetamm[11,.];
dlambda_y_epsmm=thetamm[26,.]-thetamm[12,.];
dlambda_i_vmm=thetamm[27,.]-thetamm[13,.];
dlambda_k_etamm=thetamm[28,.]-thetamm[14,.];
dlambda_k_epsmm=zeros(1,cols(thetamm));

dsig_dymm=sig_dy_postmm-sig_dy_premm;
dsig_dsmm=sig_ds_postmm-sig_ds_premm;
dsig_ddimm=sig_ddi_postmm-sig_ddi_premm;
dsig_dimm=sig_di_postmm-sig_di_premm;
dsig_di_umm=sig_di_u_postmm-sig_di_u_premm;
dsig_di_pmm=sig_di_p_postmm-sig_di_p_premm;


dalpha_ymm=alpha_y_postmm-alpha_y_premm;
dalpha_smm=alpha_s_postmm-alpha_s_premm;

dcorr_ddidsmm=corr_ddids_postmm-corr_ddids_premm;
dcorr_diudsmm=corr_diuds_postmm-corr_diuds_premm;

@Store posterior draws@
    fnl_mat=thetamm'~zeros(2,cols(thetamm))'~
sig_dy_premm'~
sig_dy_postmm'~
sig_ds_premm'~
sig_ds_postmm'~
sig_ddi_premm'~
sig_ddi_postmm'~
sig_di_u_premm'~
sig_di_u_postmm'~
sig_di_p_premm'~
sig_di_p_postmm'~
sig_dy_cf5mm'~
sig_ds_cf5mm'~
sig_dy_cf6mm'~
sig_ds_cf6mm'~
sig_dy_cf7mm'~
sig_ds_cf7mm'~
sig_dy_cf8mm'~
sig_ds_cf8mm'~
sig_dy_cf9mm'~
sig_ds_cf9mm'~   @31-50@

dsig_dy_cf5mm'~
dsig_ds_cf5mm'~
dsig_dy_cf6mm'~
dsig_ds_cf6mm'~
dsig_dy_cf7mm'~
dsig_ds_cf7mm'~
dsig_dy_cf8mm'~
dsig_ds_cf8mm'~
dsig_dy_cf9mm'~
dsig_ds_cf9mm'~  @51-60@

ratio_eta_Di_u_premm'~
ratio_v_Di_u_premm'~
ratio_eps_Di_u_premm'~
ratio_u_Di_u_premm'~
alpha_y_eta_premm'~
alpha_y_v_premm'~
alpha_y_eps_premm'~
alpha_y_u_premm'~
alpha_y_premm'~
alpha_s_eta_premm'~
alpha_s_v_premm'~
alpha_s_eps_premm'~
alpha_s_u_premm'~
alpha_s_premm'~  @61-74@

ratio_eta_Di_u_postmm'~
ratio_v_Di_u_postmm'~
ratio_eps_Di_u_postmm'~
ratio_u_Di_u_postmm'~
alpha_y_eta_postmm'~
alpha_y_v_postmm'~
alpha_y_eps_postmm'~
alpha_y_u_postmm'~
alpha_y_postmm'~
alpha_s_eta_postmm'~
alpha_s_v_postmm'~
alpha_s_eps_postmm'~
alpha_s_u_postmm'~
alpha_s_postmm'~
dsig_dy_postmm'~
dsig_ds_postmm'~  @75-90@

dsig_etamm'~
dsig_epsmm'~
dphi_smm'~
dlambda_s_etamm'~
dsig_vmm'~
dsig_umm'~
dphi_imm'~
dmu_kmm'~
dlambda_y_etamm'~
dlambda_y_epsmm'~
dlambda_i_vmm'~
dlambda_k_etamm'~
dlambda_k_epsmm'~  @91-103@

sig_di_premm'~ 
sig_di_postmm'~
dsig_dymm'~
dsig_dsmm'~
dsig_ddimm'~
dsig_dimm'~
dsig_di_umm'~
dsig_di_pmm'~  @104-111@

dalpha_ymm'~
dalpha_smm'~ @112-113@

corr_ddids_premm'~
corr_ddids_postmm'~
dcorr_ddidsmm'~
corr_diuds_premm'~
corr_diuds_postmm'~
dcorr_diudsmm';	@114-119@

  





@Generate Output@
       I_NDX=1;
         nnn=rows(fnl_mat);

         sort_out={};


          do until I_NDX>COLS(FNL_MAT);
          
         corr_out=0;
         var_out=0;
         cov_out=0;
         corr5_out=0;
         var5_out=0;
         cov5_out=0;
         corr10_out=0;
         var10_out=0;
         cov10_out=0;
         corr20_out=0;
         var20_out=0;
         cov20_out=0;

          tmpm1=fnl_mat[.,i_ndx];
          mn_out=meanc(tmpm1);
          std_out=stdc(tmpm1);
          med_out=median(tmpm1);
          
          jjj=2;
          do until jjj>nnn;
          cov_out=cov_out+(tmpm1[jjj]-mn_out)*(tmpm1[jjj-1]-mn_out);
          var_out=var_out+(tmpm1[jjj]-mn_out)*(tmpm1[jjj]-mn_out);
          jjj=jjj+1;
          endo;
          corr_out=cov_out/var_out;  @calculates correlation of draws@
          
          jjj=6;
          do until jjj>nnn;
          cov5_out=cov5_out+(tmpm1[jjj]-mn_out)*(tmpm1[jjj-5]-mn_out);
          var5_out=var5_out+(tmpm1[jjj]-mn_out)*(tmpm1[jjj]-mn_out);
          jjj=jjj+1;
          endo;
          corr5_out=cov5_out/var5_out;  @calculates correlation of draws@
          
          jjj=11;
          do until jjj>nnn;
          cov10_out=cov10_out+(tmpm1[jjj]-mn_out)*(tmpm1[jjj-10]-mn_out);
          var10_out=var10_out+(tmpm1[jjj]-mn_out)*(tmpm1[jjj]-mn_out);
          jjj=jjj+1;
          endo;
          corr10_out=cov10_out/var10_out;  @calculates correlation of draws@
          
          jjj=21;
          do until jjj>nnn;
          cov20_out=cov20_out+(tmpm1[jjj]-mn_out)*(tmpm1[jjj-20]-mn_out);
          var20_out=var20_out+(tmpm1[jjj]-mn_out)*(tmpm1[jjj]-mn_out);
          jjj=jjj+1;
          endo;
          corr20_out=cov20_out/var20_out;  @calculates correlation of draws@
                   
          tmpm2=sortc(tmpm1,1);
          up_0=tmpm2[ceil(0.975*rows(tmpm1))];
          low_0=tmpm2[ceil(0.025*rows(tmpm1))];
          
          sort_out=sort_out|(mn_out~std_out~low_0~med_out~up_0~corr_out~corr5_out~corr10_out~corr20_out);
          
          i_ndx=i_ndx+1;
          endo;


cls;
output file = output_mh/prmtr.out reset;
output on;
"Size of Burn in ";;N0;
"Number of Draws after Burn in ";;MM;

"";
 "   Parameters = ";
""; 

"mean ~ std dev  ~ 2.5th Percentile ~ median ~ 97.5th Percentile ";

    SORT_OUT[.,1:5];

"";
"";

"correlation between draws(1, 5, 10, 20 draws apart)";
    SORT_OUT[.,6:9];
output off;















@CALCULATE VARIANCES OF UNOBSERVED COMPONENTS, ETC...@
prm_fnl_all=prm_fnl_;    @pre and post parametrs@
prm_fnl_all[5:8 19:22]=sqrt(prm_fnl_all[5:8 19:22]);

prm_fnl_pre=prm_fnl_all[1:14];    @parameters before the break@
prm_fnl_post=prm_fnl_pre;  @start with the same parameters. Then alter for parameters that undergo a break@
prm_fnl_post[1:14]=prm_fnl_all[15:28];  
                          


prm_fnl=prm_fnl_pre;

phi_s1_fnl=prm_fnl[1]+prm_fnl[2];
phi_s2_fnl=-prm_fnl[2];
phi_i1_fnl=prm_fnl[3]+prm_fnl[4];
phi_i2_fnl=-prm_fnl[4];
sig_eta_fnl=prm_fnl[5];
sig_eps_fnl=prm_fnl[6];
sig_v_fnl=prm_fnl[7];
sig_u_fnl=prm_fnl[8];
gamma_itao_fnl=1;
lambda_s_eta_fnl=prm_fnl[10];          
lambda_y_eta_fnl=prm_fnl[11];
lambda_y_eps_fnl=prm_fnl[12];         
lambda_i_v_fnl=  prm_fnl[13];      
lambda_k_eta_fnl=  prm_fnl[14];  
lambda_k_eps_fnl=  0;
lambda_i_eta_fnl=lambda_y_eta_fnl-lambda_s_eta_fnl - gamma_itao_fnl-lambda_k_eta_fnl;
lambda_i_eps_fnl=lambda_y_eps_fnl-1-lambda_k_eps_fnl;


sig_di_u=sqrt(((lambda_y_eta_fnl-lambda_s_eta_fnl)^2)*sig_eta_fnl^2+((lambda_y_eps_fnl-1)^2)*sig_eps_fnl^2
+((1+lambda_i_v_fnl)^2)*sig_v_fnl^2+sig_u_fnl^2);      @standard deviation of unplanned/unexpected inventories Feb 2009@




@Calculate standard deviation of change in output and of change in inventories@
@phi(L)*z_t = x_t - x_t-1@
@x_t = lambda_ieta*eta + lambda_iv*v + lambda_ieps*eps + u@
@z_t = phi(L)^-1*x_t - phi(L)^-1*x_t-1@
@see notes from Nov. 26, 2008@
@Beta_t = F * Beta_t-1 + w_t@
@where Beta_t = z_st, z_st-1, z_it, z_it-1, x_st, x_it@
@w_t = x_st, 0, x_it, 0, x_st, x_it@



f_= phi_s1_fnl~phi_s2_fnl~0~0~-1~0|
    1~0~0~0~0~0|
    0~0~phi_i1_fnl~phi_i2_fnl~0~-1|
    0~0~1~0~0~0|
    0~0~0~0~0~0|
    0~0~0~0~0~0;
q_= zeros(rows(f_),rows(f_));
q_[1,1]=lambda_s_eta_fnl^2*sig_eta_fnl^2 + sig_eps_fnl^2;
q_[5,5]=q_[1,1];
q_[1,5]=q_[1,1];
q_[5,1]=q_[1,1];
q_[3,3]=lambda_i_eta_fnl^2*sig_eta_fnl^2+lambda_i_v_fnl^2*sig_v_fnl^2+lambda_i_eps_fnl^2*sig_eps_fnl^2+sig_u_fnl^2;
q_[6,6]=q_[3,3];
q_[3,6]=q_[3,3];
q_[6,3]=q_[3,3];
q_[1,3]=lambda_s_eta_fnl*lambda_i_eta_fnl*sig_eta_fnl^2 + lambda_i_eps_fnl*sig_eps_fnl^2;
q_[3,1]=q_[1,3];
q_[1,6]=q_[1,3];
q_[6,1]=q_[1,3];
q_[3,5]=q_[1,3];
q_[5,3]=q_[1,3];
q_[5,6]=q_[1,3];
q_[6,5]=q_[1,3];

var_beta=reshape(inv(eye(6^2)-f_.*.f_)*vec(q_),6,6);


sig_di=sqrt((gamma_itao_fnl+lambda_k_eta_fnl)^2*sig_eta_fnl^2+lambda_k_eps_fnl^2*sig_eps_fnl^2+sig_v_fnl^2+var_beta[3,3]+
            2*(gamma_itao_fnl+lambda_k_eta_fnl)*lambda_i_eta_fnl*sig_eta_fnl^2+2*lambda_k_eps_fnl*lambda_i_eps_fnl*sig_eps_fnl^2+2*lambda_i_v_fnl*sig_v_fnl^2);

sig_ds = sqrt( sig_eta_fnl^2 + var_beta[1,1] + 2*lambda_s_eta_fnl*sig_eta_fnl^2);

cov_il=(phi_i1_fnl-1)*((gamma_itao_fnl+lambda_k_eta_fnl)*lambda_i_eta_fnl*sig_eta_fnl^2 + lambda_i_v_fnl*sig_v_fnl^2+lambda_i_eps_fnl*lambda_k_eps_fnl*sig_eps_fnl^2)
        +var_beta[3,4];

cov_is=sig_eta_fnl^2*(gamma_itao_fnl + lambda_k_eta_fnl +lambda_i_eta_fnl + (gamma_itao_fnl+lambda_k_eta_fnl)*lambda_s_eta_fnl) + lambda_k_eps_fnl*sig_eps_fnl^2
        +var_beta[1,3];

cov_ils=(phi_s1_fnl-1)*((gamma_itao_fnl+lambda_k_eta_fnl)*lambda_s_eta_fnl*sig_eta_fnl^2+lambda_k_eps_fnl*sig_eps_fnl^2) 
        +var_beta[1,4];


@see updated version of Nov. 26, 2008 document for following@
sig_dy = sqrt( 2*sig_di^2 +sig_ds^2 
                -2*cov_il + 2*cov_is - 2*cov_ils );

sig_ddi = sqrt( 2*sig_di^2 - 2*cov_il);

@see notes from Nov. 3, 2014 for the following@
cov_ddids=(1+lambda_k_eta_fnl+lambda_i_eta_fnl+lambda_s_eta_fnl*(1+lambda_k_eta_fnl)-(phi_s1_fnl -1)*lambda_s_eta_fnl*(1+lambda_k_eta_fnl))*sig_eta_fnl^2+
                                var_beta[1,3]-var_beta[1,4];		
cov_diuds=(lambda_y_eta_fnl-lambda_s_eta_fnl)*sig_eta_fnl^2+(lambda_y_eps_fnl-1)*sig_eps_fnl^2+lambda_s_eta_fnl*(lambda_y_eta_fnl-lambda_s_eta_fnl)*sig_eta_fnl^2;
corr_ddids=cov_ddids/(sig_ddi*sig_ds);
corr_diuds=cov_diuds/(sig_di_u*sig_ds);

corr_ddids_pre=corr_ddids;
corr_diuds_pre=corr_diuds;
      
sig_ddi_pre=sig_ddi;
cov_il_pre=cov_il;
cov_is_pre=cov_is;
cov_ils_pre=cov_ils;
sig_dy_pre=sig_dy;
sig_ds_pre=sig_ds;
sig_eta_pre=sig_eta_fnl;
sig_v_pre=sig_v_fnl;
sig_eps_pre=sig_eps_fnl;
sig_di_pre=sig_di;
sig_di_u_pre=sig_di_u;
sig_di_p_pre=sqrt(sig_di^2-sig_di_u^2);
ratio_pre=sig_di_u/sig_di;

sig_di_u_eta_pre=sqrt(((lambda_y_eta_fnl-lambda_s_eta_fnl)^2)*sig_eta_fnl^2);
sig_di_u_eps_pre=sqrt(((lambda_y_eps_fnl-1)^2)*sig_eps_fnl^2);
sig_di_u_v_pre=sqrt(((1+lambda_i_v_fnl)^2)*sig_v_fnl^2);

ratio_eta_di_u_pre=sig_di_u_eta_pre^2/(sig_di_u^2);
ratio_eps_di_u_pre=sig_di_u_eps_pre^2/(sig_di_u^2);
ratio_v_di_u_pre=sig_di_u_v_pre^2/(sig_di_u^2);
ratio_u_di_u_pre=sig_u_fnl^2/(sig_di_u^2);

alpha_y_eta_pre=((phi_s1_fnl-1)*lambda_s_eta_fnl+(phi_i1_fnl-2)*lambda_i_eta_fnl-gamma_itao_fnl-lambda_k_eta_fnl)/(gamma_itao_fnl+lambda_i_eta_fnl+lambda_k_eta_fnl);
alpha_y_eps_pre=((phi_s1_fnl-1)+(phi_i1_fnl-2)*lambda_i_eps_fnl)/(lambda_i_eps_fnl+lambda_k_eps_fnl);
alpha_y_v_pre=((phi_i1_fnl-2)*lambda_i_v_fnl-1)/(1+lambda_i_v_fnl);
alpha_y_u_pre=(phi_i1_fnl-2);

alpha_s_eta_pre=((phi_s1_fnl-1)*lambda_s_eta_fnl)/(gamma_itao_fnl+lambda_i_eta_fnl+lambda_k_eta_fnl);
alpha_s_eps_pre=((phi_s1_fnl-1))/(lambda_i_eps_fnl+lambda_k_eps_fnl);
alpha_s_v_pre=0;
alpha_s_u_pre=0;


prm_fnl=prm_fnl_post;

phi_s1_fnl=prm_fnl[1]+prm_fnl[2];
phi_s2_fnl=-prm_fnl[2];
phi_i1_fnl=prm_fnl[3]+prm_fnl[4];
phi_i2_fnl=-prm_fnl[4];
sig_eta_fnl=prm_fnl[5];
sig_eps_fnl=prm_fnl[6];
sig_v_fnl=prm_fnl[7];
sig_u_fnl=prm_fnl[8];
gamma_itao_fnl=1;
lambda_s_eta_fnl=prm_fnl[10];          
lambda_y_eta_fnl=prm_fnl[11];
lambda_y_eps_fnl=prm_fnl[12];         
lambda_i_v_fnl=  prm_fnl[13];      
lambda_k_eta_fnl=  prm_fnl[14];  
lambda_k_eps_fnl=  0;
lambda_i_eta_fnl=lambda_y_eta_fnl-lambda_s_eta_fnl - gamma_itao_fnl-lambda_k_eta_fnl;
lambda_i_eps_fnl=lambda_y_eps_fnl-1-lambda_k_eps_fnl;


sig_di_u=sqrt(((lambda_y_eta_fnl-lambda_s_eta_fnl)^2)*sig_eta_fnl^2+((lambda_y_eps_fnl-1)^2)*sig_eps_fnl^2
+((1+lambda_i_v_fnl)^2)*sig_v_fnl^2+sig_u_fnl^2);      @standard deviation of unplanned/unexpected inventories Feb 2009@

@Calculate standard deviation of change in output and of change in inventories@
@phi(L)*z_t = x_t - x_t-1@
@x_t = lambda_ieta*eta + lambda_iv*v + lambda_ieps*eps + u@
@z_t = phi(L)^-1*x_t - phi(L)^-1*x_t-1@
@see notes from Nov. 26, 2008@
@Beta_t = F * Beta_t-1 + w_t@
@where Beta_t = z_st, z_st-1, z_it, z_it-1, x_st, x_it@
@w_t = x_st, 0, x_it, 0, x_st, x_it@



f_= phi_s1_fnl~phi_s2_fnl~0~0~-1~0|
    1~0~0~0~0~0|
    0~0~phi_i1_fnl~phi_i2_fnl~0~-1|
    0~0~1~0~0~0|
    0~0~0~0~0~0|
    0~0~0~0~0~0;
q_= zeros(rows(f_),rows(f_));
q_[1,1]=lambda_s_eta_fnl^2*sig_eta_fnl^2 + sig_eps_fnl^2;
q_[5,5]=q_[1,1];
q_[1,5]=q_[1,1];
q_[5,1]=q_[1,1];
q_[3,3]=lambda_i_eta_fnl^2*sig_eta_fnl^2+lambda_i_v_fnl^2*sig_v_fnl^2+lambda_i_eps_fnl^2*sig_eps_fnl^2+sig_u_fnl^2;
q_[6,6]=q_[3,3];
q_[3,6]=q_[3,3];
q_[6,3]=q_[3,3];
q_[1,3]=lambda_s_eta_fnl*lambda_i_eta_fnl*sig_eta_fnl^2 + lambda_i_eps_fnl*sig_eps_fnl^2;
q_[3,1]=q_[1,3];
q_[1,6]=q_[1,3];
q_[6,1]=q_[1,3];
q_[3,5]=q_[1,3];
q_[5,3]=q_[1,3];
q_[5,6]=q_[1,3];
q_[6,5]=q_[1,3];

var_beta=reshape(inv(eye(6^2)-f_.*.f_)*vec(q_),6,6);


sig_di=sqrt((gamma_itao_fnl+lambda_k_eta_fnl)^2*sig_eta_fnl^2+lambda_k_eps_fnl^2*sig_eps_fnl^2+sig_v_fnl^2+var_beta[3,3]+
            2*(gamma_itao_fnl+lambda_k_eta_fnl)*lambda_i_eta_fnl*sig_eta_fnl^2+2*lambda_k_eps_fnl*lambda_i_eps_fnl*sig_eps_fnl^2+2*lambda_i_v_fnl*sig_v_fnl^2);

sig_ds = sqrt( sig_eta_fnl^2 + var_beta[1,1] + 2*lambda_s_eta_fnl*sig_eta_fnl^2);

cov_il=(phi_i1_fnl-1)*((gamma_itao_fnl+lambda_k_eta_fnl)*lambda_i_eta_fnl*sig_eta_fnl^2 + lambda_i_v_fnl*sig_v_fnl^2+lambda_i_eps_fnl*lambda_k_eps_fnl*sig_eps_fnl^2)
        +var_beta[3,4];

cov_is=sig_eta_fnl^2*(gamma_itao_fnl + lambda_k_eta_fnl +lambda_i_eta_fnl + (gamma_itao_fnl+lambda_k_eta_fnl)*lambda_s_eta_fnl) + lambda_k_eps_fnl*sig_eps_fnl^2
        +var_beta[1,3];

cov_ils=(phi_s1_fnl-1)*((gamma_itao_fnl+lambda_k_eta_fnl)*lambda_s_eta_fnl*sig_eta_fnl^2+lambda_k_eps_fnl*sig_eps_fnl^2) 
        +var_beta[1,4];


@see updated version of Nov. 26, 2008 document for following@
sig_dy = sqrt( 2*sig_di^2 +sig_ds^2 
                -2*cov_il + 2*cov_is - 2*cov_ils );

sig_ddi = sqrt( 2*sig_di^2 - 2*cov_il);


@see notes from Nov. 3, 2014 for the following@
cov_ddids=(1+lambda_k_eta_fnl+lambda_i_eta_fnl+lambda_s_eta_fnl*(1+lambda_k_eta_fnl)-(phi_s1_fnl -1)*lambda_s_eta_fnl*(1+lambda_k_eta_fnl))*sig_eta_fnl^2+
                                var_beta[1,3]-var_beta[1,4];		
cov_diuds=(lambda_y_eta_fnl-lambda_s_eta_fnl)*sig_eta_fnl^2+(lambda_y_eps_fnl-1)*sig_eps_fnl^2+lambda_s_eta_fnl*(lambda_y_eta_fnl-lambda_s_eta_fnl)*sig_eta_fnl^2;
corr_ddids=cov_ddids/(sig_ddi*sig_ds);
corr_diuds=cov_diuds/(sig_di_u*sig_ds);

corr_ddids_post=corr_ddids;
corr_diuds_post=corr_diuds;
      
sig_ddi_post=sig_ddi;
cov_il_post=cov_il;
cov_is_post=cov_is;
cov_ils_post=cov_ils;
sig_dy_post=sig_dy;
sig_ds_post=sig_ds;
sig_eta_post=sig_eta_fnl;
sig_v_post=sig_v_fnl;
sig_eps_post=sig_eps_fnl;
sig_di_post=sig_di;
sig_di_u_post=sig_di_u;
sig_di_p_post=sqrt(sig_di^2-sig_di_u^2);
ratio_post=sig_di_u/sig_di;

sig_di_u_eta_post=sqrt(((lambda_y_eta_fnl-lambda_s_eta_fnl)^2)*sig_eta_fnl^2);
sig_di_u_eps_post=sqrt(((lambda_y_eps_fnl-1)^2)*sig_eps_fnl^2);
sig_di_u_v_post=sqrt(((1+lambda_i_v_fnl)^2)*sig_v_fnl^2);

ratio_eta_di_u_post=sig_di_u_eta_post^2/(sig_di_u^2);
ratio_eps_di_u_post=sig_di_u_eps_post^2/(sig_di_u^2);
ratio_v_di_u_post=sig_di_u_v_post^2/(sig_di_u^2);
ratio_u_di_u_post=sig_u_fnl^2/(sig_di_u^2);

alpha_y_eta_post=((phi_s1_fnl-1)*lambda_s_eta_fnl+(phi_i1_fnl-2)*lambda_i_eta_fnl-gamma_itao_fnl-lambda_k_eta_fnl)/(gamma_itao_fnl+lambda_i_eta_fnl+lambda_k_eta_fnl);
alpha_y_eps_post=((phi_s1_fnl-1)+(phi_i1_fnl-2)*lambda_i_eps_fnl)/(lambda_i_eps_fnl+lambda_k_eps_fnl);
alpha_y_v_post=((phi_i1_fnl-2)*lambda_i_v_fnl-1)/(1+lambda_i_v_fnl);
alpha_y_u_post=(phi_i1_fnl-2);

alpha_s_eta_post=((phi_s1_fnl-1)*lambda_s_eta_fnl)/(gamma_itao_fnl+lambda_i_eta_fnl+lambda_k_eta_fnl);
alpha_s_eps_post=((phi_s1_fnl-1))/(lambda_i_eps_fnl+lambda_k_eps_fnl);
alpha_s_v_post=0;
alpha_s_u_post=0;






@*******************************************************************@
@Counterfactual Variances for change in shocks only@
@Start with Pre sample as benchmark@
@Write over some pre-parameters with post-parameters@
@E.g., only shock variances are overwritten below@

prm_fnl=prm_fnl_pre;

phi_s1_fnl=prm_fnl[1]+prm_fnl[2];
phi_s2_fnl=-prm_fnl[2];
phi_i1_fnl=prm_fnl[3]+prm_fnl[4];
phi_i2_fnl=-prm_fnl[4];
sig_eta_fnl=prm_fnl[5];
sig_eps_fnl=prm_fnl[6];
sig_v_fnl=prm_fnl[7];
sig_u_fnl=prm_fnl[8];
gamma_itao_fnl=1;
lambda_s_eta_fnl=prm_fnl[10];          
lambda_y_eta_fnl=prm_fnl[11];
lambda_y_eps_fnl=prm_fnl[12];         
lambda_i_v_fnl=  prm_fnl[13];      
lambda_k_eta_fnl=  prm_fnl[14];  
lambda_k_eps_fnl=  0;
lambda_i_eta_fnl=lambda_y_eta_fnl-lambda_s_eta_fnl - gamma_itao_fnl-lambda_k_eta_fnl;
lambda_i_eps_fnl=lambda_y_eps_fnl-1-lambda_k_eps_fnl;


prm_fnl=prm_fnl_post;


sig_eta_fnl=prm_fnl[5];
sig_eps_fnl=prm_fnl[6];
sig_v_fnl=prm_fnl[7];
sig_u_fnl=prm_fnl[8];


sig_di_u=sqrt(((lambda_y_eta_fnl-lambda_s_eta_fnl)^2)*sig_eta_fnl^2+((lambda_y_eps_fnl-1)^2)*sig_eps_fnl^2
+((1+lambda_i_v_fnl)^2)*sig_v_fnl^2+sig_u_fnl^2);      @standard deviation of unplanned/unexpected inventories Feb 2009@

@Calculate standard deviation of change in output and of change in inventories@
@phi(L)*z_t = x_t - x_t-1@
@x_t = lambda_ieta*eta + lambda_iv*v + lambda_ieps*eps + u@
@z_t = phi(L)^-1*x_t - phi(L)^-1*x_t-1@
@see notes from Nov. 26, 2008@
@Beta_t = F * Beta_t-1 + w_t@
@where Beta_t = z_st, z_st-1, z_it, z_it-1, x_st, x_it@
@w_t = x_st, 0, x_it, 0, x_st, x_it@



f_= phi_s1_fnl~phi_s2_fnl~0~0~-1~0|
    1~0~0~0~0~0|
    0~0~phi_i1_fnl~phi_i2_fnl~0~-1|
    0~0~1~0~0~0|
    0~0~0~0~0~0|
    0~0~0~0~0~0;
q_= zeros(rows(f_),rows(f_));
q_[1,1]=lambda_s_eta_fnl^2*sig_eta_fnl^2 + sig_eps_fnl^2;
q_[5,5]=q_[1,1];
q_[1,5]=q_[1,1];
q_[5,1]=q_[1,1];
q_[3,3]=lambda_i_eta_fnl^2*sig_eta_fnl^2+lambda_i_v_fnl^2*sig_v_fnl^2+lambda_i_eps_fnl^2*sig_eps_fnl^2+sig_u_fnl^2;
q_[6,6]=q_[3,3];
q_[3,6]=q_[3,3];
q_[6,3]=q_[3,3];
q_[1,3]=lambda_s_eta_fnl*lambda_i_eta_fnl*sig_eta_fnl^2 + lambda_i_eps_fnl*sig_eps_fnl^2;
q_[3,1]=q_[1,3];
q_[1,6]=q_[1,3];
q_[6,1]=q_[1,3];
q_[3,5]=q_[1,3];
q_[5,3]=q_[1,3];
q_[5,6]=q_[1,3];
q_[6,5]=q_[1,3];

var_beta=reshape(inv(eye(6^2)-f_.*.f_)*vec(q_),6,6);


sig_di=sqrt((gamma_itao_fnl+lambda_k_eta_fnl)^2*sig_eta_fnl^2+lambda_k_eps_fnl^2*sig_eps_fnl^2+sig_v_fnl^2+var_beta[3,3]+
            2*(gamma_itao_fnl+lambda_k_eta_fnl)*lambda_i_eta_fnl*sig_eta_fnl^2+2*lambda_k_eps_fnl*lambda_i_eps_fnl*sig_eps_fnl^2+2*lambda_i_v_fnl*sig_v_fnl^2);

sig_ds = sqrt( sig_eta_fnl^2 + var_beta[1,1] + 2*lambda_s_eta_fnl*sig_eta_fnl^2);

cov_il=(phi_i1_fnl-1)*((gamma_itao_fnl+lambda_k_eta_fnl)*lambda_i_eta_fnl*sig_eta_fnl^2 + lambda_i_v_fnl*sig_v_fnl^2+lambda_i_eps_fnl*lambda_k_eps_fnl*sig_eps_fnl^2)
        +var_beta[3,4];

cov_is=sig_eta_fnl^2*(gamma_itao_fnl + lambda_k_eta_fnl +lambda_i_eta_fnl + (gamma_itao_fnl+lambda_k_eta_fnl)*lambda_s_eta_fnl) + lambda_k_eps_fnl*sig_eps_fnl^2
        +var_beta[1,3];

cov_ils=(phi_s1_fnl-1)*((gamma_itao_fnl+lambda_k_eta_fnl)*lambda_s_eta_fnl*sig_eta_fnl^2+lambda_k_eps_fnl*sig_eps_fnl^2) 
        +var_beta[1,4];


@see updated version of Nov. 26, 2008 document for following@
sig_dy = sqrt( 2*sig_di^2 +sig_ds^2 
                -2*cov_il + 2*cov_is - 2*cov_ils );

sig_ddi = sqrt( 2*sig_di^2 - 2*cov_il);

      
sig_ddi_cf1=sig_ddi;
cov_il_cf1=cov_il;
cov_is_cf1=cov_is;
cov_ils_cf1=cov_ils;
sig_dy_cf1=sig_dy;
sig_ds_cf1=sig_ds;
sig_eta_cf1=sig_eta_fnl;
sig_v_cf1=sig_v_fnl;
sig_eps_cf1=sig_eps_fnl;
sig_di_cf1=sig_di;
sig_di_u_cf1=sig_di_u;
sig_di_p_cf1=sqrt(sig_di^2-sig_di_u^2);
ratio_cf1=sig_di_u/sig_di;


@********@

@Counterfactual Variances for change in AR and impact coefficients@
@Start with Pre sample as benchmark@
@Write over some pre-parameters with post-parameters@
@E.g., everything but shock variances are overwritten below@

prm_fnl=prm_fnl_pre;

phi_s1_fnl=prm_fnl[1]+prm_fnl[2];
phi_s2_fnl=-prm_fnl[2];
phi_i1_fnl=prm_fnl[3]+prm_fnl[4];
phi_i2_fnl=-prm_fnl[4];
sig_eta_fnl=prm_fnl[5];
sig_eps_fnl=prm_fnl[6];
sig_v_fnl=prm_fnl[7];
sig_u_fnl=prm_fnl[8];
gamma_itao_fnl=1;
lambda_s_eta_fnl=prm_fnl[10];          
lambda_y_eta_fnl=prm_fnl[11];
lambda_y_eps_fnl=prm_fnl[12];         
lambda_i_v_fnl=  prm_fnl[13];      
lambda_k_eta_fnl=  prm_fnl[14];  
lambda_k_eps_fnl=  0;
lambda_i_eta_fnl=lambda_y_eta_fnl-lambda_s_eta_fnl - gamma_itao_fnl-lambda_k_eta_fnl;
lambda_i_eps_fnl=lambda_y_eps_fnl-1-lambda_k_eps_fnl;

prm_fnl=prm_fnl_post;

phi_s1_fnl=prm_fnl[1]+prm_fnl[2];
phi_s2_fnl=-prm_fnl[2];
phi_i1_fnl=prm_fnl[3]+prm_fnl[4];
phi_i2_fnl=-prm_fnl[4];

gamma_itao_fnl=1;
lambda_s_eta_fnl=prm_fnl[10];          
lambda_y_eta_fnl=prm_fnl[11];
lambda_y_eps_fnl=prm_fnl[12];         
lambda_i_v_fnl=  prm_fnl[13];      
lambda_k_eta_fnl=  prm_fnl[14];  
lambda_k_eps_fnl=  0;
lambda_i_eta_fnl=lambda_y_eta_fnl-lambda_s_eta_fnl - gamma_itao_fnl-lambda_k_eta_fnl;
lambda_i_eps_fnl=lambda_y_eps_fnl-1-lambda_k_eps_fnl;



sig_di_u=sqrt(((lambda_y_eta_fnl-lambda_s_eta_fnl)^2)*sig_eta_fnl^2+((lambda_y_eps_fnl-1)^2)*sig_eps_fnl^2
+((1+lambda_i_v_fnl)^2)*sig_v_fnl^2+sig_u_fnl^2);      @standard deviation of unplanned/unexpected inventories Feb 2009@


@Calculate standard deviation of change in output and of change in inventories@
@phi(L)*z_t = x_t - x_t-1@
@x_t = lambda_ieta*eta + lambda_iv*v + lambda_ieps*eps + u@
@z_t = phi(L)^-1*x_t - phi(L)^-1*x_t-1@
@see notes from Nov. 26, 2008@
@Beta_t = F * Beta_t-1 + w_t@
@where Beta_t = z_st, z_st-1, z_it, z_it-1, x_st, x_it@
@w_t = x_st, 0, x_it, 0, x_st, x_it@



f_= phi_s1_fnl~phi_s2_fnl~0~0~-1~0|
    1~0~0~0~0~0|
    0~0~phi_i1_fnl~phi_i2_fnl~0~-1|
    0~0~1~0~0~0|
    0~0~0~0~0~0|
    0~0~0~0~0~0;
q_= zeros(rows(f_),rows(f_));
q_[1,1]=lambda_s_eta_fnl^2*sig_eta_fnl^2 + sig_eps_fnl^2;
q_[5,5]=q_[1,1];
q_[1,5]=q_[1,1];
q_[5,1]=q_[1,1];
q_[3,3]=lambda_i_eta_fnl^2*sig_eta_fnl^2+lambda_i_v_fnl^2*sig_v_fnl^2+lambda_i_eps_fnl^2*sig_eps_fnl^2+sig_u_fnl^2;
q_[6,6]=q_[3,3];
q_[3,6]=q_[3,3];
q_[6,3]=q_[3,3];
q_[1,3]=lambda_s_eta_fnl*lambda_i_eta_fnl*sig_eta_fnl^2 + lambda_i_eps_fnl*sig_eps_fnl^2;
q_[3,1]=q_[1,3];
q_[1,6]=q_[1,3];
q_[6,1]=q_[1,3];
q_[3,5]=q_[1,3];
q_[5,3]=q_[1,3];
q_[5,6]=q_[1,3];
q_[6,5]=q_[1,3];

var_beta=reshape(inv(eye(6^2)-f_.*.f_)*vec(q_),6,6);


sig_di=sqrt((gamma_itao_fnl+lambda_k_eta_fnl)^2*sig_eta_fnl^2+lambda_k_eps_fnl^2*sig_eps_fnl^2+sig_v_fnl^2+var_beta[3,3]+
            2*(gamma_itao_fnl+lambda_k_eta_fnl)*lambda_i_eta_fnl*sig_eta_fnl^2+2*lambda_k_eps_fnl*lambda_i_eps_fnl*sig_eps_fnl^2+2*lambda_i_v_fnl*sig_v_fnl^2);

sig_ds = sqrt( sig_eta_fnl^2 + var_beta[1,1] + 2*lambda_s_eta_fnl*sig_eta_fnl^2);

cov_il=(phi_i1_fnl-1)*((gamma_itao_fnl+lambda_k_eta_fnl)*lambda_i_eta_fnl*sig_eta_fnl^2 + lambda_i_v_fnl*sig_v_fnl^2+lambda_i_eps_fnl*lambda_k_eps_fnl*sig_eps_fnl^2)
        +var_beta[3,4];

cov_is=sig_eta_fnl^2*(gamma_itao_fnl + lambda_k_eta_fnl +lambda_i_eta_fnl + (gamma_itao_fnl+lambda_k_eta_fnl)*lambda_s_eta_fnl) + lambda_k_eps_fnl*sig_eps_fnl^2
        +var_beta[1,3];

cov_ils=(phi_s1_fnl-1)*((gamma_itao_fnl+lambda_k_eta_fnl)*lambda_s_eta_fnl*sig_eta_fnl^2+lambda_k_eps_fnl*sig_eps_fnl^2) 
        +var_beta[1,4];


@see updated version of Nov. 26, 2008 document for following@
sig_dy = sqrt( 2*sig_di^2 +sig_ds^2 
                -2*cov_il + 2*cov_is - 2*cov_ils );

sig_ddi = sqrt( 2*sig_di^2 - 2*cov_il);

      
sig_ddi_cf2=sig_ddi;
cov_il_cf2=cov_il;
cov_is_cf2=cov_is;
cov_ils_cf2=cov_ils;
sig_dy_cf2=sig_dy;
sig_ds_cf2=sig_ds;
sig_eta_cf2=sig_eta_fnl;
sig_v_cf2=sig_v_fnl;
sig_eps_cf2=sig_eps_fnl;
sig_di_cf2=sig_di;
sig_di_u_cf2=sig_di_u;
sig_di_p_cf2=sqrt(sig_di^2-sig_di_u^2);
ratio_cf2=sig_di_u/sig_di;



@***********@
@Counterfactual Variances for change in AR only@
@Start with Pre sample as benchmark@
@Write over some pre-parameters with post-parameters@
@E.g., only AR coeffs are overwritten below@

prm_fnl=prm_fnl_pre;

phi_s1_fnl=prm_fnl[1]+prm_fnl[2];
phi_s2_fnl=-prm_fnl[2];
phi_i1_fnl=prm_fnl[3]+prm_fnl[4];
phi_i2_fnl=-prm_fnl[4];
sig_eta_fnl=prm_fnl[5];
sig_eps_fnl=prm_fnl[6];
sig_v_fnl=prm_fnl[7];
sig_u_fnl=prm_fnl[8];
gamma_itao_fnl=1;
lambda_s_eta_fnl=prm_fnl[10];          
lambda_y_eta_fnl=prm_fnl[11];
lambda_y_eps_fnl=prm_fnl[12];         
lambda_i_v_fnl=  prm_fnl[13];      
lambda_k_eta_fnl=  prm_fnl[14];  
lambda_k_eps_fnl=  0;
lambda_i_eta_fnl=lambda_y_eta_fnl-lambda_s_eta_fnl - gamma_itao_fnl-lambda_k_eta_fnl;
lambda_i_eps_fnl=lambda_y_eps_fnl-1-lambda_k_eps_fnl;

prm_fnl=prm_fnl_post;

phi_s1_fnl=prm_fnl[1]+prm_fnl[2];
phi_s2_fnl=-prm_fnl[2];
phi_i1_fnl=prm_fnl[3]+prm_fnl[4];
phi_i2_fnl=-prm_fnl[4];    




sig_di_u=sqrt(((lambda_y_eta_fnl-lambda_s_eta_fnl)^2)*sig_eta_fnl^2+((lambda_y_eps_fnl-1)^2)*sig_eps_fnl^2
+((1+lambda_i_v_fnl)^2)*sig_v_fnl^2+sig_u_fnl^2);      @standard deviation of unplanned/unexpected inventories Feb 2009@


@Calculate standard deviation of change in output and of change in inventories@
@phi(L)*z_t = x_t - x_t-1@
@x_t = lambda_ieta*eta + lambda_iv*v + lambda_ieps*eps + u@
@z_t = phi(L)^-1*x_t - phi(L)^-1*x_t-1@
@see notes from Nov. 26, 2008@
@Beta_t = F * Beta_t-1 + w_t@
@where Beta_t = z_st, z_st-1, z_it, z_it-1, x_st, x_it@
@w_t = x_st, 0, x_it, 0, x_st, x_it@



f_= phi_s1_fnl~phi_s2_fnl~0~0~-1~0|
    1~0~0~0~0~0|
    0~0~phi_i1_fnl~phi_i2_fnl~0~-1|
    0~0~1~0~0~0|
    0~0~0~0~0~0|
    0~0~0~0~0~0;
q_= zeros(rows(f_),rows(f_));
q_[1,1]=lambda_s_eta_fnl^2*sig_eta_fnl^2 + sig_eps_fnl^2;
q_[5,5]=q_[1,1];
q_[1,5]=q_[1,1];
q_[5,1]=q_[1,1];
q_[3,3]=lambda_i_eta_fnl^2*sig_eta_fnl^2+lambda_i_v_fnl^2*sig_v_fnl^2+lambda_i_eps_fnl^2*sig_eps_fnl^2+sig_u_fnl^2;
q_[6,6]=q_[3,3];
q_[3,6]=q_[3,3];
q_[6,3]=q_[3,3];
q_[1,3]=lambda_s_eta_fnl*lambda_i_eta_fnl*sig_eta_fnl^2 + lambda_i_eps_fnl*sig_eps_fnl^2;
q_[3,1]=q_[1,3];
q_[1,6]=q_[1,3];
q_[6,1]=q_[1,3];
q_[3,5]=q_[1,3];
q_[5,3]=q_[1,3];
q_[5,6]=q_[1,3];
q_[6,5]=q_[1,3];

var_beta=reshape(inv(eye(6^2)-f_.*.f_)*vec(q_),6,6);


sig_di=sqrt((gamma_itao_fnl+lambda_k_eta_fnl)^2*sig_eta_fnl^2+lambda_k_eps_fnl^2*sig_eps_fnl^2+sig_v_fnl^2+var_beta[3,3]+
            2*(gamma_itao_fnl+lambda_k_eta_fnl)*lambda_i_eta_fnl*sig_eta_fnl^2+2*lambda_k_eps_fnl*lambda_i_eps_fnl*sig_eps_fnl^2+2*lambda_i_v_fnl*sig_v_fnl^2);

sig_ds = sqrt( sig_eta_fnl^2 + var_beta[1,1] + 2*lambda_s_eta_fnl*sig_eta_fnl^2);

cov_il=(phi_i1_fnl-1)*((gamma_itao_fnl+lambda_k_eta_fnl)*lambda_i_eta_fnl*sig_eta_fnl^2 + lambda_i_v_fnl*sig_v_fnl^2+lambda_i_eps_fnl*lambda_k_eps_fnl*sig_eps_fnl^2)
        +var_beta[3,4];

cov_is=sig_eta_fnl^2*(gamma_itao_fnl + lambda_k_eta_fnl +lambda_i_eta_fnl + (gamma_itao_fnl+lambda_k_eta_fnl)*lambda_s_eta_fnl) + lambda_k_eps_fnl*sig_eps_fnl^2
        +var_beta[1,3];

cov_ils=(phi_s1_fnl-1)*((gamma_itao_fnl+lambda_k_eta_fnl)*lambda_s_eta_fnl*sig_eta_fnl^2+lambda_k_eps_fnl*sig_eps_fnl^2) 
        +var_beta[1,4];


@see updated version of Nov. 26, 2008 document for following@
sig_dy = sqrt( 2*sig_di^2 +sig_ds^2 
                -2*cov_il + 2*cov_is - 2*cov_ils );

sig_ddi = sqrt( 2*sig_di^2 - 2*cov_il);

      
sig_ddi_cf3=sig_ddi;
cov_il_cf3=cov_il;
cov_is_cf3=cov_is;
cov_ils_cf3=cov_ils;
sig_dy_cf3=sig_dy;
sig_ds_cf3=sig_ds;
sig_eta_cf3=sig_eta_fnl;
sig_v_cf3=sig_v_fnl;
sig_eps_cf3=sig_eps_fnl;
sig_di_cf3=sig_di;
sig_di_u_cf3=sig_di_u;
sig_di_p_cf3=sqrt(sig_di^2-sig_di_u^2);
ratio_cf3=sig_di_u/sig_di;


@***********@
@Counterfactual Variances for change in impact coeffs only@
@Start with Pre sample as benchmark@
@Write over some pre-parameters with post-parameters@
@E.g., only impact coeffs are overwritten below@

prm_fnl=prm_fnl_pre;

phi_s1_fnl=prm_fnl[1]+prm_fnl[2];
phi_s2_fnl=-prm_fnl[2];
phi_i1_fnl=prm_fnl[3]+prm_fnl[4];
phi_i2_fnl=-prm_fnl[4];
sig_eta_fnl=prm_fnl[5];
sig_eps_fnl=prm_fnl[6];
sig_v_fnl=prm_fnl[7];
sig_u_fnl=prm_fnl[8];
gamma_itao_fnl=1;
lambda_s_eta_fnl=prm_fnl[10];          
lambda_y_eta_fnl=prm_fnl[11];
lambda_y_eps_fnl=prm_fnl[12];         
lambda_i_v_fnl=  prm_fnl[13];      
lambda_k_eta_fnl=  prm_fnl[14];  
lambda_k_eps_fnl=  0;
lambda_i_eta_fnl=lambda_y_eta_fnl-lambda_s_eta_fnl - gamma_itao_fnl-lambda_k_eta_fnl;
lambda_i_eps_fnl=lambda_y_eps_fnl-1-lambda_k_eps_fnl;

prm_fnl=prm_fnl_post;


gamma_itao_fnl=1;
lambda_s_eta_fnl=prm_fnl[10];          
lambda_y_eta_fnl=prm_fnl[11];
lambda_y_eps_fnl=prm_fnl[12];         
lambda_i_v_fnl=  prm_fnl[13];      
lambda_k_eta_fnl=  prm_fnl[14];  
lambda_k_eps_fnl=  0;
lambda_i_eta_fnl=lambda_y_eta_fnl-lambda_s_eta_fnl - gamma_itao_fnl-lambda_k_eta_fnl;
lambda_i_eps_fnl=lambda_y_eps_fnl-1-lambda_k_eps_fnl;


sig_di_u=sqrt(((lambda_y_eta_fnl-lambda_s_eta_fnl)^2)*sig_eta_fnl^2+((lambda_y_eps_fnl-1)^2)*sig_eps_fnl^2
+((1+lambda_i_v_fnl)^2)*sig_v_fnl^2+sig_u_fnl^2);      @standard deviation of unplanned/unexpected inventories Feb 2009@


@Calculate standard deviation of change in output and of change in inventories@
@phi(L)*z_t = x_t - x_t-1@
@x_t = lambda_ieta*eta + lambda_iv*v + lambda_ieps*eps + u@
@z_t = phi(L)^-1*x_t - phi(L)^-1*x_t-1@
@see notes from Nov. 26, 2008@
@Beta_t = F * Beta_t-1 + w_t@
@where Beta_t = z_st, z_st-1, z_it, z_it-1, x_st, x_it@
@w_t = x_st, 0, x_it, 0, x_st, x_it@



f_= phi_s1_fnl~phi_s2_fnl~0~0~-1~0|
    1~0~0~0~0~0|
    0~0~phi_i1_fnl~phi_i2_fnl~0~-1|
    0~0~1~0~0~0|
    0~0~0~0~0~0|
    0~0~0~0~0~0;
q_= zeros(rows(f_),rows(f_));
q_[1,1]=lambda_s_eta_fnl^2*sig_eta_fnl^2 + sig_eps_fnl^2;
q_[5,5]=q_[1,1];
q_[1,5]=q_[1,1];
q_[5,1]=q_[1,1];
q_[3,3]=lambda_i_eta_fnl^2*sig_eta_fnl^2+lambda_i_v_fnl^2*sig_v_fnl^2+lambda_i_eps_fnl^2*sig_eps_fnl^2+sig_u_fnl^2;
q_[6,6]=q_[3,3];
q_[3,6]=q_[3,3];
q_[6,3]=q_[3,3];
q_[1,3]=lambda_s_eta_fnl*lambda_i_eta_fnl*sig_eta_fnl^2 + lambda_i_eps_fnl*sig_eps_fnl^2;
q_[3,1]=q_[1,3];
q_[1,6]=q_[1,3];
q_[6,1]=q_[1,3];
q_[3,5]=q_[1,3];
q_[5,3]=q_[1,3];
q_[5,6]=q_[1,3];
q_[6,5]=q_[1,3];

var_beta=reshape(inv(eye(6^2)-f_.*.f_)*vec(q_),6,6);


sig_di=sqrt((gamma_itao_fnl+lambda_k_eta_fnl)^2*sig_eta_fnl^2+lambda_k_eps_fnl^2*sig_eps_fnl^2+sig_v_fnl^2+var_beta[3,3]+
            2*(gamma_itao_fnl+lambda_k_eta_fnl)*lambda_i_eta_fnl*sig_eta_fnl^2+2*lambda_k_eps_fnl*lambda_i_eps_fnl*sig_eps_fnl^2+2*lambda_i_v_fnl*sig_v_fnl^2);

sig_ds = sqrt( sig_eta_fnl^2 + var_beta[1,1] + 2*lambda_s_eta_fnl*sig_eta_fnl^2);

cov_il=(phi_i1_fnl-1)*((gamma_itao_fnl+lambda_k_eta_fnl)*lambda_i_eta_fnl*sig_eta_fnl^2 + lambda_i_v_fnl*sig_v_fnl^2+lambda_i_eps_fnl*lambda_k_eps_fnl*sig_eps_fnl^2)
        +var_beta[3,4];

cov_is=sig_eta_fnl^2*(gamma_itao_fnl + lambda_k_eta_fnl +lambda_i_eta_fnl + (gamma_itao_fnl+lambda_k_eta_fnl)*lambda_s_eta_fnl) + lambda_k_eps_fnl*sig_eps_fnl^2
        +var_beta[1,3];

cov_ils=(phi_s1_fnl-1)*((gamma_itao_fnl+lambda_k_eta_fnl)*lambda_s_eta_fnl*sig_eta_fnl^2+lambda_k_eps_fnl*sig_eps_fnl^2) 
        +var_beta[1,4];


@see updated version of Nov. 26, 2008 document for following@
sig_dy = sqrt( 2*sig_di^2 +sig_ds^2 
                -2*cov_il + 2*cov_is - 2*cov_ils );

sig_ddi = sqrt( 2*sig_di^2 - 2*cov_il);

      
sig_ddi_cf4=sig_ddi;
cov_il_cf4=cov_il;
cov_is_cf4=cov_is;
cov_ils_cf4=cov_ils;
sig_dy_cf4=sig_dy;
sig_ds_cf4=sig_ds;
sig_eta_cf4=sig_eta_fnl;
sig_v_cf4=sig_v_fnl;
sig_eps_cf4=sig_eps_fnl;
sig_di_cf4=sig_di;
sig_di_u_cf4=sig_di_u;
sig_di_p_cf4=sqrt(sig_di^2-sig_di_u^2);
ratio_cf4=sig_di_u/sig_di;

@*******************************************************************@
@5th Counterfactual Variances for change in inventory process only@
@Start with Pre sample as benchmark@
@Write over some pre-parameters with post-parameters@

prm_fnl=prm_fnl_pre;

phi_s1_fnl=prm_fnl[1]+prm_fnl[2];
phi_s2_fnl=-prm_fnl[2];
phi_i1_fnl=prm_fnl[3]+prm_fnl[4];
phi_i2_fnl=-prm_fnl[4];
sig_eta_fnl=prm_fnl[5];
sig_eps_fnl=prm_fnl[6];
sig_v_fnl=prm_fnl[7];
sig_u_fnl=prm_fnl[8];
gamma_itao_fnl=1;
lambda_s_eta_fnl=prm_fnl[10];          
lambda_y_eta_fnl=prm_fnl[11];
lambda_y_eps_fnl=prm_fnl[12];         
lambda_i_v_fnl=  prm_fnl[13];      
lambda_k_eta_fnl=  prm_fnl[14];  
lambda_k_eps_fnl=  0;
lambda_i_eta_fnl=lambda_y_eta_fnl-lambda_s_eta_fnl - gamma_itao_fnl-lambda_k_eta_fnl;
lambda_i_eps_fnl=lambda_y_eps_fnl-1-lambda_k_eps_fnl;


prm_fnl=prm_fnl_post;



phi_i1_fnl=prm_fnl[3]+prm_fnl[4];
phi_i2_fnl=-prm_fnl[4];
sig_v_fnl=prm_fnl[7];
sig_u_fnl=prm_fnl[8];
gamma_itao_fnl=1;         
lambda_y_eta_fnl=prm_fnl[11];
lambda_y_eps_fnl=prm_fnl[12];         
lambda_i_v_fnl=  prm_fnl[13];      
lambda_k_eta_fnl=  prm_fnl[14];  
lambda_k_eps_fnl=  0;
lambda_i_eta_fnl=lambda_y_eta_fnl-lambda_s_eta_fnl - gamma_itao_fnl-lambda_k_eta_fnl;
lambda_i_eps_fnl=lambda_y_eps_fnl-1-lambda_k_eps_fnl;



sig_di_u=sqrt(((lambda_y_eta_fnl-lambda_s_eta_fnl)^2)*sig_eta_fnl^2+((lambda_y_eps_fnl-1)^2)*sig_eps_fnl^2
+((1+lambda_i_v_fnl)^2)*sig_v_fnl^2+sig_u_fnl^2);      @standard deviation of unplanned/unexpected inventories Feb 2009@


@Calculate standard deviation of change in output and of change in inventories@
@phi(L)*z_t = x_t - x_t-1@
@x_t = lambda_ieta*eta + lambda_iv*v + lambda_ieps*eps + u@
@z_t = phi(L)^-1*x_t - phi(L)^-1*x_t-1@
@see notes from Nov. 26, 2008@
@Beta_t = F * Beta_t-1 + w_t@
@where Beta_t = z_st, z_st-1, z_it, z_it-1, x_st, x_it@
@w_t = x_st, 0, x_it, 0, x_st, x_it@



f_= phi_s1_fnl~phi_s2_fnl~0~0~-1~0|
    1~0~0~0~0~0|
    0~0~phi_i1_fnl~phi_i2_fnl~0~-1|
    0~0~1~0~0~0|
    0~0~0~0~0~0|
    0~0~0~0~0~0;
q_= zeros(rows(f_),rows(f_));
q_[1,1]=lambda_s_eta_fnl^2*sig_eta_fnl^2 + sig_eps_fnl^2;
q_[5,5]=q_[1,1];
q_[1,5]=q_[1,1];
q_[5,1]=q_[1,1];
q_[3,3]=lambda_i_eta_fnl^2*sig_eta_fnl^2+lambda_i_v_fnl^2*sig_v_fnl^2+lambda_i_eps_fnl^2*sig_eps_fnl^2+sig_u_fnl^2;
q_[6,6]=q_[3,3];
q_[3,6]=q_[3,3];
q_[6,3]=q_[3,3];
q_[1,3]=lambda_s_eta_fnl*lambda_i_eta_fnl*sig_eta_fnl^2 + lambda_i_eps_fnl*sig_eps_fnl^2;
q_[3,1]=q_[1,3];
q_[1,6]=q_[1,3];
q_[6,1]=q_[1,3];
q_[3,5]=q_[1,3];
q_[5,3]=q_[1,3];
q_[5,6]=q_[1,3];
q_[6,5]=q_[1,3];

var_beta=reshape(inv(eye(6^2)-f_.*.f_)*vec(q_),6,6);


sig_di=sqrt((gamma_itao_fnl+lambda_k_eta_fnl)^2*sig_eta_fnl^2+lambda_k_eps_fnl^2*sig_eps_fnl^2+sig_v_fnl^2+var_beta[3,3]+
            2*(gamma_itao_fnl+lambda_k_eta_fnl)*lambda_i_eta_fnl*sig_eta_fnl^2+2*lambda_k_eps_fnl*lambda_i_eps_fnl*sig_eps_fnl^2+2*lambda_i_v_fnl*sig_v_fnl^2);

sig_ds = sqrt( sig_eta_fnl^2 + var_beta[1,1] + 2*lambda_s_eta_fnl*sig_eta_fnl^2);

cov_il=(phi_i1_fnl-1)*((gamma_itao_fnl+lambda_k_eta_fnl)*lambda_i_eta_fnl*sig_eta_fnl^2 + lambda_i_v_fnl*sig_v_fnl^2+lambda_i_eps_fnl*lambda_k_eps_fnl*sig_eps_fnl^2)
        +var_beta[3,4];

cov_is=sig_eta_fnl^2*(gamma_itao_fnl + lambda_k_eta_fnl +lambda_i_eta_fnl + (gamma_itao_fnl+lambda_k_eta_fnl)*lambda_s_eta_fnl) + lambda_k_eps_fnl*sig_eps_fnl^2
        +var_beta[1,3];

cov_ils=(phi_s1_fnl-1)*((gamma_itao_fnl+lambda_k_eta_fnl)*lambda_s_eta_fnl*sig_eta_fnl^2+lambda_k_eps_fnl*sig_eps_fnl^2) 
        +var_beta[1,4];


@see updated version of Nov. 26, 2008 document for following@
sig_dy = sqrt( 2*sig_di^2 +sig_ds^2 
                -2*cov_il + 2*cov_is - 2*cov_ils );

sig_ddi = sqrt( 2*sig_di^2 - 2*cov_il);

      
sig_ddi_cf5=sig_ddi;
cov_il_cf5=cov_il;
cov_is_cf5=cov_is;
cov_ils_cf5=cov_ils;
sig_dy_cf5=sig_dy;
sig_ds_cf5=sig_ds;
sig_eta_cf5=sig_eta_fnl;
sig_v_cf5=sig_v_fnl;
sig_eps_cf5=sig_eps_fnl;
sig_di_cf5=sig_di;
sig_di_u_cf5=sig_di_u;
sig_di_p_cf5=sqrt(sig_di^2-sig_di_u^2);
ratio_cf5=sig_di_u/sig_di;


@********@

@*******************************************************************@
@6th Counterfactual Variances for change in INVENTORY shocks only@
@Start with Pre sample as benchmark@
@Write over some pre-parameters with post-parameters@
@E.g., only shock variances are overwritten below@

prm_fnl=prm_fnl_pre;

phi_s1_fnl=prm_fnl[1]+prm_fnl[2];
phi_s2_fnl=-prm_fnl[2];
phi_i1_fnl=prm_fnl[3]+prm_fnl[4];
phi_i2_fnl=-prm_fnl[4];
sig_eta_fnl=prm_fnl[5];
sig_eps_fnl=prm_fnl[6];
sig_v_fnl=prm_fnl[7];
sig_u_fnl=prm_fnl[8];
gamma_itao_fnl=1;
lambda_s_eta_fnl=prm_fnl[10];          
lambda_y_eta_fnl=prm_fnl[11];
lambda_y_eps_fnl=prm_fnl[12];         
lambda_i_v_fnl=  prm_fnl[13];      
lambda_k_eta_fnl=  prm_fnl[14];  
lambda_k_eps_fnl=  0;
lambda_i_eta_fnl=lambda_y_eta_fnl-lambda_s_eta_fnl - gamma_itao_fnl-lambda_k_eta_fnl;
lambda_i_eps_fnl=lambda_y_eps_fnl-1-lambda_k_eps_fnl;

prm_fnl=prm_fnl_post;

sig_v_fnl=prm_fnl[7];
sig_u_fnl=prm_fnl[8];    



sig_di_u=sqrt(((lambda_y_eta_fnl-lambda_s_eta_fnl)^2)*sig_eta_fnl^2+((lambda_y_eps_fnl-1)^2)*sig_eps_fnl^2
+((1+lambda_i_v_fnl)^2)*sig_v_fnl^2+sig_u_fnl^2);      @standard deviation of unplanned/unexpected inventories Feb 2009@


@Calculate standard deviation of change in output and of change in inventories@
@phi(L)*z_t = x_t - x_t-1@
@x_t = lambda_ieta*eta + lambda_iv*v + lambda_ieps*eps + u@
@z_t = phi(L)^-1*x_t - phi(L)^-1*x_t-1@
@see notes from Nov. 26, 2008@
@Beta_t = F * Beta_t-1 + w_t@
@where Beta_t = z_st, z_st-1, z_it, z_it-1, x_st, x_it@
@w_t = x_st, 0, x_it, 0, x_st, x_it@



f_= phi_s1_fnl~phi_s2_fnl~0~0~-1~0|
    1~0~0~0~0~0|
    0~0~phi_i1_fnl~phi_i2_fnl~0~-1|
    0~0~1~0~0~0|
    0~0~0~0~0~0|
    0~0~0~0~0~0;
q_= zeros(rows(f_),rows(f_));
q_[1,1]=lambda_s_eta_fnl^2*sig_eta_fnl^2 + sig_eps_fnl^2;
q_[5,5]=q_[1,1];
q_[1,5]=q_[1,1];
q_[5,1]=q_[1,1];
q_[3,3]=lambda_i_eta_fnl^2*sig_eta_fnl^2+lambda_i_v_fnl^2*sig_v_fnl^2+lambda_i_eps_fnl^2*sig_eps_fnl^2+sig_u_fnl^2;
q_[6,6]=q_[3,3];
q_[3,6]=q_[3,3];
q_[6,3]=q_[3,3];
q_[1,3]=lambda_s_eta_fnl*lambda_i_eta_fnl*sig_eta_fnl^2 + lambda_i_eps_fnl*sig_eps_fnl^2;
q_[3,1]=q_[1,3];
q_[1,6]=q_[1,3];
q_[6,1]=q_[1,3];
q_[3,5]=q_[1,3];
q_[5,3]=q_[1,3];
q_[5,6]=q_[1,3];
q_[6,5]=q_[1,3];

var_beta=reshape(inv(eye(6^2)-f_.*.f_)*vec(q_),6,6);


sig_di=sqrt((gamma_itao_fnl+lambda_k_eta_fnl)^2*sig_eta_fnl^2+lambda_k_eps_fnl^2*sig_eps_fnl^2+sig_v_fnl^2+var_beta[3,3]+
            2*(gamma_itao_fnl+lambda_k_eta_fnl)*lambda_i_eta_fnl*sig_eta_fnl^2+2*lambda_k_eps_fnl*lambda_i_eps_fnl*sig_eps_fnl^2+2*lambda_i_v_fnl*sig_v_fnl^2);

sig_ds = sqrt( sig_eta_fnl^2 + var_beta[1,1] + 2*lambda_s_eta_fnl*sig_eta_fnl^2);

cov_il=(phi_i1_fnl-1)*((gamma_itao_fnl+lambda_k_eta_fnl)*lambda_i_eta_fnl*sig_eta_fnl^2 + lambda_i_v_fnl*sig_v_fnl^2+lambda_i_eps_fnl*lambda_k_eps_fnl*sig_eps_fnl^2)
        +var_beta[3,4];

cov_is=sig_eta_fnl^2*(gamma_itao_fnl + lambda_k_eta_fnl +lambda_i_eta_fnl + (gamma_itao_fnl+lambda_k_eta_fnl)*lambda_s_eta_fnl) + lambda_k_eps_fnl*sig_eps_fnl^2
        +var_beta[1,3];

cov_ils=(phi_s1_fnl-1)*((gamma_itao_fnl+lambda_k_eta_fnl)*lambda_s_eta_fnl*sig_eta_fnl^2+lambda_k_eps_fnl*sig_eps_fnl^2) 
        +var_beta[1,4];


@see updated version of Nov. 26, 2008 document for following@
sig_dy = sqrt( 2*sig_di^2 +sig_ds^2 
                -2*cov_il + 2*cov_is - 2*cov_ils );

sig_ddi = sqrt( 2*sig_di^2 - 2*cov_il);

      
sig_ddi_cf6=sig_ddi;
cov_il_cf6=cov_il;
cov_is_cf6=cov_is;
cov_ils_cf6=cov_ils;
sig_dy_cf6=sig_dy;
sig_ds_cf6=sig_ds;
sig_eta_cf6=sig_eta_fnl;
sig_v_cf6=sig_v_fnl;
sig_eps_cf6=sig_eps_fnl;
sig_di_cf6=sig_di;
sig_di_u_cf6=sig_di_u;
sig_di_p_cf6=sqrt(sig_di^2-sig_di_u^2);
ratio_cf6=sig_di_u/sig_di;


@********@

@*******************************************************************@
@7th Counterfactual Variances for change in INVENTORY propagation only@
@Start with Pre sample as benchmark@
@Write over some pre-parameters with post-parameters@
@E.g., only shock variances are overwritten below@

prm_fnl=prm_fnl_pre;

phi_s1_fnl=prm_fnl[1]+prm_fnl[2];
phi_s2_fnl=-prm_fnl[2];
phi_i1_fnl=prm_fnl[3]+prm_fnl[4];
phi_i2_fnl=-prm_fnl[4];
sig_eta_fnl=prm_fnl[5];
sig_eps_fnl=prm_fnl[6];
sig_v_fnl=prm_fnl[7];
sig_u_fnl=prm_fnl[8];
gamma_itao_fnl=1;
lambda_s_eta_fnl=prm_fnl[10];          
lambda_y_eta_fnl=prm_fnl[11];
lambda_y_eps_fnl=prm_fnl[12];         
lambda_i_v_fnl=  prm_fnl[13];      
lambda_k_eta_fnl=  prm_fnl[14];  
lambda_k_eps_fnl=  0;
lambda_i_eta_fnl=lambda_y_eta_fnl-lambda_s_eta_fnl - gamma_itao_fnl-lambda_k_eta_fnl;
lambda_i_eps_fnl=lambda_y_eps_fnl-1-lambda_k_eps_fnl;

prm_fnl=prm_fnl_post;


phi_i1_fnl=prm_fnl[3]+prm_fnl[4];
phi_i2_fnl=-prm_fnl[4];
gamma_itao_fnl=1;        
lambda_y_eta_fnl=prm_fnl[11];
lambda_y_eps_fnl=prm_fnl[12];         
lambda_i_v_fnl=  prm_fnl[13];      
lambda_k_eta_fnl=  prm_fnl[14];  
lambda_k_eps_fnl=  0;
lambda_i_eta_fnl=lambda_y_eta_fnl-lambda_s_eta_fnl - gamma_itao_fnl-lambda_k_eta_fnl;
lambda_i_eps_fnl=lambda_y_eps_fnl-1-lambda_k_eps_fnl;


sig_di_u=sqrt(((lambda_y_eta_fnl-lambda_s_eta_fnl)^2)*sig_eta_fnl^2+((lambda_y_eps_fnl-1)^2)*sig_eps_fnl^2
+((1+lambda_i_v_fnl)^2)*sig_v_fnl^2+sig_u_fnl^2);      @standard deviation of unplanned/unexpected inventories Feb 2009@

@Calculate standard deviation of change in output and of change in inventories@
@phi(L)*z_t = x_t - x_t-1@
@x_t = lambda_ieta*eta + lambda_iv*v + lambda_ieps*eps + u@
@z_t = phi(L)^-1*x_t - phi(L)^-1*x_t-1@
@see notes from Nov. 26, 2008@
@Beta_t = F * Beta_t-1 + w_t@
@where Beta_t = z_st, z_st-1, z_it, z_it-1, x_st, x_it@
@w_t = x_st, 0, x_it, 0, x_st, x_it@



f_= phi_s1_fnl~phi_s2_fnl~0~0~-1~0|
    1~0~0~0~0~0|
    0~0~phi_i1_fnl~phi_i2_fnl~0~-1|
    0~0~1~0~0~0|
    0~0~0~0~0~0|
    0~0~0~0~0~0;
q_= zeros(rows(f_),rows(f_));
q_[1,1]=lambda_s_eta_fnl^2*sig_eta_fnl^2 + sig_eps_fnl^2;
q_[5,5]=q_[1,1];
q_[1,5]=q_[1,1];
q_[5,1]=q_[1,1];
q_[3,3]=lambda_i_eta_fnl^2*sig_eta_fnl^2+lambda_i_v_fnl^2*sig_v_fnl^2+lambda_i_eps_fnl^2*sig_eps_fnl^2+sig_u_fnl^2;
q_[6,6]=q_[3,3];
q_[3,6]=q_[3,3];
q_[6,3]=q_[3,3];
q_[1,3]=lambda_s_eta_fnl*lambda_i_eta_fnl*sig_eta_fnl^2 + lambda_i_eps_fnl*sig_eps_fnl^2;
q_[3,1]=q_[1,3];
q_[1,6]=q_[1,3];
q_[6,1]=q_[1,3];
q_[3,5]=q_[1,3];
q_[5,3]=q_[1,3];
q_[5,6]=q_[1,3];
q_[6,5]=q_[1,3];

var_beta=reshape(inv(eye(6^2)-f_.*.f_)*vec(q_),6,6);


sig_di=sqrt((gamma_itao_fnl+lambda_k_eta_fnl)^2*sig_eta_fnl^2+lambda_k_eps_fnl^2*sig_eps_fnl^2+sig_v_fnl^2+var_beta[3,3]+
            2*(gamma_itao_fnl+lambda_k_eta_fnl)*lambda_i_eta_fnl*sig_eta_fnl^2+2*lambda_k_eps_fnl*lambda_i_eps_fnl*sig_eps_fnl^2+2*lambda_i_v_fnl*sig_v_fnl^2);

sig_ds = sqrt( sig_eta_fnl^2 + var_beta[1,1] + 2*lambda_s_eta_fnl*sig_eta_fnl^2);

cov_il=(phi_i1_fnl-1)*((gamma_itao_fnl+lambda_k_eta_fnl)*lambda_i_eta_fnl*sig_eta_fnl^2 + lambda_i_v_fnl*sig_v_fnl^2+lambda_i_eps_fnl*lambda_k_eps_fnl*sig_eps_fnl^2)
        +var_beta[3,4];

cov_is=sig_eta_fnl^2*(gamma_itao_fnl + lambda_k_eta_fnl +lambda_i_eta_fnl + (gamma_itao_fnl+lambda_k_eta_fnl)*lambda_s_eta_fnl) + lambda_k_eps_fnl*sig_eps_fnl^2
        +var_beta[1,3];

cov_ils=(phi_s1_fnl-1)*((gamma_itao_fnl+lambda_k_eta_fnl)*lambda_s_eta_fnl*sig_eta_fnl^2+lambda_k_eps_fnl*sig_eps_fnl^2) 
        +var_beta[1,4];


@see updated version of Nov. 26, 2008 document for following@
sig_dy = sqrt( 2*sig_di^2 +sig_ds^2 
                -2*cov_il + 2*cov_is - 2*cov_ils );

sig_ddi = sqrt( 2*sig_di^2 - 2*cov_il);

      
sig_ddi_cf7=sig_ddi;
cov_il_cf7=cov_il;
cov_is_cf7=cov_is;
cov_ils_cf7=cov_ils;
sig_dy_cf7=sig_dy;
sig_ds_cf7=sig_ds;
sig_eta_cf7=sig_eta_fnl;
sig_v_cf7=sig_v_fnl;
sig_eps_cf7=sig_eps_fnl;
sig_di_cf7=sig_di;
sig_di_u_cf7=sig_di_u;
sig_di_p_cf7=sqrt(sig_di^2-sig_di_u^2);
ratio_cf7=sig_di_u/sig_di;


@********@

@*******************************************************************@
@8th Counterfactual Variances for change in SALES process only@
@Start with Pre sample as benchmark@
@Write over some pre-parameters with post-parameters@


prm_fnl=prm_fnl_pre;

phi_s1_fnl=prm_fnl[1]+prm_fnl[2];
phi_s2_fnl=-prm_fnl[2];
phi_i1_fnl=prm_fnl[3]+prm_fnl[4];
phi_i2_fnl=-prm_fnl[4];
sig_eta_fnl=prm_fnl[5];
sig_eps_fnl=prm_fnl[6];
sig_v_fnl=prm_fnl[7];
sig_u_fnl=prm_fnl[8];
gamma_itao_fnl=1;
lambda_s_eta_fnl=prm_fnl[10];          
lambda_y_eta_fnl=prm_fnl[11];
lambda_y_eps_fnl=prm_fnl[12];         
lambda_i_v_fnl=  prm_fnl[13];      
lambda_k_eta_fnl=  prm_fnl[14];  
lambda_k_eps_fnl=  0;
lambda_i_eta_fnl=lambda_y_eta_fnl-lambda_s_eta_fnl - gamma_itao_fnl-lambda_k_eta_fnl;
lambda_i_eps_fnl=lambda_y_eps_fnl-1-lambda_k_eps_fnl;

prm_fnl=prm_fnl_post;

phi_s1_fnl=prm_fnl[1]+prm_fnl[2];
phi_s2_fnl=-prm_fnl[2];
sig_eta_fnl=prm_fnl[5];
sig_eps_fnl=prm_fnl[6];
lambda_s_eta_fnl=prm_fnl[10];            
lambda_i_eta_fnl=lambda_y_eta_fnl-lambda_s_eta_fnl - gamma_itao_fnl-lambda_k_eta_fnl;
lambda_i_eps_fnl=lambda_y_eps_fnl-1-lambda_k_eps_fnl;



sig_di_u=sqrt(((lambda_y_eta_fnl-lambda_s_eta_fnl)^2)*sig_eta_fnl^2+((lambda_y_eps_fnl-1)^2)*sig_eps_fnl^2
+((1+lambda_i_v_fnl)^2)*sig_v_fnl^2+sig_u_fnl^2);      @standard deviation of unplanned/unexpected inventories Feb 2009@


@Calculate standard deviation of change in output and of change in inventories@
@phi(L)*z_t = x_t - x_t-1@
@x_t = lambda_ieta*eta + lambda_iv*v + lambda_ieps*eps + u@
@z_t = phi(L)^-1*x_t - phi(L)^-1*x_t-1@
@see notes from Nov. 26, 2008@
@Beta_t = F * Beta_t-1 + w_t@
@where Beta_t = z_st, z_st-1, z_it, z_it-1, x_st, x_it@
@w_t = x_st, 0, x_it, 0, x_st, x_it@



f_= phi_s1_fnl~phi_s2_fnl~0~0~-1~0|
    1~0~0~0~0~0|
    0~0~phi_i1_fnl~phi_i2_fnl~0~-1|
    0~0~1~0~0~0|
    0~0~0~0~0~0|
    0~0~0~0~0~0;
q_= zeros(rows(f_),rows(f_));
q_[1,1]=lambda_s_eta_fnl^2*sig_eta_fnl^2 + sig_eps_fnl^2;
q_[5,5]=q_[1,1];
q_[1,5]=q_[1,1];
q_[5,1]=q_[1,1];
q_[3,3]=lambda_i_eta_fnl^2*sig_eta_fnl^2+lambda_i_v_fnl^2*sig_v_fnl^2+lambda_i_eps_fnl^2*sig_eps_fnl^2+sig_u_fnl^2;
q_[6,6]=q_[3,3];
q_[3,6]=q_[3,3];
q_[6,3]=q_[3,3];
q_[1,3]=lambda_s_eta_fnl*lambda_i_eta_fnl*sig_eta_fnl^2 + lambda_i_eps_fnl*sig_eps_fnl^2;
q_[3,1]=q_[1,3];
q_[1,6]=q_[1,3];
q_[6,1]=q_[1,3];
q_[3,5]=q_[1,3];
q_[5,3]=q_[1,3];
q_[5,6]=q_[1,3];
q_[6,5]=q_[1,3];

var_beta=reshape(inv(eye(6^2)-f_.*.f_)*vec(q_),6,6);


sig_di=sqrt((gamma_itao_fnl+lambda_k_eta_fnl)^2*sig_eta_fnl^2+lambda_k_eps_fnl^2*sig_eps_fnl^2+sig_v_fnl^2+var_beta[3,3]+
            2*(gamma_itao_fnl+lambda_k_eta_fnl)*lambda_i_eta_fnl*sig_eta_fnl^2+2*lambda_k_eps_fnl*lambda_i_eps_fnl*sig_eps_fnl^2+2*lambda_i_v_fnl*sig_v_fnl^2);

sig_ds = sqrt( sig_eta_fnl^2 + var_beta[1,1] + 2*lambda_s_eta_fnl*sig_eta_fnl^2);

cov_il=(phi_i1_fnl-1)*((gamma_itao_fnl+lambda_k_eta_fnl)*lambda_i_eta_fnl*sig_eta_fnl^2 + lambda_i_v_fnl*sig_v_fnl^2+lambda_i_eps_fnl*lambda_k_eps_fnl*sig_eps_fnl^2)
        +var_beta[3,4];

cov_is=sig_eta_fnl^2*(gamma_itao_fnl + lambda_k_eta_fnl +lambda_i_eta_fnl + (gamma_itao_fnl+lambda_k_eta_fnl)*lambda_s_eta_fnl) + lambda_k_eps_fnl*sig_eps_fnl^2
        +var_beta[1,3];

cov_ils=(phi_s1_fnl-1)*((gamma_itao_fnl+lambda_k_eta_fnl)*lambda_s_eta_fnl*sig_eta_fnl^2+lambda_k_eps_fnl*sig_eps_fnl^2) 
        +var_beta[1,4];


@see updated version of Nov. 26, 2008 document for following@
sig_dy = sqrt( 2*sig_di^2 +sig_ds^2 
                -2*cov_il + 2*cov_is - 2*cov_ils );

sig_ddi = sqrt( 2*sig_di^2 - 2*cov_il);

      
sig_ddi_cf8=sig_ddi;
cov_il_cf8=cov_il;
cov_is_cf8=cov_is;
cov_ils_cf8=cov_ils;
sig_dy_cf8=sig_dy;
sig_ds_cf8=sig_ds;
sig_eta_cf8=sig_eta_fnl;
sig_v_cf8=sig_v_fnl;
sig_eps_cf8=sig_eps_fnl;
sig_di_cf8=sig_di;
sig_di_u_cf8=sig_di_u;
sig_di_p_cf8=sqrt(sig_di^2-sig_di_u^2);
ratio_cf8=sig_di_u/sig_di;


@********@
@*******************************************************************@
@9th Counterfactual Variances for change in "u" only@
@Start with Pre sample as benchmark@
@Write over some pre-parameters with post-parameters@


prm_fnl=prm_fnl_pre;

phi_s1_fnl=prm_fnl[1]+prm_fnl[2];
phi_s2_fnl=-prm_fnl[2];
phi_i1_fnl=prm_fnl[3]+prm_fnl[4];
phi_i2_fnl=-prm_fnl[4];
sig_eta_fnl=prm_fnl[5];
sig_eps_fnl=prm_fnl[6];
sig_v_fnl=prm_fnl[7];
sig_u_fnl=prm_fnl[8];
gamma_itao_fnl=1;
lambda_s_eta_fnl=prm_fnl[10];          
lambda_y_eta_fnl=prm_fnl[11];
lambda_y_eps_fnl=prm_fnl[12];         
lambda_i_v_fnl=  prm_fnl[13];      
lambda_k_eta_fnl=  prm_fnl[14];  
lambda_k_eps_fnl=  0;
lambda_i_eta_fnl=lambda_y_eta_fnl-lambda_s_eta_fnl - gamma_itao_fnl-lambda_k_eta_fnl;
lambda_i_eps_fnl=lambda_y_eps_fnl-1-lambda_k_eps_fnl;


prm_fnl=prm_fnl_post;

sig_u_fnl=prm_fnl[8];


sig_di_u=sqrt(((lambda_y_eta_fnl-lambda_s_eta_fnl)^2)*sig_eta_fnl^2+((lambda_y_eps_fnl-1)^2)*sig_eps_fnl^2
+((1+lambda_i_v_fnl)^2)*sig_v_fnl^2+sig_u_fnl^2);      @standard deviation of unplanned/unexpected inventories Feb 2009@

@Calculate standard deviation of change in output and of change in inventories@
@phi(L)*z_t = x_t - x_t-1@
@x_t = lambda_ieta*eta + lambda_iv*v + lambda_ieps*eps + u@
@z_t = phi(L)^-1*x_t - phi(L)^-1*x_t-1@
@see notes from Nov. 26, 2008@
@Beta_t = F * Beta_t-1 + w_t@
@where Beta_t = z_st, z_st-1, z_it, z_it-1, x_st, x_it@
@w_t = x_st, 0, x_it, 0, x_st, x_it@



f_= phi_s1_fnl~phi_s2_fnl~0~0~-1~0|
    1~0~0~0~0~0|
    0~0~phi_i1_fnl~phi_i2_fnl~0~-1|
    0~0~1~0~0~0|
    0~0~0~0~0~0|
    0~0~0~0~0~0;
q_= zeros(rows(f_),rows(f_));
q_[1,1]=lambda_s_eta_fnl^2*sig_eta_fnl^2 + sig_eps_fnl^2;
q_[5,5]=q_[1,1];
q_[1,5]=q_[1,1];
q_[5,1]=q_[1,1];
q_[3,3]=lambda_i_eta_fnl^2*sig_eta_fnl^2+lambda_i_v_fnl^2*sig_v_fnl^2+lambda_i_eps_fnl^2*sig_eps_fnl^2+sig_u_fnl^2;
q_[6,6]=q_[3,3];
q_[3,6]=q_[3,3];
q_[6,3]=q_[3,3];
q_[1,3]=lambda_s_eta_fnl*lambda_i_eta_fnl*sig_eta_fnl^2 + lambda_i_eps_fnl*sig_eps_fnl^2;
q_[3,1]=q_[1,3];
q_[1,6]=q_[1,3];
q_[6,1]=q_[1,3];
q_[3,5]=q_[1,3];
q_[5,3]=q_[1,3];
q_[5,6]=q_[1,3];
q_[6,5]=q_[1,3];

var_beta=reshape(inv(eye(6^2)-f_.*.f_)*vec(q_),6,6);


sig_di=sqrt((gamma_itao_fnl+lambda_k_eta_fnl)^2*sig_eta_fnl^2+lambda_k_eps_fnl^2*sig_eps_fnl^2+sig_v_fnl^2+var_beta[3,3]+
            2*(gamma_itao_fnl+lambda_k_eta_fnl)*lambda_i_eta_fnl*sig_eta_fnl^2+2*lambda_k_eps_fnl*lambda_i_eps_fnl*sig_eps_fnl^2+2*lambda_i_v_fnl*sig_v_fnl^2);

sig_ds = sqrt( sig_eta_fnl^2 + var_beta[1,1] + 2*lambda_s_eta_fnl*sig_eta_fnl^2);

cov_il=(phi_i1_fnl-1)*((gamma_itao_fnl+lambda_k_eta_fnl)*lambda_i_eta_fnl*sig_eta_fnl^2 + lambda_i_v_fnl*sig_v_fnl^2+lambda_i_eps_fnl*lambda_k_eps_fnl*sig_eps_fnl^2)
        +var_beta[3,4];

cov_is=sig_eta_fnl^2*(gamma_itao_fnl + lambda_k_eta_fnl +lambda_i_eta_fnl + (gamma_itao_fnl+lambda_k_eta_fnl)*lambda_s_eta_fnl) + lambda_k_eps_fnl*sig_eps_fnl^2
        +var_beta[1,3];

cov_ils=(phi_s1_fnl-1)*((gamma_itao_fnl+lambda_k_eta_fnl)*lambda_s_eta_fnl*sig_eta_fnl^2+lambda_k_eps_fnl*sig_eps_fnl^2) 
        +var_beta[1,4];


@see updated version of Nov. 26, 2008 document for following@
sig_dy = sqrt( 2*sig_di^2 +sig_ds^2 
                -2*cov_il + 2*cov_is - 2*cov_ils );

sig_ddi = sqrt( 2*sig_di^2 - 2*cov_il);

      
sig_ddi_cf9=sig_ddi;
cov_il_cf9=cov_il;
cov_is_cf9=cov_is;
cov_ils_cf9=cov_ils;
sig_dy_cf9=sig_dy;
sig_ds_cf9=sig_ds;
sig_eta_cf9=sig_eta_fnl;
sig_v_cf9=sig_v_fnl;
sig_eps_cf9=sig_eps_fnl;
sig_di_cf9=sig_di;
sig_di_u_cf9=sig_di_u;
sig_di_p_cf9=sqrt(sig_di^2-sig_di_u^2);
ratio_cf9=sig_di_u/sig_di;


@********@



@***************************************************@


@Sample Statistics@
sig_Di_smpl=stdc(y[start_date+1:rows(y),2]-y[start_date:rows(y)-1,2]);
sig_Di_smpl_pre=stdc(y[start_date+1:break_date,2]-y[start_date:break_date-1,2]);
sig_Di_smpl_post=stdc(y[break_date+2:rows(y),2]-y[break_date+1:rows(y)-1,2]);
sig_Dy_smpl=stdc(y[start_date+1:rows(y),3]-y[start_date:rows(y)-1,3]);
sig_Dy_smpl_pre=stdc(y[start_date+1:break_date,3]-y[start_date:break_date-1,3]);
sig_Dy_smpl_post=stdc(y[break_date+2:rows(y),3]-y[break_date+1:rows(y)-1,3]);
sig_Ds_smpl=stdc(y[start_date+1:rows(y),1]-y[start_date:rows(y)-1,1]);
sig_Ds_smpl_pre=stdc(y[start_date+1:break_date,1]-y[start_date:break_date-1,1]);
sig_Ds_smpl_post=stdc(y[break_date+2:rows(y),1]-y[break_date+1:rows(y)-1,1]);

sig_DDi_smpl=stdc((y[start_date+2:rows(y),2]-y[start_date+1:rows(y)-1,2])-(y[start_date+1:rows(y)-1,2]-y[start_date:rows(y)-2,2]));
sig_DDi_smpl_pre=stdc((y[start_date+2:break_date,2]-y[start_date+1:break_date-1,2])-(y[start_date+1:break_date-1,2]-y[start_date:break_date-2,2]));
sig_DDi_smpl_post=stdc((y[break_date+3:rows(y),2]-y[break_date+2:rows(y)-1,2])-(y[break_date+2:rows(y)-1,2]-y[break_date+1:rows(y)-2,2]));

corr_is_smpl=((0~1)*corrx((y[start_date+2:rows(y),2]-y[start_date+1:rows(y)-1,2])-(y[start_date+1:rows(y)-1,2]-y[start_date:rows(y)-2,2])~y[start_date+2:rows(y),1]-y[start_date+1:rows(y)-1,1])*(1|0));
corr_is_smpl_pre=((0~1)*corrx((y[start_date+2:break_date,2]-y[start_date+1:break_date-1,2])-(y[start_date+1:break_date-1,2]-y[start_date:break_date-2,2])~y[start_date+2:break_date,1]-y[start_date+1:break_date-1,1])*(1|0));
corr_is_smpl_post=((0~1)*corrx((y[break_date+3:rows(y),2]-y[break_date+2:rows(y)-1,2])-(y[break_date+2:rows(y)-1,2]-y[break_date+1:rows(y)-2,2])~y[break_date+3:rows(y),1]-y[break_date+2:rows(y)-1,1])*(1|0));

format /m1 /rd 6,3;


output file=output_mh/uc_si.out;  
output on;
"";
"";
"Table 5		Parameters for UC Model";
"               Pre            Post		Change";
"sig_eta";;sort_out[5,1];;"(";;sort_out[5,2];;")";;sort_out[19,1];;"(";;sort_out[19,2];;")";;sort_out[91,1];;"(";;sort_out[91,2];;")";
"sig_eps";;sort_out[6,1];;"(";;sort_out[6,2];;")";;sort_out[20,1];;"(";;sort_out[20,2];;")";;sort_out[92,1];;"(";;sort_out[92,2];;")";
"phi_s";;sort_out[1,1];;"(";;sort_out[1,2];;")";;sort_out[15,1];;"(";;sort_out[15,2];;")";;sort_out[93,1];;"(";;sort_out[93,2];;")";
"lambda_s_eta";;sort_out[10,1];;"(";;sort_out[10,2];;")";;sort_out[24,1];;"(";;sort_out[24,2];;")";;sort_out[94,1];;"(";;sort_out[94,2];;")";
"sig_v";;sort_out[7,1];;"(";;sort_out[7,2];;")";;sort_out[21,1];;"(";;sort_out[21,2];;")";;sort_out[95,1];;"(";;sort_out[95,2];;")";
"sig_u";;sort_out[8,1];;"(";;sort_out[8,2];;")";;sort_out[22,1];;"(";;sort_out[22,2];;")";;sort_out[96,1];;"(";;sort_out[96,2];;")";
"phi_i";;sort_out[3,1];;"(";;sort_out[3,2];;")";;sort_out[17,1];;"(";;sort_out[17,2];;")";;sort_out[97,1];;"(";;sort_out[97,2];;")";
"mu_k";;sort_out[9,1];;"(";;sort_out[9,2];;")";;sort_out[23,1];;"(";;sort_out[23,2];;")";;sort_out[98,1];;"(";;sort_out[98,2];;")";
"lambda_y_eta";;sort_out[11,1];;"(";;sort_out[11,2];;")";;sort_out[25,1];;"(";;sort_out[25,2];;")";;sort_out[99,1];;"(";;sort_out[99,2];;")";
"lambda_y_eps";;sort_out[12,1];;"(";;sort_out[12,2];;")";;sort_out[26,1];;"(";;sort_out[26,2];;")";;sort_out[100,1];;"(";;sort_out[100,2];;")";
"lambda_i_v";;sort_out[13,1];;"(";;sort_out[13,2];;")";;sort_out[27,1];;"(";;sort_out[27,2];;")";;sort_out[101,1];;"(";;sort_out[101,2];;")";
"lambda_k_eta";;sort_out[14,1];;"(";;sort_out[14,2];;")";;sort_out[28,1];;"(";;sort_out[28,2];;")";;sort_out[102,1];;"(";;sort_out[102,2];;")";
"lambda_k_eps";;sort_out[29,1];;"(";;sort_out[29,2];;")";;sort_out[30,1];;"(";;sort_out[30,2];;")";;sort_out[103,1];;"(";;sort_out[103,2];;")";

"";

"Table 6 Implied Volatilities";
"               Pre            Post		Change";
"sig_Dy	";;sort_out[31,1];;"(";;sort_out[31,2];;")";;sort_out[32,1];;"(";;sort_out[32,2];;")";;sort_out[106,1];;"(";;sort_out[106,2];;")";
"sig_Ds	";;sort_out[33,1];;"(";;sort_out[33,2];;")";;sort_out[34,1];;"(";;sort_out[34,2];;")";;sort_out[107,1];;"(";;sort_out[107,2];;")";
"sig_DDi	";;sort_out[35,1];;"(";;sort_out[35,2];;")";;sort_out[36,1];;"(";;sort_out[36,2];;")";;sort_out[108,1];;"(";;sort_out[108,2];;")";
"corr_DDiDs	";;sort_out[114,1];;"(";;sort_out[114,2];;")";;sort_out[115,1];;"(";;sort_out[115,2];;")";;sort_out[116,1];;"(";;sort_out[116,2];;")";
"sig_Di	";;sort_out[104,1];;"(";;sort_out[104,2];;")";;sort_out[105,1];;"(";;sort_out[105,2];;")";;sort_out[109,1];;"(";;sort_out[109,2];;")";
"sig_Di_unexpected	";;sort_out[37,1];;"(";;sort_out[37,2];;")";;sort_out[38,1];;"(";;sort_out[38,2];;")";;sort_out[110,1];;"(";;sort_out[110,2];;")";
"sig_Di_expected	";;sort_out[39,1];;"(";;sort_out[39,2];;")";;sort_out[40,1];;"(";;sort_out[40,2];;")";;sort_out[111,1];;"(";;sort_out[111,2];;")";
"corr_Di_unexpected_Ds	";;sort_out[117,1];;"(";;sort_out[117,2];;")";;sort_out[118,1];;"(";;sort_out[118,2];;")";;sort_out[119,1];;"(";;sort_out[119,2];;")";
"";


"Table 7 Counterfacutal Experiments for Dy";
"               Pre            Post		Change";
"Actual sd(Dy)	";;sort_out[31,1];;"(";;sort_out[31,2];;")";;sort_out[32,1];;"(";;sort_out[32,2];;")";;sort_out[89,1];;"(";;sort_out[89,2];;")";
"Sales process change	";;sort_out[31,1];;"(";;sort_out[31,2];;")";;sort_out[47,1];;"(";;sort_out[47,2];;")";;sort_out[57,1];;"(";;sort_out[57,2];;")";
"Inventory process change	";;sort_out[31,1];;"(";;sort_out[31,2];;")";;sort_out[41,1];;"(";;sort_out[41,2];;")";;sort_out[51,1];;"(";;sort_out[51,2];;")";
"Inventory shocks change	";;sort_out[31,1];;"(";;sort_out[31,2];;")";;sort_out[43,1];;"(";;sort_out[43,2];;")";;sort_out[53,1];;"(";;sort_out[53,2];;")";
"Ut shocks change	";;sort_out[31,1];;"(";;sort_out[31,2];;")";;sort_out[49,1];;"(";;sort_out[49,2];;")";;sort_out[59,1];;"(";;sort_out[59,2];;")";
"Inventory prop change	";;sort_out[31,1];;"(";;sort_out[31,2];;")";;sort_out[45,1];;"(";;sort_out[45,2];;")";;sort_out[55,1];;"(";;sort_out[55,2];;")";
"";

"Table 8		Implied ECM coefficients";
"               Pre            Post		Change";
"alpha_y		";;sort_out[69,1];;"(";;sort_out[69,2];;")";;sort_out[83,1];;"(";;sort_out[83,2];;")";;sort_out[112,1];;"(";;sort_out[112,2];;")";
"alpha_ys	";;sort_out[74,1];;"(";;sort_out[74,2];;")";;sort_out[88,1];;"(";;sort_out[88,2];;")";;sort_out[113,1];;"(";;sort_out[113,2];;")";

"";
"";




"";
"corr(dlinv,dinv/gdp(-1))=";;corrx(y[start_date+1:rows(y),2]-y[start_date:rows(y)-1,2]~dinv_n[start_date:rows(dinv_n)]);
"";
"(Mode)";
"Data=";;data_code;;"  3=total, 4=durables";
"";
"               Pre            Post";
"sig_Dy=";;sig_dy_pre~sig_dy_post;
"sig_Ds=";;sig_ds_pre~sig_ds_post;
"sig_DDi=";;sig_ddi_pre~sig_ddi_post;
"corr_Ds_DDi=";;corr_ddids_pre~corr_ddids_post;
"sig_Di_unexpected=";;sig_di_u_pre~sig_di_u_post;
"sig_Di_expected=";;sig_di_p_pre~sig_di_p_post;
"corr_Di_unexpected_Ds=";;corr_diuds_pre~corr_diuds_post;

"";
"sig_Dy_smpl=";;sig_Dy_smpl_pre~sig_Dy_smpl_post;
"sig_Ds_smpl=";;sig_Ds_smpl_pre~sig_Ds_smpl_post;
"sig_DDi_smpl=";;sig_DDi_smpl_pre~sig_DDi_smpl_post;
"corr_Ds_DDi_smpl=";;corr_is_smpl_pre~corr_is_smpl_post;
"";"";



"Counterfactuals (Mode)";
"Data=";;data_code;;"  3=total, 4=durables";
"";
"               SD(Dy)            SD(Ds)";
"Pre-mod 		";;sig_dy_pre~sig_ds_pre;
"Post-mod		";;sig_dy_post~sig_ds_post;
"Inv. process	";;sig_dy_cf5~sig_ds_cf5;
"Inv. shocks 	";;sig_dy_cf6~sig_ds_cf6;
"u shocks		";;sig_dy_cf9~sig_ds_cf9;
"Inv. prop		";;sig_dy_cf7~sig_ds_cf7;
"Sales process	";;sig_dy_cf8~sig_ds_cf8;
"";
"Pre-mod(smpl)  ";;sig_Dy_smpl_pre~sig_Ds_smpl_pre;
"Post-mod(smpl)	";;sig_Dy_smpl_post~sig_Ds_smpl_post;
"";"";
"UC Model and Forecasting (Mode)";
"Data=";;data_code;;"  3=total, 4=durables";
"";
"              eta            v			eps			u		total";
"								Pre-mod";
"weights";;ratio_eta_Di_u_pre;;ratio_v_Di_u_pre;;ratio_eps_Di_u_pre;;ratio_u_Di_u_pre;
"alpha_y";;alpha_y_eta_pre;;alpha_y_v_pre;;alpha_y_eps_pre;;alpha_y_u_pre;;
(ratio_eta_Di_u_pre*alpha_y_eta_pre + ratio_v_Di_u_pre*alpha_y_v_pre + ratio_eps_Di_u_pre*alpha_y_eps_pre + ratio_u_Di_u_pre*alpha_y_u_pre);
"alpha_s";;alpha_s_eta_pre;;alpha_s_v_pre;;alpha_s_eps_pre;;alpha_s_u_pre;;
(ratio_eta_Di_u_pre*alpha_s_eta_pre + ratio_v_Di_u_pre*alpha_s_v_pre + ratio_eps_Di_u_pre*alpha_s_eps_pre + ratio_u_Di_u_pre*alpha_s_u_pre);
"";
"								Post-mod";
"weights";;ratio_eta_Di_u_post;;ratio_v_Di_u_post;;ratio_eps_Di_u_post;;ratio_u_Di_u_post;
"alpha_y";;alpha_y_eta_post;;alpha_y_v_post;;alpha_y_eps_post;;alpha_y_u_post;;
(ratio_eta_Di_u_post*alpha_y_eta_post + ratio_v_Di_u_post*alpha_y_v_post + ratio_eps_Di_u_post*alpha_y_eps_post + ratio_u_Di_u_post*alpha_y_u_post);
"alpha_s";;alpha_s_eta_post;;alpha_s_v_post;;alpha_s_eps_post;;alpha_s_u_post;;
(ratio_eta_Di_u_post*alpha_s_eta_post + ratio_v_Di_u_post*alpha_s_v_post + ratio_eps_Di_u_post*alpha_s_eps_post + ratio_u_Di_u_post*alpha_s_u_post);


"";
"Bayes Output";
"                  mean ~ std dev  ~ 2.5th Percentile ~ median ~ 97.5th Percentile ";
"phi_s_pre				";;SORT_OUT[1,1:5];		
"-phi_s2_pre			    ";;SORT_OUT[2,1:5];
"phi_i_pre				";;SORT_OUT[3,1:5];
"-phi_i2_pre			    ";;SORT_OUT[4,1:5];
"sig_eta_pre			    ";;SORT_OUT[5,1:5];
"sig_eps_pre			    ";;SORT_OUT[6,1:5];
"sig_v_pre				";;SORT_OUT[7,1:5];
"sig_u_pre				";;SORT_OUT[8,1:5];
"mu_k_pre				";;SORT_OUT[9,1:5];
"lambda_s_eta_pre		";;SORT_OUT[10,1:5];
"lambda_y_eta_pre		";;SORT_OUT[11,1:5];
"lambda_y_eps_pre		";;SORT_OUT[12,1:5];        
"lambda_i_v_pre			";;SORT_OUT[13,1:5];
"lambda_k_eta_pre			";;SORT_OUT[14,1:5];        
"lambda_k_eps_pre			";;SORT_OUT[29,1:5]; 
"phi_s_post				";;SORT_OUT[15,1:5];		
"-phi_s2_post			";;SORT_OUT[16,1:5];
"phi_i_post				";;SORT_OUT[17,1:5];
"-phi_i2_post			";;SORT_OUT[18,1:5];
"sig_eta_post			";;SORT_OUT[19,1:5];
"sig_eps_post			";;SORT_OUT[20,1:5];
"sig_v_post				";;SORT_OUT[21,1:5];
"sig_u_post				";;SORT_OUT[22,1:5];
"mu_k_post				";;SORT_OUT[23,1:5];
"lambda_s_eta_post		";;SORT_OUT[24,1:5];
"lambda_y_eta_post		";;SORT_OUT[25,1:5];
"lambda_y_eps_post		";;SORT_OUT[26,1:5];        
"lambda_i_v_post		";;SORT_OUT[27,1:5];   
"lambda_k_eta_post			";;SORT_OUT[28,1:5];        
"lambda_k_eps_post			";;SORT_OUT[30,1:5]; 

"sig_dy_pre				";;SORT_OUT[31,1:5];
"sig_dy_post			    ";;SORT_OUT[32,1:5];
"sig_ds_pre				";;SORT_OUT[33,1:5];
"sig_ds_post			    ";;SORT_OUT[34,1:5];
"sig_ddi_pre			    ";;SORT_OUT[35,1:5];
"sig_ddi_post			";;SORT_OUT[36,1:5];
"sig_di_u_pre			";;SORT_OUT[37,1:5];
"sig_di_u_post			";;SORT_OUT[38,1:5];
"sig_di_p_pre			";;SORT_OUT[39,1:5];
"sig_di_p_post			";;SORT_OUT[40,1:5];


"sig_dy_cf5(inv.)	    ";;SORT_OUT[41,1:5];
"sig_dy_cf6(inv. shocks) ";;SORT_OUT[43,1:5];
"sig_dy_cf9(u shocks)	";;SORT_OUT[49,1:5];
"sig_dy_cf7(inv. prop)	";;SORT_OUT[45,1:5];
"sig_dy_cf8(sales)	    ";;SORT_OUT[47,1:5];
"sig_ds_cf5(inv.)	    ";;SORT_OUT[42,1:5];
"sig_ds_cf6(inv. shocks) ";;SORT_OUT[44,1:5];
"sig_ds_cf9(u shocks)	";;SORT_OUT[50,1:5];
"sig_ds_cf7(inv. prop)	";;SORT_OUT[46,1:5];
"sig_ds_cf8(sales)	    ";;SORT_OUT[48,1:5];


"dsig_dy_post			    ";;SORT_OUT[89,1:5];
"dsig_ds_post			    ";;SORT_OUT[90,1:5];
"dsig_dy_cf5(inv.)	    ";;SORT_OUT[51,1:5];
"dsig_dy_cf6(inv. shocks) ";;SORT_OUT[53,1:5];
"dsig_dy_cf9(u shocks)	";;SORT_OUT[59,1:5];
"dsig_dy_cf7(inv. prop)	";;SORT_OUT[55,1:5];
"dsig_dy_cf8(sales)	    ";;SORT_OUT[57,1:5];
"dsig_ds_cf5(inv.)	    ";;SORT_OUT[52,1:5];
"dsig_ds_cf6(inv. shocks) ";;SORT_OUT[54,1:5];
"dsig_ds_cf9(u shocks)	";;SORT_OUT[60,1:5];
"dsig_ds_cf7(inv. prop)	";;SORT_OUT[56,1:5];
"dsig_ds_cf8(sales)	    ";;SORT_OUT[58,1:5];



"ratio_eta_Di_u_pre		";;SORT_OUT[61,1:5];
"ratio_v_Di_u_pre		";;SORT_OUT[62,1:5];
"ratio_eps_Di_u_pre		";;SORT_OUT[63,1:5];
"ratio_u_Di_u_pre		";;SORT_OUT[64,1:5];
"alpha_y_eta_pre		    ";;SORT_OUT[65,1:5];
"alpha_y_v_pre			";;SORT_OUT[66,1:5];
"alpha_y_eps_pre		    ";;SORT_OUT[67,1:5];
"alpha_y_u_pre			";;SORT_OUT[68,1:5];
"alpha_s_eta_pre		    ";;SORT_OUT[70,1:5];
"alpha_s_v_pre			";;SORT_OUT[71,1:5];
"alpha_s_eps_pre		    ";;SORT_OUT[72,1:5];
"alpha_s_u_pre			";;SORT_OUT[73,1:5];

"ratio_eta_Di_u_post	    ";;SORT_OUT[75,1:5];
"ratio_v_Di_u_post		";;SORT_OUT[76,1:5];
"ratio_eps_Di_u_post	    ";;SORT_OUT[77,1:5];
"ratio_u_Di_u_post		";;SORT_OUT[78,1:5];
"alpha_y_eta_post		";;SORT_OUT[79,1:5];
"alpha_y_v_post			";;SORT_OUT[80,1:5];
"alpha_y_eps_post		";;SORT_OUT[81,1:5];
"alpha_y_u_post			";;SORT_OUT[82,1:5];
"alpha_s_eta_post		";;SORT_OUT[84,1:5];
"alpha_s_v_post			";;SORT_OUT[85,1:5];
"alpha_s_eps_post		";;SORT_OUT[86,1:5];
"alpha_s_u_post			";;SORT_OUT[87,1:5];
"alpha_y_pre			    ";;SORT_OUT[69,1:5];
"alpha_s_pre			    ";;SORT_OUT[74,1:5];
"alpha_y_post			";;SORT_OUT[83,1:5];
"alpha_s_post			";;SORT_OUT[88,1:5];


"";
"Correlations";
Sort_out[1:28,6:9];
"";
"accept rate = ";;100*sumc(accept)/((CAPN*blocks));;"%";
bbb=1;
do until bbb>blocks;
"accept[";;bbb;;"] rate = ";;100*accept[bbb]/(CAPN);;"%";
bbb=bbb+1;
endo;

"";




OUTPUT OFF;




{DATA,FORCST} = filter(prm_fnl_);
output file=output_mh/uc_si.dta reset;
output on;
data[.,1]~data[.,2]~data[.,4]~data[.,6]~data[.,7];
output off;

output file=output_mh/fe.dta reset;
output on;
forcst;
output off;


begwind;
window(3,3,0);
xlabel("quarter");

setwind(1);
xlabel("quarter");
title("sales");

xy(SEQA(1,1,ROWS(Y[start_date+start:T,1])),Y[start_date+start:T,1]);

nextwind;
xlabel("quarter");
title("inventories");

xy(SEQA(1,1,ROWS(Y[start_date+start:T,1])),Y[start_date+start:T,2]);

nextwind;
xlabel("quarter");
title("inventory-gamm*sales");

xy(SEQA(1,1,ROWS(Y[start_date+start:T,1])),Y[start_date+start:T,2]-gamma_itao_fnl*Y[start_date+start:T,1]);


nextwind;
xlabel("quarter");
title("Permanent Sales");
_pdate = "";

XY(SEQA(1,1,ROWS(DATA)),DATA[.,6]);

nextwind;
xlabel("quarter");
title("Permanent Inventories");
_pdate = "";

XY(SEQA(1,1,ROWS(DATA)),gamma_itao_fnl*DATA[.,6]+DATA[.,7]);

nextwind;
xlabel("quarter");
title("Permanent Inventory/Sales");
_pdate = "";

XY(SEQA(1,1,ROWS(DATA)),DATA[.,7]);


nextwind
xlabel("quarter");
title("Transitory Sales");
_pdate = "";

XY(SEQA(1,1,ROWS(DATA)),DATA[.,2]~zeros(rows(data),1));

nextwind
xlabel("quarter");
title("Transitory Inventories");
_pdate = "";

XY(SEQA(1,1,ROWS(DATA)),DATA[.,4]~zeros(rows(data),1));


endwind;

XY(SEQA(1,1,ROWS(DATA)),DATA[.,1]~zeros(rows(data),1));

end;

@ END OF MAIN PROGRAM @
@========================================================================@
@========================================================================@
PROC post_fcn(PRMTR1);

     local F,FSTAR,H,BETA_LL,P_LL,BETA_TL,P_TL,BETA_TT,P_TT,
    VECP_LL,vt,ft, VAL,J_ITER, prmtr,Q,QSTAR,mu,a,aaa,
    phi_s1,phi_s2,phi_i1,phi_i2,ddd_pre,ddd_post,ddd, sig_eta_s,sig_eta_i,
    sig_eps_s,sig_eps_i,sig_eta_si,sig_eps_si,lambda_s_eta,lambda_i,
    sig_eta,sig_v,sig_eps,sig_eta_v,lambda_i_eta,lambda_v,lambda_i_eps,lambda_i_v,
    phi_y1,phi_y2,lambda_y_eps,lambda_y_eta,invft,sig_u,
    beta_mat,fcst_mat,gamma_itao,lik_mat,prmtr1_pre,prmtr1_post,mu_k,
    prior_norm_pre,prior_norm_post,prior_lambda_pre,prior_lambda_post,
    prior_var,prior_var_pre,prior_var_post,post,likv,norm_prm,lambda_prm,lambda_k_eta,lambda_k_eps;




PRMTR1_pre=prmtr1[1:14];    @parameters before the break@
PRMTR1_post=prmtr1[15:28];  @start with the same parameters. Then alter for parameters that undergo a break@


     PRMTR=TRANS(PRMTR1_pre);



ddd_pre=meanc(y[start_date+1:break_date,1]-y[start_date:break_date-1,1]);
ddd_post=meanc(y[break_date+1:t,1]-y[break_date:t-1,1]);


ddd=ddd_pre;   @drift@


phi_s1=prmtr[1]+prmtr[2];   
phi_s2=-prmtr[2];            
phi_i1=prmtr[3]+prmtr[4];
phi_i2=-prmtr[4];

sig_eta=sqrt(prmtr[5]);
sig_eps=sqrt(prmtr[6]);
sig_v=  sqrt(prmtr[7]);
sig_u=  sqrt(prmtr[8]);

mu_k=prmtr[9];

gamma_itao=1;

lambda_s_eta=prmtr[10];          
lambda_y_eta=prmtr[11];
lambda_y_eps=prmtr[12];         
lambda_i_v=  prmtr[13];      
lambda_k_eta=prmtr[14];
lambda_k_eps=0;
lambda_i_eta=lambda_y_eta-lambda_s_eta - gamma_itao-lambda_k_eta;
lambda_i_eps=lambda_y_eps-1-lambda_k_eps;




F=      PHI_S1~PHI_S2~0~0~0~0|
        1~0~0~0~0~0|
        0~0~PHI_I1~PHI_I2~0~0|
        0~0~1~0~0~0|
        0~0~0~0~1~0|
        0~0~0~0~0~1;

FSTAR=  PHI_S1~PHI_S2~0~0|
        1~0~0~0|
        0~0~PHI_I1~PHI_I2|
        0~0~1~0;



MU=(0~0~0~0~ddd~mu_k)';

H=      1~0~0~0~1~0|
        0~0~1~0~gamma_itao~1;


Q=  lambda_s_eta^2*sig_eta^2+sig_eps^2~0~
            lambda_s_eta*lambda_i_eta*sig_eta^2+lambda_i_eps*sig_eps^2~0~
                lambda_s_eta*sig_eta^2~lambda_s_eta*lambda_k_eta*sig_eta^2+lambda_k_eps*sig_eps^2|
    0~0~0~0~0~0|
    lambda_s_eta*lambda_i_eta*sig_eta^2+lambda_i_eps*sig_eps^2~0~
            lambda_i_eta^2*sig_eta^2+lambda_i_v^2*sig_v^2+lambda_i_eps^2*sig_eps^2+sig_u^2~0~
                lambda_i_eta*sig_eta^2~lambda_i_v*sig_v^2+lambda_i_eta*lambda_k_eta*sig_eta^2+lambda_i_eps*lambda_k_eps*sig_eps^2|
    0~0~0~0~0~0|
    lambda_s_eta*sig_eta^2~0~
            lambda_i_eta*sig_eta^2~0~
                sig_eta^2~lambda_k_eta*sig_eta^2|
    lambda_s_eta*lambda_k_eta*sig_eta^2+lambda_k_eps*sig_eps^2~0~lambda_i_v*sig_v^2+lambda_i_eta*lambda_k_eta*sig_eta^2+lambda_i_eps*lambda_k_eps*sig_eps^2~0~lambda_k_eta*sig_eta^2~sig_v^2+lambda_k_eta^2*sig_eta^2+lambda_k_eps^2*sig_eps^2;

QSTAR=lambda_s_eta^2*sig_eta^2+sig_eps^2~0~
            lambda_s_eta*lambda_i_eta*sig_eta^2+lambda_i_eps*sig_eps^2~0|
    0~0~0~0|
    lambda_s_eta*lambda_i_eta*sig_eta^2+lambda_i_eps*sig_eps^2~0~
            lambda_i_eta^2*sig_eta^2+lambda_i_v^2*sig_v^2+lambda_i_eps^2*sig_eps^2+sig_u^2~0|
    0~0~0~0;



A=(0~0)';


        BETA_LL=(0~0~0~0~y[start_date,1]-ddd~y[start_date,2]-ddd-gamma_itao*(y[start_date,1]-ddd)-mu_k)';

    VECP_LL=INV(EYE(rows(QSTAR)^2) - FSTAR.*.FSTAR)*vec(QSTAR);  @vectorization of var-cov for stationary components@
        P_LL=reshape(VECP_LL,rows(QSTAR),rows(QSTAR));
        P_LL=(P_LL~zeros(rows(P_LL),2))|
            (zeros(1,rows(qstar))~prior~0)|
            (zeros(1,rows(qstar))~0~prior);


     LIK_MAT=ZEROS(T,1);

J_ITER = start_date;
do until J_ITER>T;


@============@
@CHANGE PARAMETERS AFTER BREAK@
if j_iter==break_date+1;    @only need to do this once after the break@


     PRMTR=TRANS(PRMTR1_post);

ddd=ddd_post;   @drift@


phi_s1=prmtr[1]+prmtr[2];   
phi_s2=-prmtr[2];            
phi_i1=prmtr[3]+prmtr[4];
phi_i2=-prmtr[4];

sig_eta=sqrt(prmtr[5]);
sig_eps=sqrt(prmtr[6]);
sig_v=  sqrt(prmtr[7]);
sig_u=  sqrt(prmtr[8]);

mu_k=prmtr[9];

gamma_itao=1;

lambda_s_eta=prmtr[10];          
lambda_y_eta=prmtr[11];
lambda_y_eps=prmtr[12];         
lambda_i_v=  prmtr[13];      
lambda_k_eta=prmtr[14];
lambda_k_eps=0;
lambda_i_eta=lambda_y_eta-lambda_s_eta - gamma_itao-lambda_k_eta;
lambda_i_eps=lambda_y_eps-1-lambda_k_eps;




F=      PHI_S1~PHI_S2~0~0~0~0|
        1~0~0~0~0~0|
        0~0~PHI_I1~PHI_I2~0~0|
        0~0~1~0~0~0|
        0~0~0~0~1~0|
        0~0~0~0~0~1;

FSTAR=  PHI_S1~PHI_S2~0~0|
        1~0~0~0|
        0~0~PHI_I1~PHI_I2|
        0~0~1~0;



MU=(0~0~0~0~ddd~mu_k)';

H=      1~0~0~0~1~0|
        0~0~1~0~gamma_itao~1;


Q=  lambda_s_eta^2*sig_eta^2+sig_eps^2~0~
            lambda_s_eta*lambda_i_eta*sig_eta^2+lambda_i_eps*sig_eps^2~0~
                lambda_s_eta*sig_eta^2~lambda_s_eta*lambda_k_eta*sig_eta^2+lambda_k_eps*sig_eps^2|
    0~0~0~0~0~0|
    lambda_s_eta*lambda_i_eta*sig_eta^2+lambda_i_eps*sig_eps^2~0~
            lambda_i_eta^2*sig_eta^2+lambda_i_v^2*sig_v^2+lambda_i_eps^2*sig_eps^2+sig_u^2~0~
                lambda_i_eta*sig_eta^2~lambda_i_v*sig_v^2+lambda_i_eta*lambda_k_eta*sig_eta^2+lambda_i_eps*lambda_k_eps*sig_eps^2|
    0~0~0~0~0~0|
    lambda_s_eta*sig_eta^2~0~
            lambda_i_eta*sig_eta^2~0~
                sig_eta^2~lambda_k_eta*sig_eta^2|
    lambda_s_eta*lambda_k_eta*sig_eta^2+lambda_k_eps*sig_eps^2~0~lambda_i_v*sig_v^2+lambda_i_eta*lambda_k_eta*sig_eta^2+lambda_i_eps*lambda_k_eps*sig_eps^2~0~lambda_k_eta*sig_eta^2~sig_v^2+lambda_k_eta^2*sig_eta^2+lambda_k_eps^2*sig_eps^2;

QSTAR=lambda_s_eta^2*sig_eta^2+sig_eps^2~0~
            lambda_s_eta*lambda_i_eta*sig_eta^2+lambda_i_eps*sig_eps^2~0|
    0~0~0~0|
    lambda_s_eta*lambda_i_eta*sig_eta^2+lambda_i_eps*sig_eps^2~0~
            lambda_i_eta^2*sig_eta^2+lambda_i_v^2*sig_v^2+lambda_i_eps^2*sig_eps^2+sig_u^2~0|
    0~0~0~0;


A=(0~0)';


endif;
@============@


      BETA_TL = MU + F * BETA_LL ;
      P_TL = F * P_LL * F' + Q ;

      vt=y[j_iter,1:2]' - H * BETA_TL - A; @prediction error@

      ft= H * P_TL * H' ;       @variance of forecast error@ 
      invft=inv(ft);

      BETA_TT= BETA_TL + P_TL * H' * invft * vt;
      P_TT= P_TL - P_TL * H' * invft * H * P_TL;

      LIK_MAT[J_ITER,1] =
        0.5*LN(((2*pi)^2)*det(ft)) +0.5*vt'*inv(ft)*vt;

      BETA_LL=BETA_TT;
      P_LL=P_TT;


J_ITER = J_ITER+1;
ENDO;

VAL = SUMC(LIK_MAT[START_DATE+START:T]);

likv=-val;


 


@Evaluate prior@
PRMTR=TRANS(PRMTR1_pre);

norm_prm=prmtr[1:4]|prmtr[9];
PRIOR_norm_pre=LN(NORM_D(norm_prm,b0_m,b0_v));

lambda_prm=prmtr[10:14]; 

if beta_prior==0; PRIOR_lambda_pre=LN(NORM_D(lambda_prm,b0_ml,b0_vl));
elseif beta_prior==1;
PRIOR_lambda_pre=LN(BETADNST(-lambda_prm[1],U1_01_,U1_00_)); 
PRIOR_lambda_post=PRIOR_lambda_post+LN(BETADNST(0.5+lambda_prm[2]/2,U1_01_,U1_00_));  
PRIOR_lambda_pre=PRIOR_lambda_pre+LN(BETADNST(lambda_prm[3],U1_01_,U1_00_)); 
PRIOR_lambda_pre=PRIOR_lambda_pre+LN(BETADNST(-lambda_prm[4],U1_01_,U1_00_));
PRIOR_lambda_pre=PRIOR_lambda_pre+LN(BETADNST(0.5+lambda_prm[5]/2,U1_01_,U1_00_));
endif;
@NOTE: -1<lambda_s_eta<0, -1<lambda_y_eta<1, 0<lamdbda_y_eps<1, -1<lambda_i_v<0@

if var_prior==0;
PRIOR_VAR=LN(gamma_d(1/prmtr[5],V0_,D0_));
PRIOR_VAR=PRIOR_VAR+LN(gamma_d(1/prmtr[6],V0_,D0_));
PRIOR_VAR=PRIOR_VAR+LN(gamma_d(1/prmtr[7],V0_,D0_));
PRIOR_VAR=PRIOR_VAR+LN(gamma_d(1/prmtr[8],V0_,D0_));
elseif var_prior==1;
PRIOR_VAR=LN(gamma_d(prmtr[5],V0_,D0_));
PRIOR_VAR=PRIOR_VAR+LN(gamma_d(prmtr[6],V0_,D0_));
PRIOR_VAR=PRIOR_VAR+LN(gamma_d(prmtr[7],V0_,D0_));
PRIOR_VAR=PRIOR_VAR+LN(gamma_d(prmtr[8],V0_,D0_));
endif;
PRIOR_VAR_pre=PRIOR_VAR;


PRMTR=TRANS(PRMTR1_post);

norm_prm=prmtr[1:4]|prmtr[9];
PRIOR_norm_post=LN(NORM_D(norm_prm,b0_m,b0_v));

lambda_prm=prmtr[10:14];	@includes correlations@

if beta_prior==0; PRIOR_lambda_post=LN(NORM_D(lambda_prm,b0_ml,b0_vl));
elseif beta_prior==1;
PRIOR_lambda_post=LN(BETADNST(-lambda_prm[1],U1_01_,U1_00_)); 
PRIOR_lambda_post=PRIOR_lambda_post+LN(BETADNST(0.5+lambda_prm[2]/2,U1_01_,U1_00_));  
PRIOR_lambda_post=PRIOR_lambda_post+LN(BETADNST(lambda_prm[3],U1_01_,U1_00_)); 
PRIOR_lambda_post=PRIOR_lambda_post+LN(BETADNST(-lambda_prm[4],U1_01_,U1_00_));
PRIOR_lambda_post=PRIOR_lambda_post+LN(BETADNST(0.5+lambda_prm[5]/2,U1_01_,U1_00_));
endif;
@NOTE: -1<lambda_s_eta<0, -1<lambda_y_eta<1, 0<lamdbda_y_eps<1, -1<lambda_i_v<0@

if var_prior==0;
PRIOR_VAR=LN(gamma_d(1/prmtr[5],V0_,D0_));
PRIOR_VAR=PRIOR_VAR+LN(gamma_d(1/prmtr[6],V0_,D0_));
PRIOR_VAR=PRIOR_VAR+LN(gamma_d(1/prmtr[7],V0_,D0_));
PRIOR_VAR=PRIOR_VAR+LN(gamma_d(1/prmtr[8],V0_,D0_));
elseif var_prior==1;
PRIOR_VAR=LN(gamma_d(prmtr[5],V0_,D0_));
PRIOR_VAR=PRIOR_VAR+LN(gamma_d(prmtr[6],V0_,D0_));
PRIOR_VAR=PRIOR_VAR+LN(gamma_d(prmtr[7],V0_,D0_));
PRIOR_VAR=PRIOR_VAR+LN(gamma_d(prmtr[8],V0_,D0_));
endif;
PRIOR_VAR_post=PRIOR_VAR;


post=likv + (prior_norm_pre + prior_lambda_pre + prior_var_pre) + (prior_norm_post + prior_lambda_post + prior_var_post); 


retp(-real(post));
endp;


@========================================================================@
proc post_fcn_fnl(PRMTR1);

     local F,FSTAR,H,BETA_LL,P_LL,BETA_TL,P_TL,BETA_TT,P_TT,
    VECP_LL,vt,ft, VAL,J_ITER, prmtr,Q,QSTAR,mu,a,aaa,
    phi_s1,phi_s2,phi_i1,phi_i2,ddd_pre,ddd_post,ddd, sig_eta_s,sig_eta_i,
    sig_eps_s,sig_eps_i,sig_eta_si,sig_eps_si,lambda_s_eta,lambda_i,
    sig_eta,sig_v,sig_eps,sig_eta_v,lambda_i_eta,lambda_v,lambda_i_eps,lambda_i_v,
    phi_y1,phi_y2,lambda_y_eps,lambda_y_eta,invft,sig_u,
    beta_mat,fcst_mat,gamma_itao,lik_mat,prmtr1_pre,prmtr1_post,mu_k,
    prior_norm_pre,prior_norm_post,prior_lambda_pre,prior_lambda_post,
    prior_var,prior_var_pre,prior_var_post,post,likv,norm_prm,lambda_prm,lambda_k_eta,lambda_k_eps;




PRMTR1_pre=prmtr1[1:14];    @parameters before the break@
PRMTR1_post=prmtr1[15:28];  @start with the same parameters. Then alter for parameters that undergo a break@


     PRMTR=(PRMTR1_pre);



ddd_pre=meanc(y[start_date+1:break_date,1]-y[start_date:break_date-1,1]);
ddd_post=meanc(y[break_date+1:t,1]-y[break_date:t-1,1]);


ddd=ddd_pre;   @drift@


phi_s1=prmtr[1]+prmtr[2];   
phi_s2=-prmtr[2];            
phi_i1=prmtr[3]+prmtr[4];
phi_i2=-prmtr[4];

sig_eta=sqrt(prmtr[5]);
sig_eps=sqrt(prmtr[6]);
sig_v=  sqrt(prmtr[7]);
sig_u=  sqrt(prmtr[8]);

mu_k=prmtr[9];

gamma_itao=1;

lambda_s_eta=prmtr[10];          
lambda_y_eta=prmtr[11];
lambda_y_eps=prmtr[12];         
lambda_i_v=  prmtr[13];      
lambda_k_eta=prmtr[14];
lambda_k_eps=0;
lambda_i_eta=lambda_y_eta-lambda_s_eta - gamma_itao-lambda_k_eta;
lambda_i_eps=lambda_y_eps-1-lambda_k_eps;




F=      PHI_S1~PHI_S2~0~0~0~0|
        1~0~0~0~0~0|
        0~0~PHI_I1~PHI_I2~0~0|
        0~0~1~0~0~0|
        0~0~0~0~1~0|
        0~0~0~0~0~1;

FSTAR=  PHI_S1~PHI_S2~0~0|
        1~0~0~0|
        0~0~PHI_I1~PHI_I2|
        0~0~1~0;



MU=(0~0~0~0~ddd~mu_k)';

H=      1~0~0~0~1~0|
        0~0~1~0~gamma_itao~1;


Q=  lambda_s_eta^2*sig_eta^2+sig_eps^2~0~
            lambda_s_eta*lambda_i_eta*sig_eta^2+lambda_i_eps*sig_eps^2~0~
                lambda_s_eta*sig_eta^2~lambda_s_eta*lambda_k_eta*sig_eta^2+lambda_k_eps*sig_eps^2|
    0~0~0~0~0~0|
    lambda_s_eta*lambda_i_eta*sig_eta^2+lambda_i_eps*sig_eps^2~0~
            lambda_i_eta^2*sig_eta^2+lambda_i_v^2*sig_v^2+lambda_i_eps^2*sig_eps^2+sig_u^2~0~
                lambda_i_eta*sig_eta^2~lambda_i_v*sig_v^2+lambda_i_eta*lambda_k_eta*sig_eta^2+lambda_i_eps*lambda_k_eps*sig_eps^2|
    0~0~0~0~0~0|
    lambda_s_eta*sig_eta^2~0~
            lambda_i_eta*sig_eta^2~0~
                sig_eta^2~lambda_k_eta*sig_eta^2|
    lambda_s_eta*lambda_k_eta*sig_eta^2+lambda_k_eps*sig_eps^2~0~lambda_i_v*sig_v^2+lambda_i_eta*lambda_k_eta*sig_eta^2+lambda_i_eps*lambda_k_eps*sig_eps^2~0~lambda_k_eta*sig_eta^2~sig_v^2+lambda_k_eta^2*sig_eta^2+lambda_k_eps^2*sig_eps^2;

QSTAR=lambda_s_eta^2*sig_eta^2+sig_eps^2~0~
            lambda_s_eta*lambda_i_eta*sig_eta^2+lambda_i_eps*sig_eps^2~0|
    0~0~0~0|
    lambda_s_eta*lambda_i_eta*sig_eta^2+lambda_i_eps*sig_eps^2~0~
            lambda_i_eta^2*sig_eta^2+lambda_i_v^2*sig_v^2+lambda_i_eps^2*sig_eps^2+sig_u^2~0|
    0~0~0~0;



A=(0~0)';


        BETA_LL=(0~0~0~0~y[start_date,1]-ddd~y[start_date,2]-ddd-gamma_itao*(y[start_date,1]-ddd)-mu_k)';

    VECP_LL=INV(EYE(rows(QSTAR)^2) - FSTAR.*.FSTAR)*vec(QSTAR);  @vectorization of var-cov for stationary components@
        P_LL=reshape(VECP_LL,rows(QSTAR),rows(QSTAR));
        P_LL=(P_LL~zeros(rows(P_LL),2))|
            (zeros(1,rows(qstar))~prior~0)|
            (zeros(1,rows(qstar))~0~prior);


     LIK_MAT=ZEROS(T,1);

J_ITER = start_date;
do until J_ITER>T;


@============@
@CHANGE PARAMETERS AFTER BREAK@
if j_iter==break_date+1;    @only need to do this once after the break@


     PRMTR=(PRMTR1_post);

ddd=ddd_post;   @drift@


phi_s1=prmtr[1]+prmtr[2];   
phi_s2=-prmtr[2];            
phi_i1=prmtr[3]+prmtr[4];
phi_i2=-prmtr[4];

sig_eta=sqrt(prmtr[5]);
sig_eps=sqrt(prmtr[6]);
sig_v=  sqrt(prmtr[7]);
sig_u=  sqrt(prmtr[8]);

mu_k=prmtr[9];

gamma_itao=1;

lambda_s_eta=prmtr[10];          
lambda_y_eta=prmtr[11];
lambda_y_eps=prmtr[12];         
lambda_i_v=  prmtr[13];      
lambda_k_eta=prmtr[14];
lambda_k_eps=0;
lambda_i_eta=lambda_y_eta-lambda_s_eta - gamma_itao-lambda_k_eta;
lambda_i_eps=lambda_y_eps-1-lambda_k_eps;




F=      PHI_S1~PHI_S2~0~0~0~0|
        1~0~0~0~0~0|
        0~0~PHI_I1~PHI_I2~0~0|
        0~0~1~0~0~0|
        0~0~0~0~1~0|
        0~0~0~0~0~1;

FSTAR=  PHI_S1~PHI_S2~0~0|
        1~0~0~0|
        0~0~PHI_I1~PHI_I2|
        0~0~1~0;



MU=(0~0~0~0~ddd~mu_k)';

H=      1~0~0~0~1~0|
        0~0~1~0~gamma_itao~1;


Q=  lambda_s_eta^2*sig_eta^2+sig_eps^2~0~
            lambda_s_eta*lambda_i_eta*sig_eta^2+lambda_i_eps*sig_eps^2~0~
                lambda_s_eta*sig_eta^2~lambda_s_eta*lambda_k_eta*sig_eta^2+lambda_k_eps*sig_eps^2|
    0~0~0~0~0~0|
    lambda_s_eta*lambda_i_eta*sig_eta^2+lambda_i_eps*sig_eps^2~0~
            lambda_i_eta^2*sig_eta^2+lambda_i_v^2*sig_v^2+lambda_i_eps^2*sig_eps^2+sig_u^2~0~
                lambda_i_eta*sig_eta^2~lambda_i_v*sig_v^2+lambda_i_eta*lambda_k_eta*sig_eta^2+lambda_i_eps*lambda_k_eps*sig_eps^2|
    0~0~0~0~0~0|
    lambda_s_eta*sig_eta^2~0~
            lambda_i_eta*sig_eta^2~0~
                sig_eta^2~lambda_k_eta*sig_eta^2|
    lambda_s_eta*lambda_k_eta*sig_eta^2+lambda_k_eps*sig_eps^2~0~lambda_i_v*sig_v^2+lambda_i_eta*lambda_k_eta*sig_eta^2+lambda_i_eps*lambda_k_eps*sig_eps^2~0~lambda_k_eta*sig_eta^2~sig_v^2+lambda_k_eta^2*sig_eta^2+lambda_k_eps^2*sig_eps^2;

QSTAR=lambda_s_eta^2*sig_eta^2+sig_eps^2~0~
            lambda_s_eta*lambda_i_eta*sig_eta^2+lambda_i_eps*sig_eps^2~0|
    0~0~0~0|
    lambda_s_eta*lambda_i_eta*sig_eta^2+lambda_i_eps*sig_eps^2~0~
            lambda_i_eta^2*sig_eta^2+lambda_i_v^2*sig_v^2+lambda_i_eps^2*sig_eps^2+sig_u^2~0|
    0~0~0~0;



A=(0~0)';


endif;
@============@


      BETA_TL = MU + F * BETA_LL ;
      P_TL = F * P_LL * F' + Q ;

      vt=y[j_iter,1:2]' - H * BETA_TL - A; @prediction error@

      ft= H * P_TL * H' ;       @variance of forecast error@
      invft=inv(ft);

      BETA_TT= BETA_TL + P_TL * H' * invft * vt;
      P_TT= P_TL - P_TL * H' * invft * H * P_TL;

      LIK_MAT[J_ITER,1] =
        0.5*LN(((2*pi)^2)*det(ft)) +0.5*vt'*inv(ft)*vt;

      BETA_LL=BETA_TT;
      P_LL=P_TT;


J_ITER = J_ITER+1;
endo;

VAL = SUMC(LIK_MAT[START_DATE+START:T]);

likv=-val;


 


@Evaluate prior@
PRMTR=(PRMTR1_pre);

norm_prm=prmtr[1:4]|prmtr[9];
PRIOR_norm_pre=LN(NORM_D(norm_prm,b0_m,b0_v));

lambda_prm=prmtr[10:14];	@includes correlations@

if beta_prior==0; PRIOR_lambda_pre=LN(NORM_D(lambda_prm,b0_ml,b0_vl));
elseif beta_prior==1;
PRIOR_lambda_pre=LN(BETADNST(-lambda_prm[1],U1_01_,U1_00_)); 
PRIOR_lambda_post=PRIOR_lambda_post+LN(BETADNST(0.5+lambda_prm[2]/2,U1_01_,U1_00_));  
PRIOR_lambda_pre=PRIOR_lambda_pre+LN(BETADNST(lambda_prm[3],U1_01_,U1_00_)); 
PRIOR_lambda_pre=PRIOR_lambda_pre+LN(BETADNST(-lambda_prm[4],U1_01_,U1_00_));
PRIOR_lambda_pre=PRIOR_lambda_pre+LN(BETADNST(0.5+lambda_prm[5]/2,U1_01_,U1_00_));
endif;
@NOTE: -1<lambda_s_eta<0, -1<lambda_y_eta<1, 0<lamdbda_y_eps<1, -1<lambda_i_v<0@

if var_prior==0;
PRIOR_VAR=LN(gamma_d(1/prmtr[5],V0_,D0_));
PRIOR_VAR=PRIOR_VAR+LN(gamma_d(1/prmtr[6],V0_,D0_));
PRIOR_VAR=PRIOR_VAR+LN(gamma_d(1/prmtr[7],V0_,D0_));
PRIOR_VAR=PRIOR_VAR+LN(gamma_d(1/prmtr[8],V0_,D0_));
elseif var_prior==1;
PRIOR_VAR=LN(gamma_d(prmtr[5],V0_,D0_));
PRIOR_VAR=PRIOR_VAR+LN(gamma_d(prmtr[6],V0_,D0_));
PRIOR_VAR=PRIOR_VAR+LN(gamma_d(prmtr[7],V0_,D0_));
PRIOR_VAR=PRIOR_VAR+LN(gamma_d(prmtr[8],V0_,D0_));
endif;
PRIOR_VAR_pre=PRIOR_VAR;


PRMTR=(PRMTR1_post);

norm_prm=prmtr[1:4]|prmtr[9];
PRIOR_norm_post=LN(NORM_D(norm_prm,b0_m,b0_v));

lambda_prm=prmtr[10:14];	@includes correlations@

if beta_prior==0; PRIOR_lambda_post=LN(NORM_D(lambda_prm,b0_ml,b0_vl));
elseif beta_prior==1;
PRIOR_lambda_post=LN(BETADNST(-lambda_prm[1],U1_01_,U1_00_)); 
PRIOR_lambda_post=PRIOR_lambda_post+LN(BETADNST(0.5+lambda_prm[2]/2,U1_01_,U1_00_));  
PRIOR_lambda_post=PRIOR_lambda_post+LN(BETADNST(lambda_prm[3],U1_01_,U1_00_)); 
PRIOR_lambda_post=PRIOR_lambda_post+LN(BETADNST(-lambda_prm[4],U1_01_,U1_00_));
PRIOR_lambda_post=PRIOR_lambda_post+LN(BETADNST(0.5+lambda_prm[5]/2,U1_01_,U1_00_));
endif;
@NOTE: -1<lambda_s_eta<0, -1<lambda_y_eta<1, 0<lamdbda_y_eps<1, -1<lambda_i_v<0@

if var_prior==0;
PRIOR_VAR=LN(gamma_d(1/prmtr[5],V0_,D0_));
PRIOR_VAR=PRIOR_VAR+LN(gamma_d(1/prmtr[6],V0_,D0_));
PRIOR_VAR=PRIOR_VAR+LN(gamma_d(1/prmtr[7],V0_,D0_));
PRIOR_VAR=PRIOR_VAR+LN(gamma_d(1/prmtr[8],V0_,D0_));
elseif var_prior==1;
PRIOR_VAR=LN(gamma_d(prmtr[5],V0_,D0_));
PRIOR_VAR=PRIOR_VAR+LN(gamma_d(prmtr[6],V0_,D0_));
PRIOR_VAR=PRIOR_VAR+LN(gamma_d(prmtr[7],V0_,D0_));
PRIOR_VAR=PRIOR_VAR+LN(gamma_d(prmtr[8],V0_,D0_));
endif;
PRIOR_VAR_post=PRIOR_VAR;

post=likv + (prior_norm_pre + prior_lambda_pre + prior_var_pre) + (prior_norm_post + prior_lambda_post + prior_var_post); 



post=post+0.75*fout; @re-adjusts scale to make exp(post) computable@



retp(exp(post));

endp;






@>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>@

PROC (2)=filter(PRMTR1);

     local F,FSTAR,H,BETA_LL,P_LL,BETA_TL,P_TL,BETA_TT,P_TT,
    VECP_LL,vt,ft, VAL,J_ITER, prmtr,Q,QSTAR,mu,a,aaa,
    phi_s1,phi_s2,phi_i1,phi_i2,ddd_pre,ddd_post,ddd, sig_eta_s,sig_eta_i,
    sig_eps_s,sig_eps_i,sig_eta_si,sig_eps_si,lambda_s_eta,lambda_i,
    sig_eta,sig_v,sig_eps,sig_eta_v,lambda_i_eta,lambda_v,lambda_i_eps,lambda_i_v,
    phi_y1,phi_y2,lambda_y_eps,lambda_y_eta,invft,sig_u,
    beta_mat,fcst_mat,gamma_itao,lik_mat,prmtr1_pre,prmtr1_post,mu_k,
    prior_norm_pre,prior_norm_post,prior_lambda_pre,prior_lambda_post,
    prior_var,prior_var_pre,prior_var_post,post,likv,norm_prm,lambda_prm,lambda_k_eta,lambda_k_eps;




PRMTR1_pre=prmtr1[1:14];    @parameters before the break@
PRMTR1_post=prmtr1[15:28];  @start with the same parameters. Then alter for parameters that undergo a break@


     PRMTR=@TRANS@(PRMTR1_pre);



ddd_pre=meanc(y[start_date+1:break_date,1]-y[start_date:break_date-1,1]);
ddd_post=meanc(y[break_date+1:t,1]-y[break_date:t-1,1]);


ddd=ddd_pre;   @drift@


phi_s1=prmtr[1]+prmtr[2];   
phi_s2=-prmtr[2];            
phi_i1=prmtr[3]+prmtr[4];
phi_i2=-prmtr[4];

sig_eta=sqrt(prmtr[5]);
sig_eps=sqrt(prmtr[6]);
sig_v=  sqrt(prmtr[7]);
sig_u=  sqrt(prmtr[8]);

mu_k=prmtr[9];

gamma_itao=1;

lambda_s_eta=prmtr[10];          
lambda_y_eta=prmtr[11];
lambda_y_eps=prmtr[12];         
lambda_i_v=  prmtr[13];      
lambda_k_eta=prmtr[14];
lambda_k_eps=0;
lambda_i_eta=lambda_y_eta-lambda_s_eta - gamma_itao-lambda_k_eta;
lambda_i_eps=lambda_y_eps-1-lambda_k_eps;




F=     0~0~0~0~0~0~0| 
	   0~PHI_S1~PHI_S2~0~0~0~0|
        0~1~0~0~0~0~0|
        0~0~0~PHI_I1~PHI_I2~0~0|
        0~0~0~1~0~0~0|
        0~0~0~0~0~1~0|
        0~0~0~0~0~0~1;

FSTAR=  0~0~0~0~0|
		0~PHI_S1~PHI_S2~0~0|
        0~1~0~0~0|
        0~0~0~PHI_I1~PHI_I2|
        0~0~0~1~0;



MU=(0~0~0~0~0~ddd~mu_k)';

H=      0~1~0~0~0~1~0|
        0~0~0~1~0~gamma_itao~1;



Q=  sig_u^2~0~0~sig_u^2~0~0~0|

	0~lambda_s_eta^2*sig_eta^2+sig_eps^2~0~
            lambda_s_eta*lambda_i_eta*sig_eta^2+lambda_i_eps*sig_eps^2~0~
                lambda_s_eta*sig_eta^2~lambda_s_eta*lambda_k_eta*sig_eta^2+lambda_k_eps*sig_eps^2|
    0~0~0~0~0~0~0|
    sig_u^2~lambda_s_eta*lambda_i_eta*sig_eta^2+lambda_i_eps*sig_eps^2~0~
            lambda_i_eta^2*sig_eta^2+lambda_i_v^2*sig_v^2+lambda_i_eps^2*sig_eps^2+sig_u^2~0~
                lambda_i_eta*sig_eta^2~lambda_i_v*sig_v^2+lambda_i_eta*lambda_k_eta*sig_eta^2+lambda_i_eps*lambda_k_eps*sig_eps^2|
    0~0~0~0~0~0~0|
    0~lambda_s_eta*sig_eta^2~0~
            lambda_i_eta*sig_eta^2~0~
                sig_eta^2~lambda_k_eta*sig_eta^2|
    lambda_s_eta*lambda_k_eta*sig_eta^2+lambda_k_eps*sig_eps^2~0~0~lambda_i_v*sig_v^2+lambda_i_eta*lambda_k_eta*sig_eta^2+lambda_i_eps*lambda_k_eps*sig_eps^2~0~lambda_k_eta*sig_eta^2~sig_v^2;

QSTAR=sig_u^2~0~0~sig_u^2~0|

	0~lambda_s_eta^2*sig_eta^2+sig_eps^2~0~
            lambda_s_eta*lambda_i_eta*sig_eta^2+lambda_i_eps*sig_eps^2~0|
    0~0~0~0~0|
    sig_u^2~lambda_s_eta*lambda_i_eta*sig_eta^2+lambda_i_eps*sig_eps^2~0~
            lambda_i_eta^2*sig_eta^2+lambda_i_v^2*sig_v^2+lambda_i_eps^2*sig_eps^2+sig_u^2~0|
    0~0~0~0~0;




A=(0~0)';


        BETA_LL=(0~0~0~0~0~y[start_date,1]-ddd~y[start_date,2]-ddd-gamma_itao*(y[start_date,1]-ddd)-mu_k)';

    VECP_LL=INV(EYE(rows(QSTAR)^2) - FSTAR.*.FSTAR)*vec(QSTAR);  @vectorization of var-cov for stationary components@
        P_LL=reshape(VECP_LL,rows(QSTAR),rows(QSTAR));
        P_LL=(P_LL~zeros(rows(P_LL),2))|
            (zeros(1,rows(qstar))~prior~0)|
            (zeros(1,rows(qstar))~0~prior);



      BETA_MAT=ZEROS(T,7);
      FCST_MAT=ZEROS(T,2);


J_ITER = start_date;
do until J_ITER>T;


@============@
@CHANGE PARAMETERS AFTER BREAK@
if j_iter==break_date+1;    @only need to do this once after the break@


     PRMTR=@TRANS@(PRMTR1_post);

ddd=ddd_post;   @drift@


phi_s1=prmtr[1]+prmtr[2];   
phi_s2=-prmtr[2];            
phi_i1=prmtr[3]+prmtr[4];
phi_i2=-prmtr[4];

sig_eta=sqrt(prmtr[5]);
sig_eps=sqrt(prmtr[6]);
sig_v=  sqrt(prmtr[7]);
sig_u=  sqrt(prmtr[8]);

mu_k=prmtr[9];

gamma_itao=1;

lambda_s_eta=prmtr[10];          
lambda_y_eta=prmtr[11];
lambda_y_eps=prmtr[12];         
lambda_i_v=  prmtr[13];      
lambda_k_eta=prmtr[14];
lambda_k_eps=0;
lambda_i_eta=lambda_y_eta-lambda_s_eta - gamma_itao-lambda_k_eta;
lambda_i_eps=lambda_y_eps-1-lambda_k_eps;




F=     0~0~0~0~0~0~0| 
	   0~PHI_S1~PHI_S2~0~0~0~0|
        0~1~0~0~0~0~0|
        0~0~0~PHI_I1~PHI_I2~0~0|
        0~0~0~1~0~0~0|
        0~0~0~0~0~1~0|
        0~0~0~0~0~0~1;

FSTAR=  0~0~0~0~0|
		0~PHI_S1~PHI_S2~0~0|
        0~1~0~0~0|
        0~0~0~PHI_I1~PHI_I2|
        0~0~0~1~0;



MU=(0~0~0~0~0~ddd~mu_k)';

H=      0~1~0~0~0~1~0|
        0~0~0~1~0~gamma_itao~1;


Q=  sig_u^2~0~0~sig_u^2~0~0~0|

	0~lambda_s_eta^2*sig_eta^2+sig_eps^2~0~
            lambda_s_eta*lambda_i_eta*sig_eta^2+lambda_i_eps*sig_eps^2~0~
                lambda_s_eta*sig_eta^2~lambda_s_eta*lambda_k_eta*sig_eta^2+lambda_k_eps*sig_eps^2|
    0~0~0~0~0~0~0|
    sig_u^2~lambda_s_eta*lambda_i_eta*sig_eta^2+lambda_i_eps*sig_eps^2~0~
            lambda_i_eta^2*sig_eta^2+lambda_i_v^2*sig_v^2+lambda_i_eps^2*sig_eps^2+sig_u^2~0~
                lambda_i_eta*sig_eta^2~lambda_i_v*sig_v^2+lambda_i_eta*lambda_k_eta*sig_eta^2+lambda_i_eps*lambda_k_eps*sig_eps^2|
    0~0~0~0~0~0~0|
    0~lambda_s_eta*sig_eta^2~0~
            lambda_i_eta*sig_eta^2~0~
                sig_eta^2~lambda_k_eta*sig_eta^2|
    lambda_s_eta*lambda_k_eta*sig_eta^2+lambda_k_eps*sig_eps^2~0~0~lambda_i_v*sig_v^2+lambda_i_eta*lambda_k_eta*sig_eta^2+lambda_i_eps*lambda_k_eps*sig_eps^2~0~lambda_k_eta*sig_eta^2~sig_v^2;

QSTAR=sig_u^2~0~0~sig_u^2~0|

	0~lambda_s_eta^2*sig_eta^2+sig_eps^2~0~
            lambda_s_eta*lambda_i_eta*sig_eta^2+lambda_i_eps*sig_eps^2~0|
    0~0~0~0~0|
    sig_u^2~lambda_s_eta*lambda_i_eta*sig_eta^2+lambda_i_eps*sig_eps^2~0~
            lambda_i_eta^2*sig_eta^2+lambda_i_v^2*sig_v^2+lambda_i_eps^2*sig_eps^2+sig_u^2~0|
    0~0~0~0~0;


A=(0~0)';


endif;
@============@


      BETA_TL = MU + F * BETA_LL ;
      P_TL = F * P_LL * F' + Q ;

      vt=y[j_iter,1:2]' - H * BETA_TL - A; @prediction error@

      ft= H * P_TL * H' ;       @variance of forecast error@
      invft=inv(ft);

      BETA_TT= BETA_TL + P_TL * H' * invft * vt;
      P_TT= P_TL - P_TL * H' * invft * H * P_TL;



      BETA_LL=BETA_TT;
      P_LL=P_TT;


    FCST_MAT[J_ITER,.]=vt';
    BETA_MAT[J_ITER,.]=BETA_TT';


J_ITER = J_ITER+1;
ENDO;


retp(BETA_MAT[START_DATE+START:T,.],FCST_MAT[START_DATE+START:T,.]);
endp;



@================================================================@
PROC trans(c0); @ constraining values of reg. coeff.@
         local c1,aaa,ccc,ppp,varcov;

         c1=c0;

        aaa=c0[1]./(1+abs(c0[1]));
        ccc=(1-abs(aaa))*c0[2]./(1+abs(c0[2]))+abs(aaa)-aaa^2;

c1[1]=2*aaa-(aaa^2+ccc);
c1[2]=(aaa^2+ccc);

        aaa=c0[3]./(1+abs(c0[3]));
        ccc=(1-abs(aaa))*c0[4]./(1+abs(c0[4]))+abs(aaa)-aaa^2;

c1[3]=2*aaa-(aaa^2+ccc);
c1[4]=(aaa^2+ccc);

c1[5:8]=exp(c0[5:8]);
c1[10]=-exp(c0[10])/(1+exp(c0[10])); 
c1[11]=-1+2*exp(c0[11])/(1+exp(c0[11]));
c1[12]=exp(c0[12])/(1+exp(c0[12])); 
c1[13]=-exp(c0[13])./(1+exp(c0[13]));
c1[14]=-1+2*exp(c0[14])./(1+exp(c0[14]));
		 

retp(c1);
endp;


@================================================================@
proc trans_fnl(c0); @ constraining values of reg. coeff.@
         local c1,aaa,ccc,ppp,varcov;


         c1=c0;

        aaa=c0[1]./(1+abs(c0[1]));
        ccc=(1-abs(aaa))*c0[2]./(1+abs(c0[2]))+abs(aaa)-aaa^2;

c1[1]=2*aaa-(aaa^2+ccc);
c1[2]=(aaa^2+ccc);

        aaa=c0[3]./(1+abs(c0[3]));
        ccc=(1-abs(aaa))*c0[4]./(1+abs(c0[4]))+abs(aaa)-aaa^2;

c1[3]=2*aaa-(aaa^2+ccc);
c1[4]=(aaa^2+ccc);

c1[5:8]=exp(c0[5:8]);
c1[10]=-exp(c0[10])/(1+exp(c0[10])); 
c1[11]=-1+2*exp(c0[11])/(1+exp(c0[11]));
c1[12]=exp(c0[12])/(1+exp(c0[12])); 
c1[13]=-exp(c0[13])./(1+exp(c0[13]));
c1[14]=-1+2*exp(c0[14])./(1+exp(c0[14]));



@THIS NEEDS TO BE MODIFIED if ONLY A SUBSET OF PARAMETERS CHANGE@

        aaa=c0[15]./(1+abs(c0[15]));
        ccc=(1-abs(aaa))*c0[16]./(1+abs(c0[16]))+abs(aaa)-aaa^2;

c1[15]=2*aaa-(aaa^2+ccc);
c1[16]=(aaa^2+ccc);

        aaa=c0[17]./(1+abs(c0[17]));
        ccc=(1-abs(aaa))*c0[18]./(1+abs(c0[18]))+abs(aaa)-aaa^2;

c1[17]=2*aaa-(aaa^2+ccc);
c1[18]=(aaa^2+ccc);

c1[19:22]=exp(c0[19:22]);
c1[24]=-exp(c0[24])/(1+exp(c0[24])); 
c1[25]=-1+2*exp(c0[25])/(1+exp(c0[25]));
c1[26]=exp(c0[26])/(1+exp(c0[26])); 
c1[27]=-exp(c0[27])./(1+exp(c0[27])); 
c1[28]=-1+2*exp(c0[28])./(1+exp(c0[28]));


retp(c1);
endp;


@==================================================================================@
@Procedures to allow draws from Chisquare (and, more generally, Gamma) distribution@

proc rndc(aa);       /* chisquare */
local x,w;

aa = aa/2;
w= rg1(aa);
x = w * 2;

retp(x);
endp;


proc rg1(aa);
local x,w,u;

if aa gt 1;
x = rg2(aa);
elseif aa lt 1;
aa = aa + 1;
u = rndu(1,1);
x = rg2(aa)*u^(1/aa);
elseif aa == 1;
x = -ln(rndu(1,1));
endif;

retp(x);
endp;

proc rg2(aa);
local gam,accept,b,c,x,z,u,v,w,y;

b = aa-1;
c = 3*aa - .75;
accept = 0;
do while accept == 0;
u = rndu(1,1);
v = rndu(1,1);
w = u*(1-u);
y = sqrt(c/w)*(u-.5);
x = b+y;
if x ge 0;
z = 64*(w^3)*(v^2);
accept = z le ( 1-(2*y^2)/x );
if accept == 0;
accept = ln(z) le 2*(b*ln(x/b) - y);
endif;
endif;
endo;

retp(x);
endp;


@========================================================================@

proc STUDT_D(MUFNL0,MU0,V0,NU0);
local VAL,lnnu,lnnup;

if (nu0/2)<=1;
lnnu=ln(gamma(nu0/2));
else;
lnnu=lnfact((nu0/2)-1);
endif;

if ((nu0+1)/2)<=1;
lnnup=ln(gamma((nu0+1)/2));
else;
lnnup=lnfact(((nu0+1)/2)-1);
endif;

@Scalar version@
VAL= lnnup - (1/2)*ln(v0*nu0*pi) - lnnu 
    - ((nu0+1)/2)*ln(1+(1/nu0)*(MUFNL0-MU0).*(MUFNL0-MU0)./v0);

retp(exp(VAL));
endp;

@==================================================================@
@========================================================================@

proc STUDT_D_MV(MUFNL0,MU0,V0,NU0);
local VAL,lnnu,lnnup;

if (nu0/2)<=1;
lnnu=ln(gamma(nu0/2));
else;
lnnu=lnfact((nu0/2)-1);
endif;

if ((nu0+rows(mu0))/2)<=1;
lnnup=ln(gamma((nu0+rows(mu0))/2));
else;
lnnup=lnfact(((nu0+rows(mu0))/2)-1);
endif;

VAL= lnnup - (1/2)*ln(det(v0)) - (rows(v0)/2)*ln(nu0*pi) - lnnu 
    - ((nu0+rows(v0))/2)*ln(1+(1/nu0)*(MUFNL0-MU0)'INV(V0)*(MUFNL0-MU0));

retp(exp(VAL));
endp;

@===================================================================@

proc INVGAMMA(SS2, NU_,DLTA_);
local VAL, lngam_p;

if (nu_/2)<=1;
  lngam_p = ln(gamma(nu_/2));
else;
  lngam_p = lnfact(nu_/2-1);
endif;

   VAL = 0.5*NU_*LN(DLTA_/2) - lngam_p
       -(NU_/2 + 1)*LN(SS2) - 0.5*(DLTA_/SS2);

retp(exp(VAL));
endp;

@==================================================================@

proc GAMMA_D(SS2, NU_,DLTA_);
local VAL, lngam_p;

if (nu_/2)<=1;
  lngam_p = ln(gamma(nu_/2));
else;
  lngam_p = lnfact(nu_/2-1);
endif;

   VAL = 0.5*NU_*LN(DLTA_/2) - lngam_p
       +(NU_/2 - 1)*LN(SS2) - 0.5*(DLTA_*SS2);

retp(exp(VAL));
endp;


@========================================================================@

proc NORM_D(MUFNL0,MU0,V0);
local VAL;

VAL= -(ROWS(V0)/2)*LN(2*PI)-(1/2)*LN(DET(V0))  -0.5*(MUFNL0-MU0)'INV(V0)*(MUFNL0-MU0);

retp(exp(VAL));
endp;

@========================================================================@

@==================================================================@

proc BETADNST(PPT,ALPHA_,BETA_);
local VAL,lnbeta_a,lnbeta_b,lnbeta_ab;

if alpha_<=1;
lnbeta_a=ln(gamma(alpha_));
else;
lnbeta_a=lnfact(alpha_-1);
endif;

if beta_<=1;
lnbeta_b=ln(gamma(beta_));
else;
lnbeta_b=lnfact(beta_-1);
endif;

if (alpha_+beta_)<=1;
lnbeta_ab=ln(gamma(alpha_+beta_));
else;
lnbeta_ab=lnfact(alpha_+beta_-1);
endif;

val=lnbeta_ab - lnbeta_a - lnbeta_b + (alpha_-1)*ln(ppt) + (beta_-1)*ln(1-ppt);

retp(exp(VAL));
endp;

@========================================================================@


@========================================================@
@      A PROCEDURE THAT GENERATES  theta based on independence kernel@
@========================================================@
proc (3) = GEN_Theta_mb;

     local theta,u_1,theta_star,q_theta,q_theta_star,post_theta,post_theta_star,
        alpha_theta_star,g_1,here_ia,f_,theta_star_nz, ar_eig,
    theta_,prm_p_,cov_p_,theta_star_,k__,theta_ret;


Theta=thetatt;
Theta_=thetatt[bs:be];
prm_p_=prm_p[bs:be];
cov_p_=cov_p[bs:be,bs:be];
k__=k[bbb];


u_1=rndu(1,1);

g_1=rndc(t_nu)/t_nu;

if a_==0; @IA@
theta_star_=prm_p_ + chol((1/g_1)*k__*cov_p_)'rndn(rows(prm_p_),1); @draw parameters from candidate density@
elseif a_==1; @RW@
theta_star_=theta_ + chol((1/g_1)*k__*cov_p_)'rndn(rows(prm_p_),1); @draw parameters from candidate density@
endif;
theta_star=thetatt; theta_star[bs:be]=theta_star_;



q_theta=studt_d_mv(theta_,prm_p_,k__*cov_p_,t_nu); @independence kernel@
q_theta_star=studt_d_mv(theta_star_,prm_p_,k__*cov_p_,t_nu); @independence kernel@


post_theta=post_fcn_fnl(theta);




ar_eig=0;

f_=(sumc(theta_star[1:2])~-theta_star[2])|(1~0);
ar_eig=maxc(abs(eig(f_)));
f_=(sumc(theta_star[3:4])~-theta_star[4])|(1~0);
ar_eig=maxc(ar_eig|abs(eig(f_)));
f_=(sumc(theta_star[15:16])~-theta_star[16])|(1~0);
ar_eig=maxc(ar_eig|abs(eig(f_)));
f_=(sumc(theta_star[17:18])~-theta_star[18])|(1~0);
ar_eig=maxc(ar_eig|abs(eig(f_)));

/*
theta_star_nz=theta_star[5:8]|theta_star[19:22]|
-theta_star[10]|theta_star[10]+1|
-theta_star[24]|theta_star[24]+1|
0.5-theta_star[11]/2|theta_star[11]/2+0.5|
0.5-theta_star[23]/2|theta_star[23]/2+0.5|
-minc(-1|theta_star[14])+theta_star[13]|-theta_star[13]+maxc(1|theta_star[14])|
-minc(-1|theta_star[28])+theta_star[27]|-theta_star[27]+maxc(1|theta_star[28])|
theta_star[12]|-theta_star[12]+1|
theta_star[26]|-theta_star[26]+1; 

*/

theta_star_nz=theta_star[5:8]|theta_star[19:22]|
-theta_star[10]|theta_star[10]+1|
-theta_star[24]|theta_star[24]+1|
-minc(-1|theta_star[14])+theta_star[11]|-theta_star[11]+maxc(0|1+theta_star[14])|
-minc(-1|theta_star[28])+theta_star[25]|-theta_star[25]+maxc(0|1+theta_star[28])|
theta_star[12]|-theta_star[12]+1|
theta_star[26]|-theta_star[26]+1|
-theta_star[13]|theta_star[13]+1|
-theta_star[27]|theta_star[27]+1;


if minc(theta_star_nz[1:8]) le 0 or minc(theta_star_nz[9:24]) < 0 or ar_eig ge 1;
    counter_p=counter_p+1;
    post_theta_star=0;
else;
    post_theta_star=post_fcn_fnl(theta_star);
endif;

if a_==0; @IA@
alpha_theta_star=(post_theta_star*q_theta)/(post_theta*q_theta_star);
elseif a_==1;
alpha_theta_star=post_theta_star/post_theta;
endif;



if u_1<alpha_theta_star;
    theta=theta_star;

    post_theta=post_theta_star; @necessary for ML calculation@
    q_theta=q_theta_star; @same as above@

accept[bbb]=accept[bbb]+1;
endif;

theta_ret=theta[bs:be];


retp(theta_ret,post_theta,q_theta);
endp;


